// EFTSRecGen_ImgProcTool_Cpp.cpp : main project file.

#include "stdafx.h"

#include "Imageprocessing.h"
#include <an2k_dll.h>

using namespace System;
using namespace System::IO;

// This example performs a complete image processing
// by using Green Bit tools.
// Images have already been acquired and are saved into
// the "Samples" directory. For their name convention
// see the ImageProcessing.cpp file.
// In this example the missing of the left ring finger
// will be simulated, in order to show how to use
// the segmentation procedure both in full-automatic
// and in semi-automatic modes.
// Acquired objects are:
//
// -	left 4 fingers slap (3 fingers actually because
//		left ring finger missing is simulated)
// -	right 4 fingers slap
// -	2 thumbs slap
// -	10 rolled fingerprints (9 fingers actually because
//		left ring finger missing is simulated)
//
// The following operations will be performed on the
// fingerprint images:
//
// -	slaps segmentation
// -	quality evaluation (by using NFIQ algorithm)
// -	quality evaluation (by using proprietary GB algorithm) and
//		calculation of validity and completeness indexes
// -	seqeunce check
//
// At the end of the acquisition, all the images will
// be stored into buffer formatted in compliance with the
// ANSI/NIST ITL-1 2007 standard containing:
//
// -	3 type-14 records for the slaps with segmentation
//		and quality info. Images are compressed by using
//		the WSQ algorithm
// -	9 type-4 records for the rolled fingerprints, if
//		rolling is supported. Images are compressed by
//		using the JPEG2000 algorithm
// -	1 type-2 record to store the unavailability
//		reason of the left ring finger
//
// The ANSI-NIST buffer will be saved into a binary file
// (EFTS_Record.bin) and this file will be parsed by using
// the "an2ktool.exe" tool, provided by the ANSI-NIST.
// The output of this file will be sent to the standard
// output and can be analized by the user.

int main(array<System::String ^> ^args)
{
	Int32	RetVal;
	RECORD	*RecToBeAdded = nullptr;

    // set console max number of rows
	Console::BufferHeight = 1000;
	Console::BufferWidth = 100;

	Console::WriteLine ("\t\tEFTSRecGen_ImgProcTool\n");

	/* Init global variables */
	RetVal = InitGlobals();
	if (RetVal != GB_EXAMPLE_NO_ERROR) {
		Console::WriteLine ("Error: InitGlobals returned: " + RetVal);
		Console::Read();
		Console::Read();
		return 1;
	}

	/* Create ANSI_NIST structure */
	Console::WriteLine ("Create ANSI NIST structure...");
	ANSI_NIST *ansi_nist;
	RetVal = Create_ANSI_NIST(&ansi_nist,VERSION_0400,"ImgProcTools_Ex", 0, "Test Destination Agency",
		"Green Bit", "000001",nullptr,
		(float)19.69, (float)19.69, nullptr);
	if (RetVal != AN2K_DLL_NO_ERROR) {
		Console::WriteLine ("Error: Create_ANSI_NIST returned: " + RetVal);
		Console::Read();
		Console::Read();
		return 1;
	}
	Console::WriteLine ("Ansi_NIST structure succesfully created...");

	Console::WriteLine ("");
	Console::WriteLine ("");
	

	/*************/
	// LEFT slap: a finger is missing
	/*************/
	Console::WriteLine("*************************************");
	Console::WriteLine("Processing LEFT SLAP: ring is missing");
	Console::WriteLine("*************************************");
	// process image: get segmentation, calculate minutiae for sequence check and create record 14
	RetVal = ProcessSlap(LEFT_SLAP,&RecToBeAdded);
	
	if (RetVal != GB_EXAMPLE_NO_ERROR)
	{
		Console::WriteLine("Errors in processing Left Slap: " + RetVal);
		Console::Read();
		return 1;
	}
	// add record to ansi nist file
	if (RecToBeAdded != nullptr)
	{
		Console::WriteLine ("");
		Console::WriteLine("Inserting record into ANSI NIST file...");
		RetVal = insert_ANSI_NIST_record_ETFS(ansi_nist->num_records,RecToBeAdded,ansi_nist);
		if (RetVal != GB_EXAMPLE_NO_ERROR)
		{
			Console::WriteLine("Errors in processing Left Slap record 14: " + RetVal);
			Console::Read();
			return 1;
		}
		else
		{
			Console::WriteLine("Left Slap record 14 stored");
		}
	}
	else
	{
		Console::WriteLine("Cannot add Left Slap rec 14 to ANSI/NIST file");
		Console::Read();
		return 1;
	}

	Console::WriteLine ("");
	Console::WriteLine ("");
	

	/*************/
	// RIGHT slap
	/*************/
	// init Record
	RecToBeAdded = nullptr;
	Console::WriteLine("*************************************");
	Console::WriteLine("       Processing RIGHT SLAP");
	Console::WriteLine("*************************************");
	// process image: get segmentation, calculate minutiae for sequence check and create record 14
	RetVal = ProcessSlap(RIGHT_SLAP,&RecToBeAdded);
	if (RetVal != GB_EXAMPLE_NO_ERROR)
	{
		Console::WriteLine("Errors in processing Right Slap: " + RetVal);
		Console::Read();
		return 1;
	}
	// add record to ansi nist file
	if (RecToBeAdded != nullptr)
	{
		Console::WriteLine ("");
		Console::WriteLine("Inserting record into ANSI NIST file...");
		RetVal = insert_ANSI_NIST_record_ETFS(ansi_nist->num_records,RecToBeAdded,ansi_nist);
		if (RetVal != GB_EXAMPLE_NO_ERROR)
		{
			Console::WriteLine("Errors in processing Right Slap record 14: " + RetVal);
			Console::Read();
			return 1;
		}
		else
		{
			Console::WriteLine("Right Slap record 14 stored");
		}
	}
	else
	{
		Console::WriteLine("Cannot add Right Slap rec 14 to ANSI/NIST file");
		Console::Read();
		return 1;
	}

	Console::WriteLine ("");
	Console::WriteLine ("");

	

	/*************/
	// 2 Thumbs
	/*************/
	// init Record
	RecToBeAdded = nullptr;
	Console::WriteLine("*************************************");
	Console::WriteLine("         Processing 2 THUMBS");
	Console::WriteLine("*************************************");
	// process image: get segmentation, calculate minutiae for sequence check and create record 14
	RetVal = ProcessSlap(TWO_THUMBS,&RecToBeAdded);
	if (RetVal != GB_EXAMPLE_NO_ERROR)
	{
		Console::WriteLine("Errors in processing 2 Thumbs: " + RetVal);
		Console::Read();
		return 1;
	}
	// add record to ansi nist file
	if (RecToBeAdded != nullptr)
	{
		Console::WriteLine ("");
		Console::WriteLine("Inserting record into ANSI NIST file...");
		RetVal = insert_ANSI_NIST_record_ETFS(ansi_nist->num_records,RecToBeAdded,ansi_nist);
		if (RetVal != GB_EXAMPLE_NO_ERROR)
		{
			Console::WriteLine("Errors in processing 2 Thumbs record 14: " + RetVal);
			Console::Read();
			return 1;
		}
		else
		{
			Console::WriteLine("2 Thumbs record 14 stored...");
		}
	}
	else
	{
		Console::WriteLine("Cannot add 2 Thumbs rec 14 to ANSI/NIST file...");
		Console::Read();
		return 1;
	}

	Console::WriteLine ("");
	Console::WriteLine ("");

	/********************/
	// insert record 2
	/********************/
	Console::WriteLine("*************************************");
	Console::WriteLine("Creating Record 2 for amputation list");
	Console::WriteLine("*************************************");
	RecToBeAdded = nullptr;
	RetVal = CreateRec2FromRollAmpList(ansi_nist);
	if (RetVal != GB_EXAMPLE_NO_ERROR)
	{
		Console::WriteLine("Errors for Record 2...");
		return 1;
	}
	else
	{
		Console::WriteLine("Record 2 succesfully inserted into ansi nist file...");
	}

	Console::WriteLine ("");
	Console::WriteLine ("");

	/********************/
	// Rolled Left Thumb
	/********************/
	// init Record
	RecToBeAdded = nullptr;
	Console::WriteLine("*************************************");
	Console::WriteLine("     Processing Roll Left Thumb");
	Console::WriteLine("*************************************");
	// process image: calculate NIST quality, calculate minutiae, perform sequence check and create record 4
	RetVal = ProcessRoll(LEFT_THUMB,&RecToBeAdded);
	
	if (RetVal != GB_EXAMPLE_NO_ERROR)
	{
		Console::WriteLine("Errors in processing Roll Left Thumb: " + RetVal);
		Console::Read();
		return 1;
	}
	// add record to ansi nist file
	if (RecToBeAdded != nullptr)
	{
		Console::WriteLine ("");
		Console::WriteLine("Inserting record into ANSI NIST file...");
		RetVal = insert_ANSI_NIST_record_ETFS(ansi_nist->num_records,RecToBeAdded,ansi_nist);
		if (RetVal != GB_EXAMPLE_NO_ERROR)
		{
			Console::WriteLine("Errors in processing Roll Left Thumb record 4: " + RetVal);
			Console::Read();
			return 1;
		}
		else
		{
			Console::WriteLine("Roll Left Thumb record 4 stored");
		}
	}
	else
	{
		Console::WriteLine("Cannot add Roll Left Thumb rec 4 to ANSI/NIST file");
		Console::Read();
		return 1;
	}

	Console::WriteLine ("");
	Console::WriteLine ("");

	/********************/
	// Rolled Left Index
	/********************/
	// init Record
	RecToBeAdded = nullptr;
	Console::WriteLine("*************************************");
	Console::WriteLine("     Processing Roll Left Index");
	Console::WriteLine("*************************************");
	// process image: calculate NIST quality, calculate minutiae, perform sequence check and create record 4
	RetVal = ProcessRoll(LEFT_INDEX,&RecToBeAdded);
	if (RetVal != GB_EXAMPLE_NO_ERROR)
	{
		Console::WriteLine("Errors in processing Roll Left Index: " + RetVal);
		Console::Read();
		return 1;
	}
	// add record to ansi nist file
	if (RecToBeAdded != nullptr)
	{
		Console::WriteLine ("");
		Console::WriteLine("Inserting record into ANSI NIST file...");
		RetVal = insert_ANSI_NIST_record_ETFS(ansi_nist->num_records,RecToBeAdded,ansi_nist);
		if (RetVal != GB_EXAMPLE_NO_ERROR)
		{
			Console::WriteLine("Errors in processing Roll Left Index record 4: " + RetVal);
			Console::Read();
			return 1;
		}
		else
		{
			Console::WriteLine("Roll Left Index record 4 stored");
		}
	}
	else
	{
		Console::WriteLine("Cannot add Roll Left Index rec 4 to ANSI/NIST file");
		Console::Read();
		return 1;
	}

	Console::WriteLine ("");
	Console::WriteLine ("");

	

	/********************/
	// Rolled Left Middle
	/********************/
	// init Record
	RecToBeAdded = nullptr;
	Console::WriteLine("*************************************");
	Console::WriteLine("     Processing Roll Left Middle");
	Console::WriteLine("*************************************");
	// process image: calculate NIST quality, calculate minutiae, perform sequence check and create record 4
	RetVal = ProcessRoll(LEFT_MIDDLE,&RecToBeAdded);
	if (RetVal != GB_EXAMPLE_NO_ERROR)
	{
		Console::WriteLine("Errors in processing Roll Left Middle: " + RetVal);
		Console::Read();
		return 1;
	}
	// add record to ansi nist file
	if (RecToBeAdded != nullptr)
	{
		Console::WriteLine ("");
		Console::WriteLine("Inserting record into ANSI NIST file...");
		RetVal = insert_ANSI_NIST_record_ETFS(ansi_nist->num_records,RecToBeAdded,ansi_nist);
		if (RetVal != GB_EXAMPLE_NO_ERROR)
		{
			Console::WriteLine("Errors in processing Roll Left Middle record 4: " + RetVal);
			Console::Read();
			return 1;
		}
		else
		{
			Console::WriteLine("Roll Left Middle record 4 stored");
		}
	}
	else
	{
		Console::WriteLine("Cannot add Roll Left Middle rec 4 to ANSI/NIST file");
		Console::Read();
		return 1;
	}

	Console::WriteLine ("");
	Console::WriteLine ("");

	/*******************************/
	// Rolled Left ring missing....
	/*******************************/

	Console::WriteLine("*************************************");
	Console::WriteLine("      LEFT RING IS MISSING");
	Console::WriteLine("*************************************");

	Console::WriteLine ("");
	Console::WriteLine ("");
	

	/********************/
	// Rolled Left Little
	/********************/
	// init Record
	RecToBeAdded = nullptr;
	Console::WriteLine("*************************************");
	Console::WriteLine("     Processing Roll Left Little");
	Console::WriteLine("*************************************");
	// process image: calculate NIST quality, calculate minutiae, perform sequence check and create record 4
	RetVal = ProcessRoll(LEFT_LITTLE,&RecToBeAdded);
	if (RetVal != GB_EXAMPLE_NO_ERROR)
	{
		Console::WriteLine("Errors in processing Roll Left Little: " + RetVal);
		Console::Read();
		return 1;
	}
	// add record to ansi nist file
	if (RecToBeAdded != nullptr)
	{
		Console::WriteLine ("");
		Console::WriteLine("Inserting record into ANSI NIST file...");
		RetVal = insert_ANSI_NIST_record_ETFS(ansi_nist->num_records,RecToBeAdded,ansi_nist);
		if (RetVal != GB_EXAMPLE_NO_ERROR)
		{
			Console::WriteLine("Errors in processing Roll Left Little record 4: " + RetVal);
			Console::Read();
			return 1;
		}
		else
		{
			Console::WriteLine("Roll Left Little record 4 stored");
		}
	}
	else
	{
		Console::WriteLine("Cannot add Roll Left Little rec 4 to ANSI/NIST file");
		Console::Read();
		return 1;
	}

	Console::WriteLine ("");
	Console::WriteLine ("");

	/********************/
	// Rolled Right Thumb
	/********************/
	// init Record
	RecToBeAdded = nullptr;
	Console::WriteLine("*************************************");
	Console::WriteLine("     Processing Roll Right Thumb");
	Console::WriteLine("*************************************");
	// process image: calculate NIST quality, calculate minutiae, perform sequence check and create record 4
	RetVal = ProcessRoll(RIGHT_THUMB,&RecToBeAdded);
	if (RetVal != GB_EXAMPLE_NO_ERROR)
	{
		Console::WriteLine("Errors in processing Roll Right Thumb: " + RetVal);
		Console::Read();
		return 1;
	}
	// add record to ansi nist file
	if (RecToBeAdded != nullptr)
	{
		Console::WriteLine ("");
		Console::WriteLine("Inserting record into ANSI NIST file...");
		RetVal = insert_ANSI_NIST_record_ETFS(ansi_nist->num_records,RecToBeAdded,ansi_nist);
		if (RetVal != GB_EXAMPLE_NO_ERROR)
		{
			Console::WriteLine("Errors in processing Roll Right Thumb record 4: " + RetVal);
			Console::Read();
			return 1;
		}
		else
		{
			Console::WriteLine("Roll Right Thumb record 4 stored");
		}
	}
	else
	{
		Console::WriteLine("Cannot add Roll Right Thumb rec 4 to ANSI/NIST file");
		Console::Read();
		return 1;
	}

	Console::WriteLine ("");
	Console::WriteLine ("");

	/********************/
	// Rolled Right Index
	/********************/
	// init Record
	RecToBeAdded = nullptr;
	Console::WriteLine("*************************************");
	Console::WriteLine("     Processing Roll Right Index");
	Console::WriteLine("*************************************");
	// process image: calculate NIST quality, calculate minutiae, perform sequence check and create record 4
	RetVal = ProcessRoll(RIGHT_INDEX,&RecToBeAdded);
	if (RetVal != GB_EXAMPLE_NO_ERROR)
	{
		Console::WriteLine("Errors in processing Roll Right Index: " + RetVal);
		Console::Read();
		return 1;
	}
	// add record to ansi nist file
	if (RecToBeAdded != nullptr)
	{
		Console::WriteLine ("");
		Console::WriteLine("Inserting record into ANSI NIST file...");
		RetVal = insert_ANSI_NIST_record_ETFS(ansi_nist->num_records,RecToBeAdded,ansi_nist);
		if (RetVal != GB_EXAMPLE_NO_ERROR)
		{
			Console::WriteLine("Errors in processing Roll Right Index record 4: " + RetVal);
			Console::Read();
			return 1;
		}
		else
		{
			Console::WriteLine("Roll Right Index record 4 stored");
		}
	}
	else
	{
		Console::WriteLine("Cannot add Roll Right Index rec 4 to ANSI/NIST file");
		Console::Read();
		return 1;
	}

	Console::WriteLine ("");
	Console::WriteLine ("");

	/********************/
	// Rolled Right Middle
	/********************/
	// init Record
	RecToBeAdded = nullptr;
	Console::WriteLine("*************************************");
	Console::WriteLine("     Processing Roll Right Middle");
	Console::WriteLine("*************************************");
	// process image: calculate NIST quality, calculate minutiae, perform sequence check and create record 4
	RetVal = ProcessRoll(RIGHT_MIDDLE,&RecToBeAdded);
	if (RetVal != GB_EXAMPLE_NO_ERROR)
	{
		Console::WriteLine("Errors in processing Roll Right Middle: " + RetVal);
		Console::Read();
		return 1;
	}
	// add record to ansi nist file
	if (RecToBeAdded != nullptr)
	{
		Console::WriteLine ("");
		Console::WriteLine("Inserting record into ANSI NIST file...");
		RetVal = insert_ANSI_NIST_record_ETFS(ansi_nist->num_records,RecToBeAdded,ansi_nist);
		if (RetVal != GB_EXAMPLE_NO_ERROR)
		{
			Console::WriteLine("Errors in processing Roll Right Middle record 4: " + RetVal);
			Console::Read();
			return 1;
		}
		else
		{
			Console::WriteLine("Roll Right Middle record 4 stored");
		}
	}
	else
	{
		Console::WriteLine("Cannot add Roll Right Middle rec 4 to ANSI/NIST file");
		Console::Read();
		return 1;
	}

	Console::WriteLine ("");
	Console::WriteLine ("");

	/********************/
	// Rolled Right Ring
	/********************/
	// init Record
	RecToBeAdded = nullptr;
	Console::WriteLine("*************************************");
	Console::WriteLine("     Processing Roll Right Ring");
	Console::WriteLine("*************************************");
	// process image: calculate NIST quality, calculate minutiae, perform sequence check and create record 4
	RetVal = ProcessRoll(RIGHT_RING,&RecToBeAdded);
	if (RetVal != GB_EXAMPLE_NO_ERROR)
	{
		Console::WriteLine("Errors in processing Roll Right Ring: " + RetVal);
		Console::Read();
		return 1;
	}
	// add record to ansi nist file
	if (RecToBeAdded != nullptr)
	{
		Console::WriteLine ("");
		Console::WriteLine("Inserting record into ANSI NIST file...");
		RetVal = insert_ANSI_NIST_record_ETFS(ansi_nist->num_records,RecToBeAdded,ansi_nist);
		if (RetVal != GB_EXAMPLE_NO_ERROR)
		{
			Console::WriteLine("Errors in processing Roll Right Ring record 4: " + RetVal);
			Console::Read();
			return 1;
		}
		else
		{
			Console::WriteLine("Roll Right Ring record 4 stored");
		}
	}
	else
	{
		Console::WriteLine("Cannot add Roll Right Ring rec 4 to ANSI/NIST file");
		Console::Read();
		return 1;
	}

	Console::WriteLine ("");
	Console::WriteLine ("");

	/********************/
	// Rolled Right Little
	/********************/
	// init Record
	RecToBeAdded = nullptr;
	Console::WriteLine("*************************************");
	Console::WriteLine("     Processing Roll Right Little");
	Console::WriteLine("*************************************");
	// process image: calculate NIST quality, calculate minutiae, perform sequence check and create record 4
	RetVal = ProcessRoll(RIGHT_LITTLE,&RecToBeAdded);
	if (RetVal != GB_EXAMPLE_NO_ERROR)
	{
		Console::WriteLine("Errors in processing Roll Right Little: " + RetVal);
		Console::Read();
		return 1;
	}
	// add record to ansi nist file
	if (RecToBeAdded != nullptr)
	{
		Console::WriteLine ("");
		Console::WriteLine("Inserting record into ANSI NIST file...");
		RetVal = insert_ANSI_NIST_record_ETFS(ansi_nist->num_records,RecToBeAdded,ansi_nist);
		if (RetVal != GB_EXAMPLE_NO_ERROR)
		{
			Console::WriteLine("Errors in processing Roll Right Little record 4: " + RetVal);
			Console::Read();
			return 1;
		}
		else
		{
			Console::WriteLine("Roll Right Little record 4 stored");
		}
	}
	else
	{
		Console::WriteLine("Cannot add Roll Right Little rec 4 to ANSI/NIST file");
		Console::Read();
		return 1;
	}

	Console::WriteLine ("");
	Console::WriteLine ("");

	
	/*******************************************/
	/* SAVE AND SHOW RESULTS                   */
	/*******************************************/
	/* create buffer from ansi_nist */
	Byte *ANSI_NIST_buffer;
	Int32 BufLen;
	RetVal = write_ANSI_NIST_2_buffer(&ANSI_NIST_buffer, &BufLen, ansi_nist);

    // save the ANSI_NIST buffer into a file
	array<Byte,1> ^ANSI_NIST_Array = gcnew array<Byte,1>(BufLen);
	for (Int32 i=0;i<BufLen;i++)
	{
		ANSI_NIST_Array[i] = ANSI_NIST_buffer[i];
	}
	File::WriteAllBytes("EFTS_Record.bin",ANSI_NIST_Array);

	// call the "an2ktool.exe" tool into a separate process...
	System::Diagnostics::Process ^an2ktool = gcnew System::Diagnostics::Process();

	an2ktool->StartInfo->FileName = ".\\an2ktool.exe";
	an2ktool->StartInfo->Arguments = "-print all EFTS_Record.bin";
	an2ktool->StartInfo->UseShellExecute = false;
	an2ktool->StartInfo->RedirectStandardOutput = true;

	an2ktool->Start();
	
	// ... and send its output to the Console window
	StreamReader^ an2ktool_StreamReader = an2ktool->StandardOutput;
	String^ an2ktool_outString = an2ktool_StreamReader->ReadToEnd();
	Console::WriteLine( an2ktool_outString );
	File::WriteAllText("ansi_nist.txt",an2ktool_outString);
	an2ktool->Close();

	// keep the console window opened until a character is entered...
	Console::WriteLine("End of operation: press ENTER to exit");
	Console::Read();
	Console::Read();
    return 0;
}
