#ifndef IMAGEPROCESSING_H
#define IMAGEPROCESSING_H

// To avoid compilation errors, redefine the
// followings: FAR, PASCAL and BYTE
// before including GBFINIMG.h
#ifndef FAR
#define GB_FAR_DEFINED
#define FAR
#define PASCAL __stdcall
#define BYTE unsigned char
#endif	FAR
#include <GBFINIMG.h>
#ifdef GB_FAR_DEFINED // if are defined externally keep them defined
#undef FAR
#undef PASCAL
#undef BYTE
#endif GB_FAR_DEFINED

#include <an2k_dll.h>
#include <lfs_dll.h>

using namespace System;

// error codes
#define GB_EXAMPLE_NO_ERROR						0
#define GB_EXAMPLE_MEM_ERROR					1
#define GB_EXAMPLE_SYS_ERROR					2
#define GB_EXAMPLE_PARAM_ERROR					3
#define GB_EXAMPLE_GENER_ERROR					4
#define GB_EXAMPLE_UNACCEPTABLE_SEGM_RES_ERROR	5
#define GB_EXAMPLE_ACCEPTABLE_SEGM_RES_ERROR	6

// Fingerprint objects identifiers
#define LEFT_SLAP		0
#define RIGHT_SLAP		1
#define TWO_THUMBS		2
#define LEFT_THUMB		3
#define LEFT_INDEX		4
#define LEFT_MIDDLE		5
#define LEFT_RING		6
#define LEFT_LITTLE		7
#define RIGHT_THUMB		8
#define RIGHT_INDEX		9
#define RIGHT_MIDDLE	10
#define RIGHT_RING		11
#define RIGHT_LITTLE	12

// Fingerprints file name
#define LEFT_SLAP_F_NAME	"Samples\\LeftSlap.bmp"
#define RIGHT_SLAP_F_NAME	"Samples\\RightSlap.bmp"
#define TWO_THUMBS_F_NAME	"Samples\\2Thumbs.bmp"
#define LEFT_THUMB_F_NAME	"Samples\\RollThumbLeft.bmp"
#define LEFT_INDEX_F_NAME	"Samples\\RollIndexLeft.bmp"
#define LEFT_MIDDLE_F_NAME	"Samples\\RollMiddleLeft.bmp"
#define LEFT_RING_F_NAME	"Samples\\RollRingLeft.bmp"
#define LEFT_LITTLE_F_NAME	"Samples\\RollLittleLeft.bmp"
#define RIGHT_THUMB_F_NAME	"Samples\\RollThumbRIGHT.bmp"
#define RIGHT_INDEX_F_NAME	"Samples\\RollIndexRIGHT.bmp"
#define RIGHT_MIDDLE_F_NAME	"Samples\\RollMiddleRIGHT.bmp"
#define RIGHT_RING_F_NAME	"Samples\\RollRingRIGHT.bmp"
#define RIGHT_LITTLE_F_NAME	"Samples\\RollLittleRIGHT.bmp"

/********************************************************************************************/
// Initializes the global variables used in this project
/********************************************************************************************/
int		InitGlobals(void);

/********************************************************************************************/
// Processes a slap image and returns a record to be added to the ANSI-NIST file.
// Operations performed:
//
// 1) loads image
// 2) Performs segmentation (fully or semi automatic depending by ObjID) and shows
//		results together with calculated NIST quality for each finger - see
//		CalculateAutomaticSegmentation(...) and CalculateManualSegmentation(...)
// 3) Calculates minutiae for each finger
// 4) Compresses the image in WSQ format
// 5) Creates a record 14 with segmentation data - and ampution list depending by ObjID
//
// Parameters:
// - ObjID	: Object to be processed: see Fingerprint objects identifiers (input)
// - pRecord: points to the record to be added (output)
/********************************************************************************************/
int		ProcessSlap(unsigned char ObjID,RECORD **pRecord);



/********************************************************************************************/
// Processes a roll image and returns a record to be added to the ANSI-NIST file.
// Operations performed:
//
// 1) loads image
// 2) Calculates and shows NIST Quality
// 3) Calculates minutiae for each finger
// 4) Compares minutiae calculated at the point (3) with those of the corresponding
//		flat image - see point (3) of ProcessSlap(...)
// 5) Compresses the image in JPEG2000 format
// 6) Creates a record 4 
//
// Parameters:
// - ObjID	: Object to be processed: see Fingerprint objects identifiers (input)
// - pRecord: points to the record to be added (output)
/********************************************************************************************/
int ProcessRoll(Byte ObjID,RECORD **pRecord);

////////////////////////////////////////////////////////////////
///////////////// SEGMENTATION SECTION /////////////////////////
////////////////////////////////////////////////////////////////
/**************************************************************/
// As told in the presentation of this example, here the missing
// of the left ring finger is simulated, in order to show how
// should be used fully and semi automatic segmentation.
/**************************************************************/


// segments size
#define SEGM_SIZE_X		(UInt32)800
#define SEGM_SIZE_Y		(UInt32)800

// recommended segmentation diagnostic messages
#define LEFT_RIGHT_HAND_MIX_MESSAGE	"Possible left-right hand mix-up detected:"
#define INCOMPLETE_SLAP_MESSAGE		"Possible incomplete slap detected:"
#define AMBIGUOUS_SLAP_MESSAGE		"Ambiguous slap detected:"

// pattern validity, completeness, quality and inclination thresholds
// These are recommended thresholds for a correct interpretation
// of segmentation results
#define GB_PATTERN_COMPLETENESS_MIN_THRESH	85
#define GB_PATTERN_VAIDITY_MIN_THRESH		75
#define GB_PATTERN_QUALITY_MIN_THRESH		70
#define GB_PATTERN_INCLINATION_MAX_THRESH	35
#define GB_PATTERN_INCLINATION_MIN_THRESH	-35

/********************************************************************************************/
// Processes a slap image and performs a fully automatic segmentation, by using the following
// global structure array, where segmentation results are stored:
extern GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR CompleteSlapList[4];
// Operations performed:
//
// 1) Call GBFINIMG_ProcessImage passing NULL as external info list
// 2) Shows how should correctly interpreted the results (RetVal, Diagnostic, etc...)
//
// Parameters:
// - pImage	: image buffer
// - SizeX, SizeY	: image dimensions
// - ObjID	: Object currently processed
/********************************************************************************************/
extern int CalculateAutomaticSegmentation(unsigned char *pImage,unsigned int SizeX,
										  unsigned int SizeY,Byte ObjID);

/********************************************************************************************/
// Processes a slap image and performs a semi automatic segmentation, by using the following
// global structure arrays, where segmentation results are stored:
extern GBFINIMG_SEGMENT_IMAGE_DESCRIPTOR CompleteSlapList[4];
// and where an external info list is passed to GBFINIMG_ProcessImage(...)
extern GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO LeftSlapExternInfoList[3];
// In this last vector approximate coordinates of finger center into left slap image (taken
// from file Smaples\LeftSlap.bmp) are stored. They have been got simply by opening the image
// file with an imaging appliication (for example Microsoft (R) Paint) and looking to
// mouse coordinates after having pointed it near to the center of each finger. These
// coordinates have been saved as defines:
#define LITTLE_CENTER_X	(UInt32)473
#define LITTLE_CENTER_Y	(UInt32)971
#define MIDDLE_CENTER_X	(UInt32)1005
#define MIDDLE_CENTER_Y	(UInt32)654
#define INDEX_CENTER_X	(UInt32)1272
#define INDEX_CENTER_Y	(UInt32)787
// An application is responsible for supplying these coordinates to the tool by
// providing a graphical pointing tool to the operator.
//
// Operations performed:
//
// 1) Call GBFINIMG_ProcessImage passing LeftSlapExternInfoList as external info list
// 2) Shows how should correctly interpreted the results (RetVal, Diagnostic, etc...)
//
// Parameters: (no need of getting ObjID because it's for left slap)
// - pImage	: image buffer
// - SizeX, SizeY	: image dimensions
/********************************************************************************************/
extern int CalculateManualSegmentation(unsigned char *pImage,unsigned int SizeX,
									   unsigned int SizeY);

/********************************************************************************************/
// Once that Segmentation has been performed, for each finger inside slap is called this
// function, that:
//
// 1) Shows how should correctly interpreted the results of segmentation operation(GB_Quality,
//		Pattern Validity and Completeness, Finger Inclination), as indicated in the following
//		defines (see ImageProcessing.h):
//#define GB_PATTERN_COMPLETENESS_MIN_THRESH	85
//#define GB_PATTERN_VAIDITY_MIN_THRESH			75
//#define GB_PATTERN_QUALITY_MIN_THRESH			70
//#define GB_PATTERN_INCLINATION_MAX_THRESH		35
//#define GB_PATTERN_INCLINATION_MIN_THRESH		-35
//
// 2) Calculates NIST Quality and shows it after having stored into NIST_QualityList;
//		that will be loaded as a parameter into record 14
//
//
// Parameters:
// - FingerID	: ID of the finger, choosen between defines in GBFINIMG.h
// - NISTQualityListIndex	: Index of NIST_QualityList where NIST quality must be stored,
//		choosen by the calling function following this table:
//
// -------------------------------------------------------------------------------------------
//	OBJ ID		|	FingerID				|	HandID				|	NISTQualityListIndex |
// -------------------------------------------------------------------------------------------
// -------------------------------------------------------------------------------------------
//	LEFT_SLAP	|	GBFINIMG_FINGER_INDEX	|	Don't care			|	0                    |
// -------------------------------------------------------------------------------------------
//	LEFT_SLAP	|	GBFINIMG_FINGER_MIDDLE	|	Don't care			|	1                    |
// -------------------------------------------------------------------------------------------
//	LEFT_SLAP	|	GBFINIMG_FINGER_RING	|	Don't care			|	NOTHING (Missing)    |
// -------------------------------------------------------------------------------------------
//	LEFT_SLAP	|	GBFINIMG_FINGER_LITTLE	|	Don't care			|	2                    |
// -------------------------------------------------------------------------------------------
//	RIGHT_SLAP	|	GBFINIMG_FINGER_INDEX	|	Don't care			|	0                    |
// -------------------------------------------------------------------------------------------
//	RIGHT_SLAP	|	GBFINIMG_FINGER_MIDDLE	|	Don't care			|	1                    |
// -------------------------------------------------------------------------------------------
//	RIGHT_SLAP	|	GBFINIMG_FINGER_RING	|	Don't care			|	2                    |
// -------------------------------------------------------------------------------------------
//	RIGHT_SLAP	|	GBFINIMG_FINGER_LITTLE	|	Don't care			|	3                    |
// -------------------------------------------------------------------------------------------
//	TWO_THUMBS	|	GBFINIMG_FINGER_THUMB	|	GBFINIMG_HAND_LEFT	|	0                    |
// -------------------------------------------------------------------------------------------
//	TWO_THUMBS	|	GBFINIMG_FINGER_THUMB	|	GBFINIMG_HAND_RIGHT	|	1                    |
// -------------------------------------------------------------------------------------------
//
// - ANSI_NIST_FingID	: Finger Identifier choosen between the values in an2k_dll.h and
//		corresponding to values of finger for the currently processed object (for example,
//		if current process is LEFT_SLAP and FingerID is GBFINIMG_FINGER_INDEX then
//		ANSI_NIST_FingID should be ETFS_14_FGP_LEFT_INDEX)
// - HandID	: ID of thehand, choosen between defines in GBFINIMG.h. It's needed only for
//		2 thumbs object, because for slaps all fingers belong to the same hand
/********************************************************************************************/
extern void	DisplaySegmInfoForFingerInSlap(int FingerID, int NISTQualityListIndex,
									   char *ANSI_NIST_FingID, int HandID = -1);



////////////////////////////////////////////////////////////////
///////////////// ANSI/NIST SECTION ////////////////////////////
////////////////////////////////////////////////////////////////

// scanner resolution
#define GB_RES_PPMM	(double)19.67 // 500 dpi


/********************************************************************************************/
// Initializes the LeftSlapAmpList
// Shows how should be filled a FING_AMP_LIST structure to be useful for Green Bit an2k 
// software library
/********************************************************************************************/
extern int	InitAmputationList(void);

/********************************************************************************************/
// This function allocs memory for storing the informations about the NIST quality for
// each segment extracted from a slap. Memory is allocated for the following global
// FOUR_FING_QUALITY structure:
extern FOUR_FING_QUALITY *NIST_QualityList;
//
// Parameters:
// - ObjID	: ID of the slap to be processed
/********************************************************************************************/
extern int	BuildQualityListFromObjID(int ObjID);

/********************************************************************************************/
// This function allocs memory and shows how the informations aboout the position of each
// segment inside a slap should be stored into a FOUR_FING_BOUND structure.
//
// Parameters:
// - pBoundaries: points to the FOUR_FING_BOUND structure to be filled (output)
// - ObjID	: ID of the slap to be processed (input)
/********************************************************************************************/
extern int	BuildBoundaryBoxListFromObjID(FOUR_FING_BOUND **pBoundaries,int ObjID);

/********************************************************************************************/
// This function allocs memory and shows how the informations aboout the GB quality of each
// segment inside a slap should be stored into a ALT_QUAL_LIST structure.
// Identifiers of the quality algorithm are the followings: see GBTenPrint.h
#define GB_QUAL_ALG_VENDOR_ID	0x0040 // Green Bit IBIA Vendor ID
#define GB_QUAL_ALG_PRODUCT_ID	0x0001 // Green Bit Quality Algorithm product ID
//
// Parameters:
// - pQualList: points to the ALT_QUAL_LIST structure to be filled (output)
// - ObjID	: ID of the slap to be processed (input)
/********************************************************************************************/
extern int	BuildSegmentationQualityListFromObjID(ALT_QUAL_LIST **pQualList,int ObjID);

/********************************************************************************************/
// This function returns an integer representing the finger position to be passed to the
// image2ETFStype_4 (...) function of the AN2K tool. Unlike the image2ETFStype_14 (...)
// function, it wants an integer instead of a string representing the finger position
//
// Parameters:
// - ObjID	: ID of the slap to be processed (input)
//
// Return:
// - a value >= 0 if valid finger position
// - a value < 0 in case of error
/********************************************************************************************/
extern int	getFingerPosForRec4FromObjID(Byte ObjID);

/********************************************************************************************/
// This function shows how a record 4 should correctly be built starting from input
// parameters. No preliminary operations are needed, except for getting finger position from
// the ID of the object (see getFingerPosForRec4FromObjID(...)).
//
// Parameters:
// - ObjID			: ID of the slap to be processed (input)
// - pRecord		: points to the record to be created (output)
// - ImgCompressed	: buffer of the image to be stored (input)
// - ImgCompLen		: size of the ImgCompressed buffer (input)
// - SizeX, SizeY	: dimensions of the ImgCompressed buffer (input)
// - ImgCompAlg		: AN2K (see an2k_dll.h) Id of the compression algorithm (input)
/********************************************************************************************/
extern int	CreateRec4FromObjID(Byte ObjID,
							RECORD **pRecord,
							Byte *ImgCompressed,int ImgCompLen,int SizeX, int SizeY,
							char *ImgCompAlg);

/********************************************************************************************/
// This function shows how a record 14 should correctly be built starting from input
// parameters.
// Operations performed:
//
// 1) Get Finger Position (a string)
// 2) Get current amputation list:
extern	FING_AMP_LIST *LeftSlapAmpList; // for left slap
//		NULL for right slap and two thumbs images)
// 3) Build bounding box list from segmentation info (see BuildBoundaryBoxListFromObjID(...))
// 4) Build GB alternate quality from segmentation info (see
//		BuildSegmentationQualityListFromObjID(...))
// 5) Create record 14
//
// Parameters:
// - ObjID			: ID of the slap to be processed (input)
// - pRecord		: points to the record to be created (output)
// - ImgCompressed	: buffer of the image to be stored (input)
// - ImgCompLen		: size of the ImgCompressed buffer (input)
// - SizeX, SizeY	: dimensions of the ImgCompressed buffer (input)
// - ImgCompAlg		: AN2K (see an2k_dll.h) Id of the compression algorithm (input)
/********************************************************************************************/
extern int	CreateRec14FromObjID(int ObjID,
							 RECORD **pRecord,
							 Byte *ImgCompressed,int ImgCompLen,int SizeX, int SizeY,
							 char *ImgCompAlg);

/********************************************************************************************/
// Creates a type-2 record with amputation list and insert it into an ANSI/NIST file. 
// Amputation list is defined this way:
// extern FING_AMP_LIST *LeftSlapAmpList;
//
// product serial number and device model for record 2 are:
#define GB_PSN	"Green Bit 00001"
#define GB_MOD	"Green Bit Device"
//
// Parameters:
// - ansi_nist	: Points to the ANSI/NIST structure where the record created in this function
//		must be added
// - pRecord	: Points to the record to be added
/********************************************************************************************/
extern int	CreateRec2FromRollAmpList(ANSI_NIST *ansi_nist);

////////////////////////////////////////////////////////////////
///////////////// MINUTIAE  SECTION ////////////////////////////
////////////////////////////////////////////////////////////////

/********************************************************************************************/
// This function returns the index inside
extern MINUTIAE *FlatMinutiae[10];
// array from the ObjID passed as parameter.
//
// Parameters:
// - ObjID	: ID of the slap to be processed. Must be one of those defined in 
//		ImageProcessing.h
//
// Return:
// - a value >= 0 if valid index
// - a value < 0 in case of error
/********************************************************************************************/
extern int	GetFlatMinutiaeIndexFromObjID(int ObjID);

/********************************************************************************************/
// This function shows how should correctly be used the lfs tool. Simply calculates minutiae
// for the finger whose ID parameters were passed as input.
// Operations performed:
//
// 1) Get the pointer to the list where calculated minutiae should be stored (see
//		GetFlatMinutiaeIndexFromObjID(...))
// 2) Get the segment where taking the image buffer from (see SearchSlapIndexFromFingerID(...)
//		in GB_Segmentation.h
// 3) Calculate minutiae (lfs tool)
//
// Parameters:
// - FingerID	: ID of the finger as defined in ImageProcessing.h. It will be passed to
//		GetFlatMinutiaeIndexFromObjID(...)
// - GBFINIMG_FingerID: ID of the finger as defined in GBFINIMG.h. It will be passed to
//		SearchSlapIndexFromFingerID(...) together with the Hand id if needed
/********************************************************************************************/
extern void	CalcMinForFingerInSlap(int FingerID, int GBFINIMG_FingerID);

/********************************************************************************************/
// This function simply calls CalcMinForFingerInSlap(...) for each object in the slap whose
// ID is passed as parameter.
//
// Parameters:
// - ObjID	: ID of the slap to be processed
/********************************************************************************************/
extern int	CalculateSlapMinutiae(unsigned char ObjID);

/********************************************************************************************/
// This function shows how should correctly be used the lfs  and bozorth tools.
// Operations performed:
//
// 1) Calculate minutiae (lfs tool)
// 2) Get list of the slap corresponding minutiae(see GetFlatMinutiaeIndexFromObjID(...))
// 3) Calculates matching score (bozorth tool)
// 4) If matching score is below the following threshold:
#define GB_MIN_COMPARISON_SCORE	24
//		a warning message is displayed
//
// Parameters:
// - frame			: image buffer
// - SizeX, SizeY	: image dimensions
// - ObjID			: ID of the slap to be processed
/********************************************************************************************/
extern void	CalculateRollMinutiaeAndCompare(Byte *frame,int SizeX,int SizeY, int ObjID);

#endif IMAGEPROCESSING_H