using System;
using System.Collections.Generic;
using System.Text;

namespace EFTSRecGen_ImgProcTool_Cs
{
    /// <summary>
    /// <para>Class with routines used for processing rolled fingerprints</para>
    /// <para>Operations performed in processing rolled fingerprints:</para>
    /// <para>1) Calculate and show NIST Quality</para>
    /// <para>2) Calculate MINUTIAE and compare with flat</para>
    /// <para>3) Compress image with JPEG 2000 algorithm</para>
    /// <para>6) Create RECORD 4</para>
    /// </summary>
    class ProcessRoll
    {
        /// <summary>
        /// Processes the rolled fingers.
        /// </summary>
        /// <param name="FileName">Name of the file where image is stored.</param>
        /// <param name="FingerID">The finger ID from AN2K.</param>
        /// <returns>
        /// Record 4 to be stored into ANSI_NIST structure
        /// </returns>
        /// <exception cref="System.Exception">
        /// 	<para>Thrown when an error occurs</para>
        /// 	<para>Read message</para>
        /// </exception>
        public static AN2K_NET_WRAPPER.NW_AN2K_Type4_RECORD ProcessRolledFingers(String FileName, String FingerID)
        {
            try
            {
                /************************/
                /* Load frame from file */
                /************************/
                Byte[] Frame = new Byte[1];
                Int32 SizeX, SizeY;
                try
                {
                    JPEG_NET_WRAPPER.NW_GBJPEG_BmpReader BmpRead = new JPEG_NET_WRAPPER.NW_GBJPEG_BmpReader(FileName);
                    Frame = BmpRead.Buffer.Frame;
                    SizeX = BmpRead.Buffer.SizeX;
                    SizeY = BmpRead.Buffer.SizeY;
                }
                catch (Exception ex)
                {
                    throw new Exception(ex.Message);
                }

                if (Frame == null)
                {
                    Exception ex = new Exception("Cannot load Image File for roll");
                    throw (ex);
                }

                Console.WriteLine("Image Loaded...\nCalculating NIST Quality...");

                /****************************/
                /* Calculating NIST Quality */
                /****************************/
                GBNFIQ_NET_WRAPPER.NW_GBNFIQ_ImageQuality_Info_InputData InpForNFIQ =
                    new GBNFIQ_NET_WRAPPER.NW_GBNFIQ_ImageQuality_Info_InputData();
                InpForNFIQ.Frame = Frame;
                InpForNFIQ.Width = SizeX;
                InpForNFIQ.Height = SizeY;
                GBNFIQ_NET_WRAPPER.NW_GBNFIQ_ImageQuality_Info NFIQ_Calculator;
                GBNFIQ_NET_WRAPPER.NW_GBNFIQ_ImageQuality_Info_OutputData NFIQ_Output;
                try
                {
                    NFIQ_Calculator = new GBNFIQ_NET_WRAPPER.NW_GBNFIQ_ImageQuality_Info(InpForNFIQ);
                    NFIQ_Output = NFIQ_Calculator.NW_GBNFIQ_GetImageQuality();
                }
                catch (Exception ex)
                {
                    // send exception to next level
                    throw (ex);
                }

                Console.WriteLine("Num of min from GBNFIQ: " + NFIQ_Output.MinInfo.Count);
                // show quality
                Console.WriteLine("NFIQ Quality for finger " + FingerID + " is " + NFIQ_Output.NFIQ_Quality);

                /***********************/
                /* Calculate Minutiae  */
                /* Actually we have minutiae, already
                 * calculated by using the GBNFIQ tool,
                 * but in this example we are going to
                 * show also how to use the LFS tool */
                /***********************/
                // prepare input for MinCalculator
                LFS_NET_WRAPPER.NW_LFS_get_minutiae_InputData LFSInput =
                    new LFS_NET_WRAPPER.NW_LFS_get_minutiae_InputData();
                LFSInput.Frame = Frame;
                LFSInput.Height = (Int32)SizeY;
                LFSInput.Width = (Int32)SizeX;
                LFSInput.PixelPermm = 19.69; // correspond to 500 dpi
                LFSInput.BitsPerPixel = 8; // 256 grayscales

                // calculate minutiae
                List<LFS_NET_WRAPPER.NW_LFS_MINUTIA> MinList;
                // NW_LFS_get_minutiae can launch exceptions:
                // user should catch them
                try
                {
                    LFS_NET_WRAPPER.NW_LFS_get_minutiae MinCalculator = new LFS_NET_WRAPPER.NW_LFS_get_minutiae(LFSInput);
                    MinList = MinCalculator.minutiae;
                }
                catch (Exception ex)
                {
                    // send exception to next level
                    throw ex;
                }

                Console.WriteLine("Num of min: " + MinList.Count);

                // compare minutiae
                Int32 IndexInMinVector = MinutiaeStorage.GetIndexFromFingerID(FingerID);
                if (IndexInMinVector < 0)
                {
                    throw new Exception("Process roll minutiae searching: Some error occurred in FingerID Calculation");
                }
                List<LFS_NET_WRAPPER.NW_LFS_MINUTIA> MinToCompare = MinutiaeStorage.MinVector[IndexInMinVector];
                BOZORTH_NET_WRAPPER.NW_BOZORTH_MatchScoreCalculator Bozorth =
                    new BOZORTH_NET_WRAPPER.NW_BOZORTH_MatchScoreCalculator(MinList, MinToCompare);
                Int32 Score;
                try
                {
                    Score = Bozorth.NW_boz_direct_call(200);
                }
                catch (Exception ex)
                {
                    // send exception to next level
                    throw ex;
                }
                // show results
                if (Score > MinutiaeStorage.GBMinComparisonScore)
                {
                    Console.WriteLine("Comparison OK: sequence check successful for this image...");
                }
                else
                {
                    Console.WriteLine("Very Low matching score: verify that the roll image corresponds to the flat one...");
                }

                /*********************************/
                /* Compress Image with JPEG 2000
                 * For JPEG 2000, as encoding parameters I need:
                 * 1) Image Frame, width and height
                 * 2) Bit Per Pixel number
                 * 3) Compression rate, in this case LOSSLESS will be used
                /*********************************/
                Console.WriteLine("Compressing image in JPEG2000 format...");
                JPEG_NET_WRAPPER.NW_GBJPEG_JPX_Encode Encoder = new JPEG_NET_WRAPPER.NW_GBJPEG_JPX_Encode(Frame, (UInt32)SizeX, (UInt32)SizeY);
                Encoder.BitPerPixel = JPEG_NET_WRAPPER.NW_GBJPEG_PIXEL_DEPTH_DEFINITIONS.NW_GBJPEG_8_BITS_PER_PIXEL;
                Encoder.CompressionRate = JPEG_NET_WRAPPER.NW_GBJPEG_RATE_DEFINITIONS.NW_GBJPEG_RATE_LOSSLESS;
                Encoder.CompressionAlgorithm = JPEG_NET_WRAPPER.NW_GBJPEG_ALGORITHMS_DEFINITIONS.NW_GBJPEG_COMP_ALG_JPEG2000;
                // encode
                Byte[] EncodedImage = Encoder.Encoded;
                if (EncodedImage == null)
                {
                    Console.WriteLine("Errors in WSQ compression:");
                    Console.WriteLine(JPEG_NET_WRAPPER.NW_GBJPEG_ERRORS.NW_GBJPEG_ERROR_EXCEPTION_STRING);
                    Console.WriteLine("Original image will be used for storage");
                    EncodedImage = Frame;
                }
                Console.WriteLine("Image compression performed");
                Console.WriteLine("");

                /*******************/
                /* Create record 4 */
                /*******************/
                AN2K_NET_WRAPPER.NW_AN2K_Type4_RECORD type4_rec = new AN2K_NET_WRAPPER.NW_AN2K_Type4_RECORD();
                Int32 RetVal = type4_rec.NW_image2ETFStype_4(EncodedImage,
                    EncodedImage.Length,
                    AN2K_NET_WRAPPER.NW_AN2K_IMPRESSION_CODES.NW_ETFS_14_LIVE_SCAN_ROLLED,
                    FingerID,
                    AN2K_NET_WRAPPER.NW_AN2K_TYPE_4_RESOLUTION_PARAMETERS.NW_ETFS_TYPE4_MAX_RES_VAL,
                    SizeX,
                    SizeY,
                    AN2K_NET_WRAPPER.NW_AN2K_COMP_ALGORITHMS.NW_BIN_COMP_JPEG2KL
                    );

                if (RetVal != AN2K_NET_WRAPPER.NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
                {
                    throw new Exception("Error in creation of Record 4");
                }

                return type4_rec;
            }
            catch (Exception ex)
            {
                // send exception to next level
                throw ex;
            }
        }
        
    }
}
