using System;
using System.Collections.Generic;
using System.Text;

namespace EFTSRecGen_ImgProcTool_Cs
{
    /// <summary>
    /// <para>Class with routines used for processing slaps</para>
    /// <para>Operations performed in processing slaps:</para>
    /// <para>1) Create external info list (for left only)</para>
    /// <para>2) Calculate SEGMENTATION</para>
    /// <para>3) For each finger:</para>
    /// <para>      3.1) Calculate and show NIST Quality and create NFIQ Quality structures to be stored in Record 14</para>
    /// <para>      3.2) Show SEGMENTATION QUALITY and create ALTERNATE Quality structures to be stored in Record 14</para>
    /// <para>      3.3) Show Pattern COMPLETNESS AND VALIDITY</para>
    /// <para>      3.4) Show INCLINATION</para>
    /// <para>      3.5) Calculate and store MINUTIAE for comparison with roll</para>
    /// <para>      3.6) Create BOUNDING BOX structures to be stored in Record 14</para>
    /// <para>4) Create AMPUTATION list (for left only)</para>
    /// <para>5) Compress image with WSQ</para>
    /// <para>6) Create RECORD 14</para>
    /// </summary>
    class ProcessSlap 
    {
        /// <summary>
        /// <para>Processes the left slap.</para>
        /// <para>This function shows:</para>
        /// <para>1) how SEGMENTATION should be used in SEMI-AUTOMATIC mode</para>
        /// </summary>
        /// <exception cref="System.Exception">
        /// <para>Thrown when something goes wrong:</para>
        /// <para>Read exception mesage</para>
        /// </exception>
        /// <returns>
        /// <see cref="AN2K_NET_WRAPPER.NW_AN2K_Type14_RECORD"/>
        /// <para>or null if something got wrong</para>
        /// </returns>
        public static AN2K_NET_WRAPPER.NW_AN2K_Type14_RECORD ProcessLeftSlap()
        {
            try
            {
                /************************/
                /* Load frame from file */
                /************************/
                Byte[] Frame = new Byte[1];
                Int32 SizeX, SizeY;
                try
                {
                    JPEG_NET_WRAPPER.NW_GBJPEG_BmpReader BmpRead = new JPEG_NET_WRAPPER.NW_GBJPEG_BmpReader(FileNameGlobals.LEFT_SLAP_F_NAME);
                    Frame = BmpRead.Buffer.Frame;
                    SizeX = BmpRead.Buffer.SizeX;
                    SizeY = BmpRead.Buffer.SizeY;
                }
                catch (Exception ex)
                {
                    throw new Exception(ex.Message);
                }

                if (Frame == null)
                {
                    Exception ex = new Exception("Cannot load Image File for left slap");
                    throw (ex);
                }

                Console.WriteLine("Image Loaded...\nCalculating segmentation...");

                /***************************/
                /*Create External info list*/
                /***************************/
                /* Create List */

                List<GBFINIMG_NET_WRAPPER.NW_GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO> Extinfo = new List<GBFINIMG_NET_WRAPPER.NW_GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO>();

                /* LITTLE */
                GBFINIMG_NET_WRAPPER.NW_GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO LittleExtInfo = new GBFINIMG_NET_WRAPPER.NW_GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO();
                // finger id choosen from AN2K finger positions
                LittleExtInfo.FingerID = AN2K_NET_WRAPPER.NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_LITTLE;
                LittleExtInfo.FingerCenterX = FingerCenterGlobals.LITTLE_CENTER_X;
                LittleExtInfo.FingerCenterY = FingerCenterGlobals.LITTLE_CENTER_Y;
                // add
                Extinfo.Add(LittleExtInfo);

                /* MIDDLE */
                LittleExtInfo = new GBFINIMG_NET_WRAPPER.NW_GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO();
                // finger id choosen from AN2K finger positions
                LittleExtInfo.FingerID = AN2K_NET_WRAPPER.NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_MIDDLE;
                LittleExtInfo.FingerCenterX = FingerCenterGlobals.MIDDLE_CENTER_X;
                LittleExtInfo.FingerCenterY = FingerCenterGlobals.MIDDLE_CENTER_Y;
                // add
                Extinfo.Add(LittleExtInfo);

                /* INDEX */
                LittleExtInfo = new GBFINIMG_NET_WRAPPER.NW_GBFINIMG_SEGMENT_IMAGE_EXTERNAL_INFO();
                // finger id choosen from AN2K finger positions
                LittleExtInfo.FingerID = AN2K_NET_WRAPPER.NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_INDEX;
                LittleExtInfo.FingerCenterX = FingerCenterGlobals.INDEX_CENTER_X;
                LittleExtInfo.FingerCenterY = FingerCenterGlobals.INDEX_CENTER_Y;
                // add
                Extinfo.Add(LittleExtInfo);

                /**************************/
                /* Calculate segmentation */
                /**************************/
                // input data
                GBFINIMG_NET_WRAPPER.NW_GBFINIMG_InputData LeftSlapInputData =
                    new GBFINIMG_NET_WRAPPER.NW_GBFINIMG_InputData(
                        Frame, (UInt32)SizeX, (UInt32)SizeY, // image
                        GBFINIMG_NET_WRAPPER.NW_GBFINIMG_IMAGE_TYPES.NW_GBFINIMG_INPUT_IMAGE_TYPE_LEFT_HAND_4, // image id
                        Extinfo, // external info
                    // option
                        GBFINIMG_NET_WRAPPER.NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_HALO_LATENT_ELIMINATION |
                        GBFINIMG_NET_WRAPPER.NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_SEGMENT_VERTICAL_ROTATION,
                    // segment image size
                        SegmentSizeGlobals.SEGM_SIZE_X,
                        SegmentSizeGlobals.SEGM_SIZE_Y
                    );

                // segmentation maker
                GBFINIMG_NET_WRAPPER.NW_GBFINIMG_OutputData Segments;
                // NW_GBFINIMG_ProcessImage can launch exceptions:
                // user should catch them
                try
                {
                    GBFINIMG_NET_WRAPPER.NW_GBFINIMG_ProcessImage Segmentator = new
                        GBFINIMG_NET_WRAPPER.NW_GBFINIMG_ProcessImage(LeftSlapInputData);

                    Segments = Segmentator.OutputAfterProcess;
                }
                catch (Exception ex)
                {
                    // send exception to next level
                    throw ex;
                }

                /*******************************/
                /* CHECKING AFTER SEGMENTATION:*/
                // Look carefully at next code rows:
                // you can see the correct way of evaluating
                // segmentation results
                /*******************************/
                /* Check diagnostic*/
                // in manual mode do not check diagnostic	

                /* Check segment number*/
                if (Segments.Segment_List.Count != 3)
                {
                    throw new Exception("Detected segment number = " + Segments.Segment_List.Count + ": should be 3");
                }

                /**************************/
                /* FOR EACH FINGER        */
                /**************************/

                // NFIQ quality list
                List<AN2K_NET_WRAPPER.NW_AN2K_FING_QUALITY> QualityList = new List<AN2K_NET_WRAPPER.NW_AN2K_FING_QUALITY>();
                // Segmentation quality list
                List<AN2K_NET_WRAPPER.NW_AN2K_ALT_QUAL> SegmQualList =
                    new List<AN2K_NET_WRAPPER.NW_AN2K_ALT_QUAL>();
                // Bounding box list
                List<AN2K_NET_WRAPPER.NW_AN2K_FING_SEGMENT> BoundingBoxList =
                    new List<AN2K_NET_WRAPPER.NW_AN2K_FING_SEGMENT>();

                for (int i = 0; i < Segments.Segment_List.Count; i++)
                {
                    /**************************/
                    /* Get Segment Image      */
                    /**************************/
                    Byte[] ActualSegmFrame = Segments.Segment_List[i].SegmentImagePtr;

                    /********************************/
                    /* show and store NIST Quality  */
                    /********************************/
                    GBNFIQ_NET_WRAPPER.NW_GBNFIQ_ImageQuality_Info_InputData NFIQ_Input =
                        new GBNFIQ_NET_WRAPPER.NW_GBNFIQ_ImageQuality_Info_InputData();
                    NFIQ_Input.Frame = ActualSegmFrame;
                    NFIQ_Input.Height = (Int32)SegmentSizeGlobals.SEGM_SIZE_Y;
                    NFIQ_Input.Width = (Int32)SegmentSizeGlobals.SEGM_SIZE_X;

                    GBNFIQ_NET_WRAPPER.NW_GBNFIQ_ImageQuality_Info_OutputData NFIQ_Output;
                    // NW_GBNFIQ_ImageQuality_Info can launch exceptions:
                    // user should catch them
                    try
                    {
                        GBNFIQ_NET_WRAPPER.NW_GBNFIQ_ImageQuality_Info NFIQ_Calculator =
                            new GBNFIQ_NET_WRAPPER.NW_GBNFIQ_ImageQuality_Info(NFIQ_Input);

                        NFIQ_Output = NFIQ_Calculator.NW_GBNFIQ_GetImageQuality();
                    }
                    catch (Exception ex)
                    {
                        // send exception to next level
                        throw ex;
                    }
                    Console.WriteLine("Num of min from GBNFIQ: " + NFIQ_Output.MinInfo.Count);
                    // save to quality list
                    AN2K_NET_WRAPPER.NW_AN2K_FING_QUALITY QualityItem =
                        new AN2K_NET_WRAPPER.NW_AN2K_FING_QUALITY(Segments.Segment_List[i].FingerID, (Byte)(NFIQ_Output.NFIQ_Quality));
                    QualityList.Add(QualityItem);
                    Console.WriteLine("NFIQ Quality for finger " + QualityItem.fing_id + " is " + QualityItem.quality);

                    /*******************************/
                    /* Store Segmentation Quality  */
                    /*******************************/
                    AN2K_NET_WRAPPER.NW_AN2K_ALT_QUAL SegmQualItem = Segments.Segment_List[i].SegmQuality;
                    SegmQualList.Add(SegmQualItem);
                    if (SegmQualItem.quality < SegmentationRecommendedThresholds.GB_PATTERN_QUALITY_MIN_THRESH)
                    {
                        Console.WriteLine("WARNING: Segmentation Quality = " + SegmQualItem.quality);
                        Console.WriteLine("Should be >= " + SegmentationRecommendedThresholds.GB_PATTERN_QUALITY_MIN_THRESH);
                    }

                    /******************************************************/
                    /* Show Completness, validity and inclination values  */
                    /******************************************************/
                    // validity
                    String MessageToDisplay = "";
                    if (Segments.Segment_List[i].PatternValidity < SegmentationRecommendedThresholds.GB_PATTERN_VALIDITY_MIN_THRESH)
                    {
                        MessageToDisplay = "WARNING: ";
                    }
                    MessageToDisplay += ("Segment pattern validity = " + Segments.Segment_List[i].PatternValidity);
                    Console.WriteLine(MessageToDisplay);
                    if (Segments.Segment_List[i].PatternValidity < SegmentationRecommendedThresholds.GB_PATTERN_VALIDITY_MIN_THRESH)
                    {
                        Console.WriteLine("Should be >= " + SegmentationRecommendedThresholds.GB_PATTERN_VALIDITY_MIN_THRESH);
                    }

                    // completness
                    MessageToDisplay = "";
                    if (Segments.Segment_List[i].PatternCompletness <
                        SegmentationRecommendedThresholds.GB_PATTERN_COMPLETENESS_MIN_THRESH)
                    {
                        MessageToDisplay = "WARNING: ";
                    }
                    MessageToDisplay += ("Segment pattern completness = " + Segments.Segment_List[i].PatternCompletness);
                    Console.WriteLine(MessageToDisplay);
                    if (Segments.Segment_List[i].PatternCompletness <
                        SegmentationRecommendedThresholds.GB_PATTERN_COMPLETENESS_MIN_THRESH)
                    {
                        Console.WriteLine("Should be >= " + SegmentationRecommendedThresholds.GB_PATTERN_COMPLETENESS_MIN_THRESH);
                    }

                    // inclination
                    MessageToDisplay = "";
                    if (Segments.Segment_List[i].Inclination < SegmentationRecommendedThresholds.GB_PATTERN_INCLINATION_MIN_THRESH ||
                        Segments.Segment_List[i].Inclination > SegmentationRecommendedThresholds.GB_PATTERN_INCLINATION_MAX_THRESH)
                    {
                        MessageToDisplay = "WARNING: ";
                    }
                    MessageToDisplay += ("Segment inclination = " + Segments.Segment_List[i].Inclination);
                    Console.WriteLine(MessageToDisplay);
                    if (Segments.Segment_List[i].Inclination < SegmentationRecommendedThresholds.GB_PATTERN_INCLINATION_MIN_THRESH ||
                        Segments.Segment_List[i].Inclination > SegmentationRecommendedThresholds.GB_PATTERN_INCLINATION_MAX_THRESH)
                    {
                        Console.WriteLine("Should be between " + SegmentationRecommendedThresholds.GB_PATTERN_INCLINATION_MIN_THRESH);
                        Console.WriteLine("and " + SegmentationRecommendedThresholds.GB_PATTERN_INCLINATION_MAX_THRESH);
                    }

                    /***********************/
                    /* Calculate Minutiae  */
                    /* Actually we have minutiae, already calculated by using the GBNFIQ tool,
                     * but in this example we are going to show also how to use the LFS tool */
                    /***********************/
                    // prepare input for MinCalculator
                    LFS_NET_WRAPPER.NW_LFS_get_minutiae_InputData LFSInput =
                        new LFS_NET_WRAPPER.NW_LFS_get_minutiae_InputData();
                    LFSInput.Frame = ActualSegmFrame;
                    LFSInput.Height = (Int32)SegmentSizeGlobals.SEGM_SIZE_Y;
                    LFSInput.Width = (Int32)SegmentSizeGlobals.SEGM_SIZE_X;
                    LFSInput.PixelPermm = 19.69; // correspond to 500 dpi
                    LFSInput.BitsPerPixel = 8; // 256 grayscales

                    // calculate minutiae
                    List<LFS_NET_WRAPPER.NW_LFS_MINUTIA> MinList;
                    // NW_LFS_get_minutiae can launch exceptions:
                    // user should catch them
                    try
                    {
                        LFS_NET_WRAPPER.NW_LFS_get_minutiae MinCalculator = new LFS_NET_WRAPPER.NW_LFS_get_minutiae(LFSInput);
                        MinList = MinCalculator.minutiae;
                    }
                    catch (Exception ex)
                    {
                        // send exception to next level
                        throw (new Exception("LFS_NET_WRAPPER exception: " + ex.Message));
                    }

                    Console.WriteLine("Num of min: " + MinList.Count);

                    // store minutiae
                    Int32 IndexInMinVector = MinutiaeStorage.GetIndexFromFingerID(Segments.Segment_List[i].FingerID);
                    if (IndexInMinVector < 0)
                    {
                        throw new Exception("Process slap minutiae storage: Some error occurred in FingerID Calculation");
                    }
                    MinutiaeStorage.MinVector[IndexInMinVector] = MinList;
                    Console.WriteLine("Minutiae stored at index " + IndexInMinVector);

                    /***********************************/
                    /* Store Bounding Box coordinates  */
                    /***********************************/
                    AN2K_NET_WRAPPER.NW_AN2K_FING_SEGMENT BoundBoxItem = Segments.Segment_List[i].SegmBound;
                    BoundingBoxList.Add(BoundBoxItem);

                    Console.WriteLine("");
                }

                /****************************************/
                /* Create Amputation list for left ring */
                /****************************************/
                Console.WriteLine("Creating Amputation list for left slap...");
                // create structure
                AN2K_NET_WRAPPER.NW_AN2K_FING_AMP LeftRingAmputation =
                    new AN2K_NET_WRAPPER.NW_AN2K_FING_AMP(
                        AN2K_NET_WRAPPER.NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_RING,
                        AN2K_NET_WRAPPER.NW_AN2K_AMPUTATION_CODES.NW_UNABLE_TO_PRINT
                    );
                // check if all right
                if (LeftRingAmputation == null)
                {
                    throw new Exception(AN2K_NET_WRAPPER.NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }
                // create list
                List<AN2K_NET_WRAPPER.NW_AN2K_FING_AMP> LeftSlapAmpList =
                    new List<AN2K_NET_WRAPPER.NW_AN2K_FING_AMP>();
                LeftSlapAmpList.Add(LeftRingAmputation);
                Console.WriteLine("Amputation list for left slap succesfully created");
                Console.WriteLine("");


                /***************************/
                /* Compress Image with WSQ */
                /***************************/
                Console.WriteLine("Compressing image in WSQ format...");
                // load encoding parameters

                WSQPACK_NET_WRAPPER.NW_WSQPACK_Compress Encoder = new WSQPACK_NET_WRAPPER.NW_WSQPACK_Compress(Frame, SizeX, SizeY);
                Encoder.BitPerPixel = 8;
                Encoder.CompressionRate = WSQPACK_NET_WRAPPER.NW_WSQPACK_RATE_DEFINITIONS.NW_WSQPACK_RECOMMENDED;
                Encoder.ImageResolution = 500; // in DPI
                Encoder.WSQComment = "Finger Slap # " + AN2K_NET_WRAPPER.NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_4_FING;
                // encode
                Byte[] EncodedImage = Encoder.Encoded;
                if (EncodedImage == null)
                {
                    EncodedImage = Frame;
                }
                Console.WriteLine("Image succesfully compressed");
                Console.WriteLine("");

                /********************/
                /* Create record 14 */
                /********************/
                AN2K_NET_WRAPPER.NW_AN2K_Type14_RECORD Type14_rec = new AN2K_NET_WRAPPER.NW_AN2K_Type14_RECORD();
                int RetVal = Type14_rec.NW_image2ETFStype_14(
                    EncodedImage,
                    EncodedImage.Length,
                    SizeX,
                    SizeY,
                    8,
                    19.69,
                    AN2K_NET_WRAPPER.NW_AN2K_COMP_ALGORITHMS.NW_COMP_WSQ,
                    AN2K_NET_WRAPPER.NW_AN2K_IMPRESSION_CODES.NW_ETFS_14_LIVE_SCAN_PLAN,
                    "Green BIT DS84",
                    "Finger Slap # " + AN2K_NET_WRAPPER.NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_4_FING,
                    AN2K_NET_WRAPPER.NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_4_FING,
                    LeftSlapAmpList,
                    BoundingBoxList,
                    QualityList,
                    SegmQualList,
                    null
                );

                if (RetVal != AN2K_NET_WRAPPER.NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
                {
                    throw new Exception("Error in creation of Record 14");
                }


                return Type14_rec;
            }
            catch (Exception ex)
            {
                // send exception to next level
                throw ex;
            }
        }

        /// <summary>
        /// 	<para>Processes the right and two thumbs slap.</para>
        /// 	<para>This function shows:</para>
        /// 	<para>1) how SEGMENTATION should be used in FULLY-AUTOMATIC mode</para>
        /// </summary>
        /// <param name="ObjectID">The object ID.</param>
        /// <returns>
        /// 	<see cref="AN2K_NET_WRAPPER.NW_AN2K_Type14_RECORD"/>
        /// 	<para>or null if something got wrong</para>
        /// </returns>
        /// <exception cref="System.Exception">
        /// 	<para>Thrown when something goes wrong:</para>
        /// 	<para>Read exception mesage</para>
        /// </exception>
        public static AN2K_NET_WRAPPER.NW_AN2K_Type14_RECORD ProcessRemainingSlaps(Int32 ObjectID)
        {
            try
            {
                /************************/
                /* Load frame from file */
                /************************/
                String FName;
                if (ObjectID == ObjectIDValues.RightSlap)
                {
                    FName = FileNameGlobals.RIGHT_SLAP_F_NAME;
                }
                else
                {
                    FName = FileNameGlobals.TWO_THUMBS_F_NAME;
                }
                Byte[] Frame = new Byte[1];
                Int32 SizeX, SizeY;
                try
                {
                    JPEG_NET_WRAPPER.NW_GBJPEG_BmpReader BmpRead = new JPEG_NET_WRAPPER.NW_GBJPEG_BmpReader(FName);
                    Frame = BmpRead.Buffer.Frame;
                    SizeX = BmpRead.Buffer.SizeX;
                    SizeY = BmpRead.Buffer.SizeY;
                }
                catch (Exception ex)
                {
                    throw new Exception(ex.Message);
                }

                if (Frame == null)
                {
                    Exception ex = new Exception("Cannot load Image File for slap");
                    throw (ex);
                }

                Console.WriteLine("Image Loaded...\nCalculating segmentation...");

                /**************************/
                /* Calculate segmentation */
                /**************************/
                UInt32 ImageType;
                if (ObjectID == ObjectIDValues.RightSlap)
                {
                    ImageType = GBFINIMG_NET_WRAPPER.NW_GBFINIMG_IMAGE_TYPES.NW_GBFINIMG_INPUT_IMAGE_TYPE_RIGHT_HAND_4;
                }
                else
                {
                    ImageType = GBFINIMG_NET_WRAPPER.NW_GBFINIMG_IMAGE_TYPES.NW_GBFINIMG_INPUT_IMAGE_TYPE_THUMBS_2;
                }
                // input data
                GBFINIMG_NET_WRAPPER.NW_GBFINIMG_InputData SlapInputData =
                    new GBFINIMG_NET_WRAPPER.NW_GBFINIMG_InputData(
                        Frame, (UInt32)SizeX, (UInt32)SizeY, // image
                        ImageType, // image id
                        null, // external info
                    // option
                        GBFINIMG_NET_WRAPPER.NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_HALO_LATENT_ELIMINATION |
                        GBFINIMG_NET_WRAPPER.NW_GBFINIMG_SEGMENTATION_OPTIONS.NW_GBFINIMG_SEGMENT_VERTICAL_ROTATION,
                    // segment image size
                        SegmentSizeGlobals.SEGM_SIZE_X,
                        SegmentSizeGlobals.SEGM_SIZE_Y
                    );

                // segmentation maker
                GBFINIMG_NET_WRAPPER.NW_GBFINIMG_OutputData Segments;
                // NW_GBFINIMG_ProcessImage can launch exceptions:
                // user should catch them
                try
                {
                    GBFINIMG_NET_WRAPPER.NW_GBFINIMG_ProcessImage Segmentator = new
                        GBFINIMG_NET_WRAPPER.NW_GBFINIMG_ProcessImage(SlapInputData);

                    Segments = Segmentator.OutputAfterProcess;
                }
                catch (Exception ex)
                {
                    // send exception to next level
                    throw ex;
                }

                /*******************************/
                /* CHECKING AFTER SEGMENTATION:*/
                // Look carefully at next code rows:
                // you can see the correct way of evaluating
                // segmentation results
                /*******************************/
                /* Check diagnostic*/
                Int32 ExpectedSegmentsNumber;
                if (ObjectID == ObjectIDValues.RightSlap)
                {
                    ExpectedSegmentsNumber = 4;
                }
                else
                {
                    ExpectedSegmentsNumber = 2;
                }
                if (!Segments.Diag.NW_GBFINIMG_NET_WRAPPER_NO_DIAG)
                {
                    // show diag
                    if (Segments.Diag.NW_GBFINIMG_SEGMENTATION_FAIL_LEFT_RIGHT_HAND_MIX)
                    {
                        Console.WriteLine("Possible left-right hand mix-up detected:");
                    }

                    if (Segments.Diag.NW_GBFINIMG_SEGMENTATION_FAIL_INCOMPLETE_SLAP)
                    {
                        Console.WriteLine("Possible incomplete slap detected:");
                    }

                    if (Segments.Diag.NW_GBFINIMG_SEGMENTATION_FAIL_AMBIGUOUS_SLAP)
                    {
                        Console.WriteLine("Ambiguous slap detected:");
                    }

                    /* Check segment number*/
                    if (Segments.Segment_List.Count != ExpectedSegmentsNumber)
                    {
                        // UNACCEPTABLE
                        throw new Exception("Segmentation failed...");
                    }
                    else
                    {
                        // ACCEPTABLE
                        throw new Exception("Verify that segmentation result is correct...");
                    }
                }

                /* Check segment number*/
                if (Segments.Segment_List.Count != ExpectedSegmentsNumber)
                {
                    throw new Exception("Detected segment number = " + Segments.Segment_List.Count + ": should be " + ExpectedSegmentsNumber);
                }

                /**************************/
                /* FOR EACH FINGER        */
                /**************************/

                // NFIQ quality list
                List<AN2K_NET_WRAPPER.NW_AN2K_FING_QUALITY> QualityList = new List<AN2K_NET_WRAPPER.NW_AN2K_FING_QUALITY>();
                // Segmentation quality list
                List<AN2K_NET_WRAPPER.NW_AN2K_ALT_QUAL> SegmQualList =
                    new List<AN2K_NET_WRAPPER.NW_AN2K_ALT_QUAL>();
                // Bounding box list
                List<AN2K_NET_WRAPPER.NW_AN2K_FING_SEGMENT> BoundingBoxList =
                    new List<AN2K_NET_WRAPPER.NW_AN2K_FING_SEGMENT>();

                for (int i = 0; i < Segments.Segment_List.Count; i++)
                {
                    /**************************/
                    /* Get Segment Image      */
                    /**************************/
                    Byte[] ActualSegmFrame = Segments.Segment_List[i].SegmentImagePtr;

                    /********************************/
                    /* show and store NIST Quality  */
                    /********************************/
                    GBNFIQ_NET_WRAPPER.NW_GBNFIQ_ImageQuality_Info_InputData NFIQ_Input =
                        new GBNFIQ_NET_WRAPPER.NW_GBNFIQ_ImageQuality_Info_InputData();
                    NFIQ_Input.Frame = ActualSegmFrame;
                    NFIQ_Input.Height = (Int32)SegmentSizeGlobals.SEGM_SIZE_Y;
                    NFIQ_Input.Width = (Int32)SegmentSizeGlobals.SEGM_SIZE_X;

                    GBNFIQ_NET_WRAPPER.NW_GBNFIQ_ImageQuality_Info_OutputData NFIQ_Output;
                    // NW_GBNFIQ_ImageQuality_Info can launch exceptions:
                    // user should catch them
                    try
                    {
                        GBNFIQ_NET_WRAPPER.NW_GBNFIQ_ImageQuality_Info NFIQ_Calculator =
                            new GBNFIQ_NET_WRAPPER.NW_GBNFIQ_ImageQuality_Info(NFIQ_Input);

                        NFIQ_Output = NFIQ_Calculator.NW_GBNFIQ_GetImageQuality();
                    }
                    catch (Exception ex)
                    {
                        // send exception to next level
                        throw ex;
                    }
                    Console.WriteLine("Num of min from GBNFIQ: " + NFIQ_Output.MinInfo.Count);
                    // save to quality list
                    AN2K_NET_WRAPPER.NW_AN2K_FING_QUALITY QualityItem =
                        new AN2K_NET_WRAPPER.NW_AN2K_FING_QUALITY(Segments.Segment_List[i].FingerID, (Byte)(NFIQ_Output.NFIQ_Quality));
                    QualityList.Add(QualityItem);
                    Console.WriteLine("NFIQ Quality for finger " + QualityItem.fing_id + " is " + QualityItem.quality);

                    /*******************************/
                    /* Store Segmentation Quality  */
                    /*******************************/
                    AN2K_NET_WRAPPER.NW_AN2K_ALT_QUAL SegmQualItem = Segments.Segment_List[i].SegmQuality;
                    SegmQualList.Add(SegmQualItem);
                    if (SegmQualItem.quality < SegmentationRecommendedThresholds.GB_PATTERN_QUALITY_MIN_THRESH)
                    {
                        Console.WriteLine("WARNING: Segmentation Quality = " + SegmQualItem.quality);
                        Console.WriteLine("Should be >= " + SegmentationRecommendedThresholds.GB_PATTERN_QUALITY_MIN_THRESH);
                    }

                    /******************************************************/
                    /* Show Completness, validity and inclination values  */
                    /******************************************************/
                    // validity
                    String MessageToDisplay = "";
                    if (Segments.Segment_List[i].PatternValidity < SegmentationRecommendedThresholds.GB_PATTERN_VALIDITY_MIN_THRESH)
                    {
                        MessageToDisplay = "WARNING: ";
                    }
                    MessageToDisplay += ("Segment pattern validity = " + Segments.Segment_List[i].PatternValidity);
                    Console.WriteLine(MessageToDisplay);
                    if (Segments.Segment_List[i].PatternValidity < SegmentationRecommendedThresholds.GB_PATTERN_VALIDITY_MIN_THRESH)
                    {
                        Console.WriteLine("Should be >= " + SegmentationRecommendedThresholds.GB_PATTERN_VALIDITY_MIN_THRESH);
                    }

                    // completness
                    MessageToDisplay = "";
                    if (Segments.Segment_List[i].PatternCompletness <
                        SegmentationRecommendedThresholds.GB_PATTERN_COMPLETENESS_MIN_THRESH)
                    {
                        MessageToDisplay = "WARNING: ";
                    }
                    MessageToDisplay += ("Segment pattern completness = " + Segments.Segment_List[i].PatternCompletness);
                    Console.WriteLine(MessageToDisplay);
                    if (Segments.Segment_List[i].PatternCompletness <
                        SegmentationRecommendedThresholds.GB_PATTERN_COMPLETENESS_MIN_THRESH)
                    {
                        Console.WriteLine("Should be >= " + SegmentationRecommendedThresholds.GB_PATTERN_COMPLETENESS_MIN_THRESH);
                    }

                    // inclination
                    MessageToDisplay = "";
                    if (Segments.Segment_List[i].Inclination < SegmentationRecommendedThresholds.GB_PATTERN_INCLINATION_MIN_THRESH ||
                        Segments.Segment_List[i].Inclination > SegmentationRecommendedThresholds.GB_PATTERN_INCLINATION_MAX_THRESH)
                    {
                        MessageToDisplay = "WARNING: ";
                    }
                    MessageToDisplay += ("Segment inclination = " + Segments.Segment_List[i].Inclination);
                    Console.WriteLine(MessageToDisplay);
                    if (Segments.Segment_List[i].Inclination < SegmentationRecommendedThresholds.GB_PATTERN_INCLINATION_MIN_THRESH ||
                        Segments.Segment_List[i].Inclination > SegmentationRecommendedThresholds.GB_PATTERN_INCLINATION_MAX_THRESH)
                    {
                        Console.WriteLine("Should be between " + SegmentationRecommendedThresholds.GB_PATTERN_INCLINATION_MIN_THRESH);
                        Console.WriteLine("and " + SegmentationRecommendedThresholds.GB_PATTERN_INCLINATION_MAX_THRESH);
                    }

                    /***********************/
                    /* Calculate Minutiae  */
                    /* Actually we have minutiae, already 
                     * calculated by using the GBNFIQ tool,
                     * but in this example we are going to
                     * show also how to use the LFS tool
                    /***********************/
                    // prepare input for MinCalculator
                    LFS_NET_WRAPPER.NW_LFS_get_minutiae_InputData LFSInput =
                        new LFS_NET_WRAPPER.NW_LFS_get_minutiae_InputData();
                    LFSInput.Frame = ActualSegmFrame;
                    LFSInput.Height = (Int32)SegmentSizeGlobals.SEGM_SIZE_Y;
                    LFSInput.Width = (Int32)SegmentSizeGlobals.SEGM_SIZE_X;
                    LFSInput.PixelPermm = 19.69; // correspond to 500 dpi
                    LFSInput.BitsPerPixel = 8; // 256 grayscales

                    // calculate minutiae
                    List<LFS_NET_WRAPPER.NW_LFS_MINUTIA> MinList;
                    // NW_LFS_get_minutiae can launch exceptions:
                    // user should catch them
                    try
                    {
                        LFS_NET_WRAPPER.NW_LFS_get_minutiae MinCalculator = new LFS_NET_WRAPPER.NW_LFS_get_minutiae(LFSInput);
                        MinList = MinCalculator.minutiae;
                    }
                    catch (Exception ex)
                    {
                        // send exception to next level
                        throw ex;
                    }

                    Console.WriteLine("Num of min: " + MinList.Count);

                    // store minutiae
                    Int32 IndexInMinVector = MinutiaeStorage.GetIndexFromFingerID(Segments.Segment_List[i].FingerID);
                    if (IndexInMinVector < 0)
                    {
                        throw new Exception("Process slap minutiae storage: Some error occurred in FingerID Calculation");
                    }
                    MinutiaeStorage.MinVector[IndexInMinVector] = MinList;
                    Console.WriteLine("Minutiae stored at index " + IndexInMinVector);

                    /***********************************/
                    /* Store Bounding Box coordinates  */
                    /***********************************/
                    AN2K_NET_WRAPPER.NW_AN2K_FING_SEGMENT BoundBoxItem = Segments.Segment_List[i].SegmBound;
                    BoundingBoxList.Add(BoundBoxItem);

                    Console.WriteLine("");
                }

                /***************************/
                /* Compress Image with WSQ */
                /***************************/
                String ActualFingID;
                if (ObjectID == ObjectIDValues.RightSlap)
                {
                    ActualFingID = AN2K_NET_WRAPPER.NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_4_FING;
                }
                else
                {
                    ActualFingID = AN2K_NET_WRAPPER.NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_2_THUMBS;
                }
                Console.WriteLine("Compressing image in WSQ format...");
                // load encoding parameters
                WSQPACK_NET_WRAPPER.NW_WSQPACK_Compress Encoder = new WSQPACK_NET_WRAPPER.NW_WSQPACK_Compress(Frame, SizeX, SizeY);
                Encoder.BitPerPixel = 8;
                Encoder.CompressionRate = WSQPACK_NET_WRAPPER.NW_WSQPACK_RATE_DEFINITIONS.NW_WSQPACK_RECOMMENDED;
                Encoder.ImageResolution = 500; // in DPI
                Encoder.WSQComment = "Finger Slap # " + ActualFingID;
                // encode
                Byte[] EncodedImage = Encoder.Encoded;
                if (EncodedImage == null)
                {
                    Console.WriteLine("Errors in WSQ compression:");
                    Console.WriteLine(WSQPACK_NET_WRAPPER.NW_WSQPACK_ERRORS.NW_WSQPACK_ERROR_EXCEPTION_STRING);
                    Console.WriteLine("Original image will be used for storage");
                    EncodedImage = Frame;
                }
                Console.WriteLine("Image compression performed");
                Console.WriteLine("");

                /********************/
                /* Create record 14 */
                /********************/
                AN2K_NET_WRAPPER.NW_AN2K_Type14_RECORD Type14_rec = new AN2K_NET_WRAPPER.NW_AN2K_Type14_RECORD();
                int RetVal = Type14_rec.NW_image2ETFStype_14(
                    EncodedImage,
                    EncodedImage.Length,
                    SizeX,
                    SizeY,
                    8,
                    19.69,
                    AN2K_NET_WRAPPER.NW_AN2K_COMP_ALGORITHMS.NW_COMP_WSQ,
                    AN2K_NET_WRAPPER.NW_AN2K_IMPRESSION_CODES.NW_ETFS_14_LIVE_SCAN_PLAN,
                    "Green BIT DS84",
                    "XXXXXXX",//"Finger Slap # " + ActualFingID,
                    ActualFingID,
                    null, // amputation list
                    BoundingBoxList,
                    QualityList,
                    SegmQualList,
                    null
                );

                if (RetVal != AN2K_NET_WRAPPER.NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
                {
                    if (RetVal == AN2K_NET_WRAPPER.NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION)
                    {
                        Console.WriteLine("Exception: " + AN2K_NET_WRAPPER.NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                    }
                    throw new Exception("Error in creation of Record 14: " + RetVal);
                }


                return Type14_rec;
            }
            catch (Exception ex)
            {
                // send exception to next level
                throw ex;
            }
        }
    }
}
