using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using AN2K_NET_WRAPPER;

namespace EFTSRecGen_ImgProcTool_Cs
{
    /// <summary>
    /// <para>This example performs a complete image processing</para>
    /// <para>by using Green Bit tools.</para>
    /// <para>Images have already been acquired and are saved into</para>
    /// <para>the "Samples" directory. For their name convention</para>
    /// <para>see the ImageProcessing.cpp file.</para>
    /// <para>In this example the missing of the left ring finger</para>
    /// <para>will be simulated, in order to show how to use</para>
    /// <para>the segmentation procedure both in full-automatic</para>
    /// <para>and in semi-automatic modes.</para>
    /// <para>Acquired objects are:</para>
    /// <para></para>
    /// <para>-	left 4 fingers slap (3 fingers actually because
    ///		left ring finger missing is simulated)</para>
    /// <para>-	right 4 fingers slap</para>
    /// <para>-	2 thumbs slap</para>
    /// <para>-	10 rolled fingerprints (9 fingers actually because
    ///		left ring finger missing is simulated)</para>
    /// <para></para>
    /// <para>The following operations will be performed on the
    /// fingerprint images:</para>
    /// <para></para>
    /// <para>-	slaps segmentation</para>
    /// <para>-	quality evaluation (by using NFIQ algorithm)</para>
    /// <para>-	quality evaluation (by using proprietary GB algorithm) and
    ///		calculation of validity and completeness indexes</para>
    /// <para>-	seqeunce check</para>
    /// <para></para>
    /// <para>At the end of the acquisition, all the images will</para>
    /// <para>be stored into buffer formatted in compliance with the</para>
    /// <para>ANSI/NIST ITL-1 2007 standard containing:</para>
    /// <para></para>
    /// <para>-	3 type-14 records for the slaps with segmentation
    ///		and quality info. Images are compressed by using
    ///		the WSQ algorithm</para>
    /// <para>-	9 type-4 records for the rolled fingerprints, if
    ///		rolling is supported. Images are compressed by
    ///		using the JPEG2000 algorithm</para>
    /// <para>-	1 type-2 record to store the unavailability
    ///		reason of the left ring finger</para>
    /// <para></para>
    /// <para>The ANSI-NIST buffer will be saved into a binary file</para>
    /// <para>(EFTS_Record.bin) and this file will be parsed by using</para>
    /// <para>the "an2ktool.exe" tool, provided by the ANSI-NIST.</para>
    /// <para>The output of this file will be sent to the standard</para>
    /// <para>output and can be analized by the user.</para>
    /// </summary>
    class Program
    {
        static void Main(string[] args)
        {
            // set console max number of rows
	        //Console.BufferHeight = 1000;
	        //Console.BufferWidth = 100;

            Console.WriteLine ("\t\tEFTSRecGen_ImgProcTool C#\n");

            /* Create ANSI_NIST structure */
	        Console.WriteLine ("Create ANSI NIST structure...");
            NW_AN2K_ANSI_NIST ansi_nist = new NW_AN2K_ANSI_NIST(
                NW_AN2K_VERSIONS.NW_VERSION_0400,
                "ImgProcTool CS",
                2,
                "Destination",
                "Green Bit",
                "000001",
                null,
                (Single)19.69,
                (Single)19.69,
                null);
            if (ansi_nist.Buffer == null) {
		        Console.WriteLine ("Error: Create_ANSI_NIST exception: " + NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
		        Console.Read();
		        Console.Read();
		        return;
	        }
	        Console.WriteLine ("Ansi_NIST structure succesfully created...");

	        Console.WriteLine ("");
	        Console.WriteLine ("");

            /*************/
	        // LEFT slap: a finger is missing
	        /*************/
	        Console.WriteLine("*************************************");
	        Console.WriteLine("Processing LEFT SLAP: ring is missing");
	        Console.WriteLine("*************************************");

            AN2K_NET_WRAPPER.NW_AN2K_Type14_RECORD type14_rec = new NW_AN2K_Type14_RECORD();

            try
            {
                type14_rec = ProcessSlap.ProcessLeftSlap();
            }
            catch (Exception ex)
            {
                Console.WriteLine("Exception in processing left slap. Message:");
                Console.WriteLine(ex.Message);
                Console.Read();
                return;
            }

            Int32 RetVal = ansi_nist.NW_insert_ANSI_NIST_record_ETFS(type14_rec, ansi_nist.num_records);
            if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
		    {
			    Console.WriteLine("Cannot insert record 14: error code = " + RetVal);
                if (RetVal == NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION)
                {
                    Console.WriteLine("Exception message: " + NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }
			    Console.Read();
			    return;
		    }
		    else
		    {
			    Console.WriteLine("Left slap record 14 stored...");
		    }

            Console.WriteLine("");
            Console.WriteLine("");

            /*************/
            // RIGHT slap
            /*************/
            Console.WriteLine("*************************************");
            Console.WriteLine("Processing RIGHT SLAP");
            Console.WriteLine("*************************************");

            try
            {
                type14_rec = ProcessSlap.ProcessRemainingSlaps(ObjectIDValues.RightSlap);
            }
            catch (Exception ex)
            {
                Console.WriteLine("Exception in processing right slap. Message:");
                Console.WriteLine(ex.Message);
                Console.Read();
                return;
            }

            RetVal = ansi_nist.NW_insert_ANSI_NIST_record_ETFS(type14_rec, ansi_nist.num_records);
            if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
            {
                Console.WriteLine("Cannot insert record 14: error code = " + RetVal);
                if (RetVal == NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION)
                {
                    Console.WriteLine("Exception message: " + NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }
                Console.Read();
                return;
            }
            else
            {
                Console.WriteLine("Right slap record 14 stored...");
            }

            Console.WriteLine("");
            Console.WriteLine("");

            /*************/
            // TWO THUMBS slap
            /*************/
            Console.WriteLine("*************************************");
            Console.WriteLine("Processing TWO THUMBS SLAP");
            Console.WriteLine("*************************************");

            try
            {
                type14_rec = ProcessSlap.ProcessRemainingSlaps(ObjectIDValues.TwoThSlap);
            }
            catch (Exception ex)
            {
                Console.WriteLine("Exception in processing two thumbs slap. Message:");
                Console.WriteLine(ex.Message);
            }

            RetVal = ansi_nist.NW_insert_ANSI_NIST_record_ETFS(type14_rec, ansi_nist.num_records);
            if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
            {
                Console.WriteLine("Cannot insert record 14: error code = " + RetVal);
                if (RetVal == NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION)
                {
                    Console.WriteLine("Exception message: " + NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }
                Console.Read();
                return;
            }
            else
            {
                Console.WriteLine("2 thumbs slap record 14 stored...");
            }

            Console.WriteLine ("");
	        Console.WriteLine ("");

	        /********************/
	        // insert record 2
	        /********************/
	        Console.WriteLine("*************************************");
	        Console.WriteLine("Creating Record 2 for amputation list");
	        Console.WriteLine("*************************************");

            NW_AN2K_Type2_RECORD type2_rec = new NW_AN2K_Type2_RECORD();
            List<NW_AN2K_FING_AMP> Rec2AmpList = new List<NW_AN2K_FING_AMP>();
            NW_AN2K_FING_AMP Item = new NW_AN2K_FING_AMP(
                NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_RING,
                NW_AN2K_AMPUTATION_CODES.NW_UNABLE_TO_PRINT
                );
            Rec2AmpList.Add(Item);
            RetVal = type2_rec.NW_CreateETFStype_2("Green Bit", "DactyScan84", "00000001", Rec2AmpList);
            if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
            {
                Console.WriteLine("Cannot create record 2: error code = " + RetVal);
                if (RetVal == NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION)
                {
                    Console.WriteLine("Exception message: " + NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }
                Console.Read();
                return;
            }
            else
            {
                Console.WriteLine("Record 2 for storing amputation list created...");
            }
            // insert record 2 in second position
            RetVal = ansi_nist.NW_insert_ANSI_NIST_record_ETFS(type2_rec, 1);
            if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
            {
                Console.WriteLine("Cannot insert record 2: error code = " + RetVal);
                if (RetVal == NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION)
                {
                    Console.WriteLine("Exception message: " + NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }
                Console.Read();
                return;
            }
            else
            {
                Console.WriteLine("Record 2 stored...");
            }

            Console.WriteLine("");
            Console.WriteLine("");

            /********************/
	        // Rolled Left Thumb
	        /********************/
	        Console.WriteLine("*************************************");
	        Console.WriteLine("     Processing Roll Left Thumb");
	        Console.WriteLine("*************************************");

            NW_AN2K_Type4_RECORD type4_rec = new NW_AN2K_Type4_RECORD();
            try
            {
                type4_rec = ProcessRoll.ProcessRolledFingers(FileNameGlobals.LEFT_THUMB_F_NAME,
                    NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_THUMB);
            }
            catch (Exception ex)
            {
                Console.WriteLine("Exception in processing Roll Left Thumb. Message:");
                Console.WriteLine(ex.Message);
            }

            RetVal = ansi_nist.NW_insert_ANSI_NIST_record_ETFS(type4_rec, ansi_nist.num_records);
            if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
            {
                Console.WriteLine("Cannot insert record 4: error code = " + RetVal);
                if (RetVal == NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION)
                {
                    Console.WriteLine("Exception message: " + NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }
                Console.Read();
                return;
            }
            else
            {
                Console.WriteLine("Rolled Left Thumb record 4 stored...");
            }

            Console.WriteLine("");
            Console.WriteLine("");

            /********************/
            // Rolled Left Index
            /********************/
            Console.WriteLine("*************************************");
            Console.WriteLine("     Processing Roll Left Index");
            Console.WriteLine("*************************************");

            type4_rec = new NW_AN2K_Type4_RECORD();
            try
            {
                type4_rec = ProcessRoll.ProcessRolledFingers(FileNameGlobals.LEFT_INDEX_F_NAME,
                    NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_INDEX);
            }
            catch (Exception ex)
            {
                Console.WriteLine("Exception in processing Roll Left Index. Message:");
                Console.WriteLine(ex.Message);
            }

            RetVal = ansi_nist.NW_insert_ANSI_NIST_record_ETFS(type4_rec, ansi_nist.num_records);
            if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
            {
                Console.WriteLine("Cannot insert record 4: error code = " + RetVal);
                if (RetVal == NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION)
                {
                    Console.WriteLine("Exception message: " + NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }
                Console.Read();
                return;
            }
            else
            {
                Console.WriteLine("Rolled Left Index record 4 stored...");
            }

            Console.WriteLine("");
            Console.WriteLine("");

            /********************/
            // Rolled Left Middle
            /********************/
            Console.WriteLine("*************************************");
            Console.WriteLine("     Processing Roll Left Middle");
            Console.WriteLine("*************************************");

            type4_rec = new NW_AN2K_Type4_RECORD();
            try
            {
                type4_rec = ProcessRoll.ProcessRolledFingers(FileNameGlobals.LEFT_MIDDLE_F_NAME,
                    NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_MIDDLE);
            }
            catch (Exception ex)
            {
                Console.WriteLine("Exception in processing Roll Left Middle. Message:");
                Console.WriteLine(ex.Message);
            }

            RetVal = ansi_nist.NW_insert_ANSI_NIST_record_ETFS(type4_rec, ansi_nist.num_records);
            if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
            {
                Console.WriteLine("Cannot insert record 4: error code = " + RetVal);
                if (RetVal == NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION)
                {
                    Console.WriteLine("Exception message: " + NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }
                Console.Read();
                return;
            }
            else
            {
                Console.WriteLine("Rolled Left Middle record 4 stored...");
            }

            Console.WriteLine("");
            Console.WriteLine("");

            /********************/
            // Rolled Left Little
            /********************/
            Console.WriteLine("*************************************");
            Console.WriteLine("     Processing Roll Left Little");
            Console.WriteLine("*************************************");

            type4_rec = new NW_AN2K_Type4_RECORD();
            try
            {
                type4_rec = ProcessRoll.ProcessRolledFingers(FileNameGlobals.LEFT_LITTLE_F_NAME,
                    NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_LEFT_LITTLE);
            }
            catch (Exception ex)
            {
                Console.WriteLine("Exception in processing Roll Left Little. Message:");
                Console.WriteLine(ex.Message);
            }

            RetVal = ansi_nist.NW_insert_ANSI_NIST_record_ETFS(type4_rec, ansi_nist.num_records);
            if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
            {
                Console.WriteLine("Cannot insert record 4: error code = " + RetVal);
                if (RetVal == NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION)
                {
                    Console.WriteLine("Exception message: " + NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }
                Console.Read();
                return;
            }
            else
            {
                Console.WriteLine("Rolled Left Little record 4 stored...");
            }

            Console.WriteLine("");
            Console.WriteLine("");

            /********************/
            // Rolled Right Thumb
            /********************/
            Console.WriteLine("*************************************");
            Console.WriteLine("     Processing Roll Right Thumb");
            Console.WriteLine("*************************************");

            type4_rec = new NW_AN2K_Type4_RECORD();
            try
            {
                type4_rec = ProcessRoll.ProcessRolledFingers(FileNameGlobals.RIGHT_THUMB_F_NAME,
                    NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_THUMB);
            }
            catch (Exception ex)
            {
                Console.WriteLine("Exception in processing Roll Right Thumb. Message:");
                Console.WriteLine(ex.Message);
            }

            RetVal = ansi_nist.NW_insert_ANSI_NIST_record_ETFS(type4_rec, ansi_nist.num_records);
            if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
            {
                Console.WriteLine("Cannot insert record 4: error code = " + RetVal);
                if (RetVal == NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION)
                {
                    Console.WriteLine("Exception message: " + NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }
                Console.Read();
                return;
            }
            else
            {
                Console.WriteLine("Rolled Right Thumb record 4 stored...");
            }

            Console.WriteLine("");
            Console.WriteLine("");

            /********************/
            // Rolled Right Index
            /********************/
            Console.WriteLine("*************************************");
            Console.WriteLine("     Processing Roll Right Index");
            Console.WriteLine("*************************************");

            type4_rec = new NW_AN2K_Type4_RECORD();
            try
            {
                type4_rec = ProcessRoll.ProcessRolledFingers(FileNameGlobals.RIGHT_INDEX_F_NAME,
                    NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_INDEX);
            }
            catch (Exception ex)
            {
                Console.WriteLine("Exception in processing Roll Right Index. Message:");
                Console.WriteLine(ex.Message);
            }

            RetVal = ansi_nist.NW_insert_ANSI_NIST_record_ETFS(type4_rec, ansi_nist.num_records);
            if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
            {
                Console.WriteLine("Cannot insert record 4: error code = " + RetVal);
                if (RetVal == NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION)
                {
                    Console.WriteLine("Exception message: " + NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }
                Console.Read();
                return;
            }
            else
            {
                Console.WriteLine("Rolled Right Index record 4 stored...");
            }

            Console.WriteLine("");
            Console.WriteLine("");

            /********************/
            // Rolled Right Middle
            /********************/
            Console.WriteLine("*************************************");
            Console.WriteLine("     Processing Roll Right Middle");
            Console.WriteLine("*************************************");

            type4_rec = new NW_AN2K_Type4_RECORD();
            try
            {
                type4_rec = ProcessRoll.ProcessRolledFingers(FileNameGlobals.RIGHT_MIDDLE_F_NAME,
                    NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_MIDDLE);
            }
            catch (Exception ex)
            {
                Console.WriteLine("Exception in processing Roll Right Middle. Message:");
                Console.WriteLine(ex.Message);
            }

            RetVal = ansi_nist.NW_insert_ANSI_NIST_record_ETFS(type4_rec, ansi_nist.num_records);
            if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
            {
                Console.WriteLine("Cannot insert record 4: error code = " + RetVal);
                if (RetVal == NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION)
                {
                    Console.WriteLine("Exception message: " + NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }
                Console.Read();
                return;
            }
            else
            {
                Console.WriteLine("Rolled Right Middle record 4 stored...");
            }

            Console.WriteLine("");
            Console.WriteLine("");

            /********************/
            // Rolled Right Ring
            /********************/
            Console.WriteLine("*************************************");
            Console.WriteLine("     Processing Roll Right Ring");
            Console.WriteLine("*************************************");

            type4_rec = new NW_AN2K_Type4_RECORD();
            try
            {
                type4_rec = ProcessRoll.ProcessRolledFingers(FileNameGlobals.RIGHT_RING_F_NAME,
                    NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_RING);
            }
            catch (Exception ex)
            {
                Console.WriteLine("Exception in processing Roll Right Ring. Message:");
                Console.WriteLine(ex.Message);
            }

            RetVal = ansi_nist.NW_insert_ANSI_NIST_record_ETFS(type4_rec, ansi_nist.num_records);
            if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
            {
                Console.WriteLine("Cannot insert record 4: error code = " + RetVal);
                if (RetVal == NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION)
                {
                    Console.WriteLine("Exception message: " + NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }
                Console.Read();
                return;
            }
            else
            {
                Console.WriteLine("Rolled Right Ring record 4 stored...");
            }

            Console.WriteLine("");
            Console.WriteLine("");

            /********************/
            // Rolled Right Little
            /********************/
            Console.WriteLine("*************************************");
            Console.WriteLine("     Processing Roll Little Thumb");
            Console.WriteLine("*************************************");

            type4_rec = new NW_AN2K_Type4_RECORD();
            try
            {
                type4_rec = ProcessRoll.ProcessRolledFingers(FileNameGlobals.RIGHT_LITTLE_F_NAME,
                    NW_AN2K_FINGER_POSITIONS.NW_ETFS_14_FGP_RIGHT_LITTLE);
            }
            catch (Exception ex)
            {
                Console.WriteLine("Exception in processing Roll Right Little. Message:");
                Console.WriteLine(ex.Message);
            }

            RetVal = ansi_nist.NW_insert_ANSI_NIST_record_ETFS(type4_rec, ansi_nist.num_records);
            if (RetVal != NW_AN2K_ERRORS.NW_AN2K_DLL_NO_ERROR)
            {
                Console.WriteLine("Cannot insert record 4: error code = " + RetVal);
                if (RetVal == NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION)
                {
                    Console.WriteLine("Exception message: " + NW_AN2K_ERRORS.NW_AN2K_DLL_EXCEPTION_STRING);
                }
                Console.Read();
                return;
            }
            else
            {
                Console.WriteLine("Rolled Right Little record 4 stored...");
            }

            Console.WriteLine("");
            Console.WriteLine("");

            /*******************************************/
            /* SAVE AND SHOW RESULTS                   */
            /*******************************************/
            /* create buffer from ansi_nist */
            Byte[] ANSI_NIST_Array = ansi_nist.Buffer;
            File.WriteAllBytes("EFTS_Record.bin", ANSI_NIST_Array);

            // call the "an2ktool.exe" tool into a separate process...
            System.Diagnostics.Process an2ktool = new System.Diagnostics.Process();

            an2ktool.StartInfo.FileName = "an2ktool.exe";
            an2ktool.StartInfo.Arguments = "-print all EFTS_Record.bin";
            an2ktool.StartInfo.UseShellExecute = false;
            an2ktool.StartInfo.RedirectStandardOutput = true;

            an2ktool.Start();

            // ... and send its output to the Console window
            StreamReader an2ktool_StreamReader = an2ktool.StandardOutput;
            String an2ktool_outString = an2ktool_StreamReader.ReadToEnd();
            Console.WriteLine(an2ktool_outString);
            File.WriteAllText("ansi_nist.txt", an2ktool_outString);
            an2ktool.Close();

            // keep the console window opened until a character is entered...
            Console.WriteLine("End of operation: press ENTER to exit");
            Console.Read();
            Console.Read();

            Console.Read();

            return;
        }
    }
}
