#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "GBFIR.h"
#include <QFileDialog>
#include <QMessageBox>
#include <QImage>
#include <QTextEdit>
#include <QPlainTextEdit>

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

	// needed to find the plugins directory
	QString LibPath = QDir::currentPath() + "/plugins/";
	QApplication::addLibraryPath(LibPath);

    ui->cmbFIRType->addItem("ISO-IEC 19794-4", GBFIR_FORMAT_ISO);
    ui->cmbFIRType->addItem("ANSI/INCITS 381", GBFIR_FORMAT_ANSI);

    LastSelectedDir = ".";
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::on_btnOpenImage_clicked()
{
    QString FileName = QFileDialog::getOpenFileName(this, tr("Open File"),
        LastSelectedDir, "Image file (*.bmp *.jpg *.pgm)");
    if (FileName.isEmpty())
        return;

	ClearLog();
    LastSelectedDir = QFileInfo(FileName).absoluteDir().path();

	int FIRType = ui->cmbFIRType->itemData(ui->cmbFIRType->currentIndex()).toInt();

    QString Extension = QFileInfo(FileName).suffix();

	// read image from file
	QImage Image;
	bool Result = Image.load(FileName);
	if (!Result)
	{
		QMessageBox::critical(this, "TestGBFIR", "Unsupported file type", QMessageBox::Ok);
		return;
	}

	// read image size
	int ImageSizeX, ImageSizeY;
	int ImageBufferSize;
	unsigned char *ImageBuffer;

	ImageSizeX = Image.width();
	ImageSizeY = Image.height();

    unsigned char CompressionAlgorithm;
    if (Extension == "bmp")
        CompressionAlgorithm = GBFIR_CA_UNCOMPRESSED_NO_BIT_PACKING;
    else if (Extension == "jpg")
        CompressionAlgorithm = GBFIR_CA_COMPRESSED_JPEG;
    else if (Extension == "png")
        CompressionAlgorithm = GBFIR_CA_PNG;
	else
	{
		QMessageBox::critical(this, "TestGBFIR", "Unsupported format", QMessageBox::Ok);
		return;
	}

    if (Extension == "bmp")
	{
		// copy raw data from bitmap
		ImageBufferSize = Image.byteCount();
		ImageBuffer = (unsigned char *)malloc(ImageBufferSize);
		if (ImageBuffer == NULL)
		{
			QMessageBox::critical(this, "TestGBFIR", "Not enough memory", QMessageBox::Ok);
			return;
		}
		memcpy(ImageBuffer, Image.bits(), ImageBufferSize);
	}
	else
	{
		QFile ImageFile(FileName);
		if (!ImageFile.open(QIODevice::ReadOnly))
		{
			QMessageBox::critical(this, "TestGBFIR", "Error reading file", QMessageBox::Ok);
			return;
		}

		ImageBufferSize = ImageFile.size();
		ImageBuffer = (unsigned char *)malloc(ImageBufferSize);
		if (ImageBuffer == NULL)
		{
			QMessageBox::critical(this, "TestGBFIR", "Not enough memory", QMessageBox::Ok);
			ImageFile.close();
			return;
		}
		ImageFile.read((char *)ImageBuffer, ImageBufferSize);
		ImageFile.close();
	}

    // create FIR with 1 image
    int ret;
    ret = GBFIR_CreateFIR(
		FIRType, 
		31,		// Image Acquisition Level
		NULL,	// CBEFF Product ID
		0,		// Capture Device ID
		GBFIR_SU_PIXEL_PER_INCH,
		500,	// ScanResolutionH
		500,	// ScanResolutionV
		500,	// ImageResolutionH
		500,	// ImageResolutionV
		8,		// PixelDepth
		CompressionAlgorithm);
    if (ret != GBFIR_RET_SUCCESS)
    {
        DisplayGBFIRError(ret, "GBFIR_CreateFIR");
        free(ImageBuffer);
        return;
    }

	ret = GBFIR_AddImageToFIR(ImageBuffer, ImageBufferSize, 
			ImageSizeX, ImageSizeY, 
			1,		// CountView
			1,		// ViewNumber
			100,	// ImageQuality	 
			GBFIR_FP_UNKNOWN, 
			GBFIR_IT_LIVE_SCAN_PLAIN);
	free(ImageBuffer);
	if (ret != GBFIR_RET_SUCCESS)
	{
		DisplayGBFIRError(ret, "GBFIR_AddImageToFIR");
		return;
	}
	LogMessage(FileName + " added fo FIR");

    int FIRBuffLen = GBFIR_GetFIRLen();
    unsigned char *FIRBuffer = (unsigned char *)malloc(FIRBuffLen);
    if (FIRBuffer == NULL)
    {
        QMessageBox::critical(this, "TestGBFIR", "Not enough memory", QMessageBox::Ok);
        return;
    }
    ret = GBFIR_WriteFIRToBuffer(FIRBuffer);
    if (ret != GBFIR_RET_SUCCESS)
    {
        DisplayGBFIRError(ret, "GBFIR_WriteFIRToBuffer");
        return;
    }
    GBFIR_FreeFIR();

    // save FIR to file
	QString FirExt;
	if (FIRType == GBFIR_FORMAT_ISO)
		FirExt = ".fir-iso";
	else
		FirExt = ".fir-ansi";
    QString FIRFileName = LastSelectedDir + "/" + QFileInfo(FileName).baseName() + FirExt;
    QFile FIRFile(FIRFileName);
    if (!FIRFile.open(QIODevice::WriteOnly))
    {
        QMessageBox::critical(this, "TestGBFIR", "Error writing file", QMessageBox::Ok);
        free (FIRBuffer);
        return;
    }
    FIRFile.write((const char *)FIRBuffer, FIRBuffLen);
    FIRFile.close();
    free (FIRBuffer);
	LogMessage("FIR saved in " + FIRFileName);
}

void MainWindow::on_btnOpenFIR_clicked()
{
    QString FileName = QFileDialog::getOpenFileName(this, tr("Open File"),
        LastSelectedDir, "FIR file (*.fir-iso *.fir-ansi)");
    if (FileName.isEmpty())
        return;

	ClearLog();
    LastSelectedDir = QFileInfo(FileName).absoluteDir().path();

    QString Extension = QFileInfo(FileName).suffix();
	int FIRType;
	if (Extension == "fir-iso")
		FIRType = GBFIR_FORMAT_ISO;
	else
		FIRType = GBFIR_FORMAT_ANSI;

    QFile FIRFile(FileName);
    if (!FIRFile.open(QIODevice::ReadOnly))
    {
        QMessageBox::critical(this, "TestGBFIR", "Error reading file", QMessageBox::Ok);
        return;
    }

    // read FIR file content
    int FIRBufferSize = FIRFile.size();
    unsigned char *FIRBuffer = (unsigned char *)malloc(FIRBufferSize);
    if (FIRBuffer == NULL)
    {
        QMessageBox::critical(this, "TestGBFIR", "Not enough memory", QMessageBox::Ok);
        FIRFile.close();
        return;
    }
    FIRFile.read((char *)FIRBuffer, FIRBufferSize);
    FIRFile.close();

    int ret;
    ret = GBFIR_ReadFIRFromBuffer(FIRBuffer, FIRBufferSize, FIRType);
    free (FIRBuffer);
    if (ret != GBFIR_RET_SUCCESS)
    {
        DisplayGBFIRError(ret, "GBFIR_ReadFIRFromBuffer");
        return;
    }

    // read information
	unsigned short ImageAcquisitionLevel, CaptureDeviceID, 
		ScanResolutionH, ScanResolutionV,
		ImageResolutionH, ImageResolutionV;
	unsigned char CBEFFProductId[CBEFF_PRODUCT_ID_LEN];
	unsigned char ScaleUnits, PixelDepth, ImageCompressionAlgorithm;

    ret = GBFIR_GetFIRHeaderFields(
		&ImageAcquisitionLevel, 
		CBEFFProductId, 
		&CaptureDeviceID,
		&ScaleUnits,
		&ScanResolutionH, 
		&ScanResolutionV,
		&ImageResolutionH, 
		&ImageResolutionV,
		&PixelDepth, 
		&ImageCompressionAlgorithm);
    if (ret != GBFIR_RET_SUCCESS)
    {
        DisplayGBFIRError(ret, "GBFIR_GetFIRHeaderFields");
        return;
    }

	// display informations
	LogMessage("FIR Header");
	LogMessage("    Image Acquisition Level = " + QString::number(ImageAcquisitionLevel));
	LogMessage("    Capture Device ID = " + QString::number(CaptureDeviceID));
	if (ScaleUnits == GBFIR_SU_PIXEL_PER_INCH)
		LogMessage("    Scale Units = Pixel-per-inch");
	else if (ScaleUnits == GBFIR_SU_PIXEL_PER_CM)
		LogMessage("    Scale Units = Pixel-per-cm");
	else
		LogMessage("    Scale Units = " + QString::number(ScaleUnits));
	
	LogMessage("    Scan Resolution H = " + QString::number(ScanResolutionH));
	LogMessage("    Scan Resolution V = " + QString::number(ScanResolutionV));
	LogMessage("    Image Resolution H = " + QString::number(ImageResolutionH));
	LogMessage("    Image Resolution V = " + QString::number(ImageResolutionV));
	LogMessage("    Pixel Depth = " + QString::number(PixelDepth));
	LogMessage("    Image Compression Algorithm = " + ImageCompressionAlgorithmToString(ImageCompressionAlgorithm));

    int NumFingers = GBFIR_GetNumberOfFingers();
    int ImageBufferSize;
    unsigned char *ImageBuffer;
    int ImageWidth, ImageHeight, ImageQuality, FingerPosition, ImpressionType;
	unsigned char CountView, ViewNumber;

	LogMessage("Number of records = " + QString::number(NumFingers));

    for (int i=0; i<NumFingers; i++)
    {
		LogMessage("### Record " + QString::number(i));

		ret = GBFIR_GetFingerRecordHeaderFields(i, 
			&CountView,
			&ViewNumber,
			&ImageQuality,
			&FingerPosition,
			&ImpressionType);
        if (ret != GBFIR_RET_SUCCESS)
        {
            DisplayGBFIRError(ret, "GBFIR_GetFingerRecordHeaderFields");
            return;
        }

		LogMessage("    Count View = " + QString::number(CountView));
		LogMessage("    View Number = " + QString::number(ViewNumber));
		LogMessage("    Image Quality = " + QString::number(ImageQuality));
		LogMessage("    FingerPosition = " + FingerPositionToString(FingerPosition));
		LogMessage("    ImpressionType = " + ImpressionTypeToString(ImpressionType));

        ret = GBFIR_GetFingerImageSize(i, &ImageBufferSize);
        if (ret != GBFIR_RET_SUCCESS)
        {
            DisplayGBFIRError(ret, "GBFIR_GetFingerImageSize");
            return;
        }

        ImageBuffer = (unsigned char *)malloc(ImageBufferSize);
        if (ImageBuffer == NULL)
        {
            QMessageBox::critical(this, "TestGBFIR", "Not enough memory", QMessageBox::Ok);
            return;
        }
        ret = GBFIR_GetFingerImage(i, ImageBuffer, &ImageWidth, &ImageHeight);
        if (ret != GBFIR_RET_SUCCESS)
        {
            DisplayGBFIRError(ret, "GBFIR_GetFingerImage");
            free (ImageBuffer);
            return;
        }

		LogMessage("    Image Width = " + QString::number(ImageWidth));
		LogMessage("    Image Height = " + QString::number(ImageHeight));

        QString ImageFileName;
		if (ImageCompressionAlgorithm == GBFIR_CA_UNCOMPRESSED_NO_BIT_PACKING)
		{
			// create bitmap from buffer
			QImage Image(ImageBuffer, ImageWidth, ImageHeight, QImage::Format_Indexed8);
			// create gray palette
			QVector<QRgb> GrayTable;
			GrayTable.resize(256);
			for( int k = 0; k < 256; k++ )
			   GrayTable[k] = qRgb( k, k, k );
		    Image.setColorTable(GrayTable);

			ImageFileName = LastSelectedDir + "/" + QFileInfo(FileName).baseName() + "_"
                + QString::number(i) + ".bmp";
			Image.save(ImageFileName, "BMP");
		    GrayTable.clear();

			LogMessage("    Image file saved: " + ImageFileName);
		}
		else
		{
			if (ImageCompressionAlgorithm == GBFIR_CA_UNCOMPRESSED_BIT_PACKED)
			{
				LogMessage("   Unsupported format, image not saved");
		        free (ImageBuffer);
				continue;
			}

			// save according to ImageCompressionAlgorithm
			QString Extension;
			switch (ImageCompressionAlgorithm)
			{
				case GBFIR_CA_COMPRESSED_JPEG:
					Extension = ".jpg";
					break;
				case GBFIR_CA_PNG:
					Extension = ".png";
					break;
				case GBFIR_CA_COMPRESSED_WSQ:
					Extension = ".wsq";
					break;
				case GBFIR_CA_COMPRESSED_JPEG2000:
					Extension = ".j2k";
					break;
				default:
					break;
			}
			ImageFileName = LastSelectedDir + "/" + QFileInfo(FileName).baseName() + "_"
                + QString::number(i) + Extension;

			QFile ImageFile(ImageFileName);
			if (!ImageFile.open(QIODevice::WriteOnly))
			{
				QMessageBox::critical(this, "TestGBFIR", "Error writing file", QMessageBox::Ok);
				free (ImageBuffer);
				return;
			}
			ImageFile.write((const char *)ImageBuffer, ImageBufferSize);
			ImageFile.close();
			LogMessage("    Image file saved: " + ImageFileName);
		}

        free (ImageBuffer);
    }

    GBFIR_FreeFIR();
}

void MainWindow::DisplayGBFIRError(int ErrorCode, QString FunctionName)
{
    QString Title = QApplication::instance()->applicationName();
    QString ErrMsg;

    switch (ErrorCode)
    {
        case GBFIR_RET_ERROR_PARAMETER:
            ErrMsg = "Parameter error";
            break;
        case GBFIR_RET_ERROR_FIR_UNDEFINED:
            ErrMsg = "FIR undefined (GBFIR_CreateFIR not called)";
            break;
        case GBFIR_RET_ERROR_MEMORY:
            ErrMsg = "Not enough memory";
            break;
        case GBFIR_RET_ERROR_PARSING:
            ErrMsg = "Error parsing FIR";
            break;
        default:
            ErrMsg = "";
            break;
    }

    QString Message = QString("%1 (%2)").arg(ErrMsg).arg(FunctionName);

    QMessageBox::critical(this, Title, Message, QMessageBox::Ok);
}

void MainWindow::LogMessage(QString Message)
{
	ui->txtLog->append(Message);
}

void MainWindow::ClearLog()
{
	ui->txtLog->clear();
}

QString MainWindow::FingerPositionToString(int FingerPosition)
{
	QString RetStr = "";

	switch (FingerPosition)
	{
		case GBFIR_FP_UNKNOWN:
			RetStr = "UNKNOWN";
			break;
		case GBFIR_FP_RIGHT_THUMB:
			RetStr = "RIGHT_THUMB";
			break;
		case GBFIR_FP_RIGHT_INDEX:
			RetStr = "RIGHT_INDEX";
			break;
		case GBFIR_FP_RIGHT_MIDDLE:
			RetStr = "RIGHT_MIDDLE";
			break;
		case GBFIR_FP_RIGHT_RING:
			RetStr = "RIGHT_RING";
			break;
		case GBFIR_FP_RIGHT_LITTLE:
			RetStr = "RIGHT_LITTLE";
			break;
		case GBFIR_FP_LEFT_THUMB:
			RetStr = "LEFT_THUMB";
			break;
		case GBFIR_FP_LEFT_INDEX:
			RetStr = "LEFT_INDEX";
			break;
		case GBFIR_FP_LEFT_MIDDLE:
			RetStr = "LEFT_MIDDLE";
			break;
		case GBFIR_FP_LEFT_RING:
			RetStr = "LEFT_RING";
			break;
		case GBFIR_FP_LEFT_LITTLE:
			RetStr = "LEFT_LITTLE";
			break;
		case GBFIR_FP_RIGHT_FOUR_FINGERS:
			RetStr = "RIGHT_FOUR_FINGERS";
			break;
		case GBFIR_FP_LEFT_FOUR_FINGERS:
			RetStr = "LEFT_FOUR_FINGERS";
			break;
		case GBFIR_FP_TWO_THUMBS:
			RetStr = "TWO_THUMBS";
			break;
		case GBFIR_PP_UNKNOWN_PALM:
			RetStr = "UNKNOWN_PALM";
			break;
		case GBFIR_PP_RIGHT_FULL_PALM:
			RetStr = "RIGHT_FULL_PALM";
			break;
		case GBFIR_PP_RIGHT_WRITERS_PALM:
			RetStr = "RIGHT_WRITERS_PALM";
			break;
		case GBFIR_PP_LEFT_FULL_PALM:
			RetStr = "LEFT_FULL_PALM";
			break;
		case GBFIR_PP_LEFT_WRITERS_PALM:
			RetStr = "LEFT_WRITERS_PALM";
			break;
		case GBFIR_PP_RIGHT_LOWER_PALM:
			RetStr = "RIGHT_LOWER_PALM";
			break;
		case GBFIR_PP_RIGHT_UPPER_PALM:
			RetStr = "RIGHT_UPPER_PALM";
			break;
		case GBFIR_PP_LEFT_LOWER_PALM:
			RetStr = "LEFT_LOWER_PALM";
			break;
		case GBFIR_PP_LEFT_UPPER_PALM:
			RetStr = "LEFT_UPPER_PALM";
			break;
		case GBFIR_PP_RIGHT_OTHER:
			RetStr = "RIGHT_OTHER";
			break;
		case GBFIR_PP_LEFT_OTHER:
			RetStr = "LEFT_OTHER";
			break;
		case GBFIR_PP_RIGHT_INTERDIGITAL:
			RetStr = "RIGHT_INTERDIGITAL";
			break;
		case GBFIR_PP_RIGHT_THENAR:
			RetStr = "RIGHT_THENAR";
			break;
		case GBFIR_PP_RIGHT_HYPOTHENAR:
			RetStr = "RIGHT_HYPOTHENAR";
			break;
		case GBFIR_PP_LEFT_INTERDIGITAL:
			RetStr = "LEFT_INTERDIGITAL";
			break;
		case GBFIR_PP_LEFT_THENAR:
			RetStr = "LEFT_THENAR";
			break;
		case GBFIR_PP_LEFT_HYPOTHENAR:
			RetStr = "LEFT_HYPOTHENAR";
			break;
	}

	return RetStr;
}

QString MainWindow::ImpressionTypeToString(int ImpressionType)
{
	QString RetStr = "";

	switch (ImpressionType)
	{
		case GBFIR_IT_LIVE_SCAN_PLAIN:
			RetStr = "LIVE_SCAN_PLAIN";
			break;
		case GBFIR_IT_LIVE_SCAN_ROLLED:
			RetStr = "LIVE_SCAN_ROLLED";
			break;
		case GBFIR_IT_NONLIVE_SCAN_PLAIN:
			RetStr = "NONLIVE_SCAN_PLAIN";
			break;
		case GBFIR_IT_NONLIVE_SCAN_ROLLED:
			RetStr = "NONLIVE_SCAN_ROLLED";
			break;
		case GBFIR_IT_LATENT:
			RetStr = "LATENT";
			break;
		case GBFIR_IT_SWIPE:
			RetStr = "SWIPE";
			break;
		case GBFIR_IT_LIVE_SCAN_CONTACTLESS:
			RetStr = "LIVE_SCAN_CONTACTLESS";
			break;
	}

	return RetStr;
}

QString MainWindow::ImageCompressionAlgorithmToString(int ImageCompressionAlgorithm)
{
	QString RetStr = "";

	switch (ImageCompressionAlgorithm)
	{
		case GBFIR_CA_UNCOMPRESSED_NO_BIT_PACKING:
			RetStr = "UNCOMPRESSED_NO_BIT_PACKING";
			break;
		case GBFIR_CA_UNCOMPRESSED_BIT_PACKED:
			RetStr = "UNCOMPRESSED_BIT_PACKED";
			break;
		case GBFIR_CA_COMPRESSED_JPEG:
			RetStr = "COMPRESSED_JPEG";
			break;
		case GBFIR_CA_PNG:
			RetStr = "PNG";
			break;
		case GBFIR_CA_COMPRESSED_WSQ:
			RetStr = "WSQ";
			break;
		case GBFIR_CA_COMPRESSED_JPEG2000:
			RetStr = "JPEG2000";
			break;
	}

	return RetStr;
}
