#ifndef GBFIR_H
#define GBFIR_H

#ifdef WIN32
    #include <Windows.h>
    #if defined(GBFIR_LIBRARY)
        #define GBFIR_API __declspec(dllexport)
    #else
        #define GBFIR_API __declspec(dllimport)
    #endif
#else
    #define GBFIR_API
    #define __stdcall
#endif

//------------------------------------------------------------------------------
// FIR = Finger Image Record
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// GENERAL NOTE:
//      Refer to ISO/IEC 19794-4 or ANSI-INCITS 381 specification for the meaning
//		and the allowed values of the fields.
//		Validity and consistency check of inserted data is responsibility of the
//		calling application.
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// USAGE TIPS
//
// CREATING A FINGER IMAGE RECORD
//		- call GBFIR_CreateFIR
//		- call GBFIR_AddImageToFIR for each image you want to add to FIR
//		- call GBFIR_GetFIRLen and allocate a buffer of the obtained lenght
//		- call GBFIR_WriteFIRToBuffer: the buffer will contain the FIR record
//		- call GBFIR_FreeFIR
//
// READING A FINGER IMAGE RECORD
//		- call GBFIR_ReadFIRFromBuffer passing the buffer containing the FIR
//		- call GBFIR_GetFIRHeaderFields to get the general header fields
//		- call GBFIR_GetNumberOfFingers to get the number of images int the record
//		- for each index, call GBFIR_GetFingerRecordHeaderFields to get the fields of the finger image record
//		- for each index, call GBFIR_GetFingerImageSize and allocate a buffer of the obtained lenght
//		- for each index, call GBFIR_GetFingerImage to obtain the image buffer
//		- call GBFIR_FreeFIR
//------------------------------------------------------------------------------

// return values
#define GBFIR_RET_SUCCESS				0
#define GBFIR_RET_ERROR_PARAMETER		1
#define GBFIR_RET_ERROR_FIR_UNDEFINED   2   // GBFIR_CreateFIR not called...
#define GBFIR_RET_ERROR_MEMORY			3
#define GBFIR_RET_ERROR_PARSING         4

// supported FIR (Finger Image Record) formats
#define GBFIR_FORMAT_ISO		1	// ISO-IEC 19794-4
#define GBFIR_FORMAT_ANSI		2	// ANSI/INCITS 381

#define CBEFF_PRODUCT_ID_LEN	4


extern "C" {

//------------------------------------------------------------------------------
// GBFIR_CreateFIR
//      Create a Finger Image Record with the specified fields values in the
//		General Record Header. Call this function before adding imaged to the
//		record calling GBFIR_AddImageToFIR.
//		Call GBFIR_FreeFIR at the end to free the memory allocated internally.
// Parameters:
//		ImageAcquisitionLevel: refer to specification
//		CBEFFProductId:	(only for ANSI/INCITS 381 format) refer to specification
//		CaptureDeviceID: refer to specification
//		ScaleUnits: refer to specification; the following values reported for convenience
#define GBFIR_SU_PIXEL_PER_INCH			0x01
#define GBFIR_SU_PIXEL_PER_CM			0x02
//		ScanResolutionH: refer to specification
//		ScanResolutionV: refer to specification
//		ImageResolutionH: refer to specification
//		ImageResolutionV: refer to specification
//		PixelDepth: refer to specification
//		ImageCompressionAlgorithm: refer to specification; the following values reported for convenience
#define GBFIR_CA_UNCOMPRESSED_NO_BIT_PACKING	0
#define GBFIR_CA_UNCOMPRESSED_BIT_PACKED		1
#define GBFIR_CA_COMPRESSED_WSQ					2
#define GBFIR_CA_COMPRESSED_JPEG				3
#define GBFIR_CA_COMPRESSED_JPEG2000			4
#define GBFIR_CA_PNG							5
// Return values:
//      GBFIR_RET_SUCCESS - no errors 
//      or one of the return values specified above
//------------------------------------------------------------------------------
GBFIR_API int __stdcall GBFIR_CreateFIR(int Type, unsigned short ImageAcquisitionLevel,
                                        unsigned char *CBEFFProductId, unsigned short CaptureDeviceID,
                                        unsigned char ScaleUnits, unsigned short ScanResolutionH,
                                        unsigned short ScanResolutionV, unsigned short ImageResolutionH,
                                        unsigned short ImageResolutionV, unsigned char PixelDepth,
                                        unsigned char ImageCompressionAlgorithm);
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
// GBFIR_AddImageToFIR
//      Add an image to the FIR record previously created with GBFIR_CreateFIR.
// Parameters:
//		ImageBuffer: buffer containing the image in the format specified in ImageCompressionAlgorithm parameter in GBFIR_CreateFIR
//		BufferLen: lenght of the buffer in bytes
//		ImageWidth: width of the image in pixels
//		ImageHeight: height of the image in pixels
//		CountView: refer to specification
//		ViewNumber: refer to specification
//		ImageQuality: refer to specification
//		FingerPosition: refer to specification; the following values reported for convenience
// Finger positions
#define GBFIR_FP_UNKNOWN				0
#define GBFIR_FP_RIGHT_THUMB			1
#define GBFIR_FP_RIGHT_INDEX			2
#define GBFIR_FP_RIGHT_MIDDLE			3
#define GBFIR_FP_RIGHT_RING				4
#define GBFIR_FP_RIGHT_LITTLE			5
#define GBFIR_FP_LEFT_THUMB				6
#define GBFIR_FP_LEFT_INDEX				7
#define GBFIR_FP_LEFT_MIDDLE			8
#define GBFIR_FP_LEFT_RING				9
#define GBFIR_FP_LEFT_LITTLE			10
#define GBFIR_FP_RIGHT_FOUR_FINGERS		13
#define GBFIR_FP_LEFT_FOUR_FINGERS		14
#define GBFIR_FP_TWO_THUMBS				15
// Palm positions
#define GBFIR_PP_UNKNOWN_PALM			20
#define GBFIR_PP_RIGHT_FULL_PALM		21
#define GBFIR_PP_RIGHT_WRITERS_PALM		22
#define GBFIR_PP_LEFT_FULL_PALM			23
#define GBFIR_PP_LEFT_WRITERS_PALM		24
#define GBFIR_PP_RIGHT_LOWER_PALM		25
#define GBFIR_PP_RIGHT_UPPER_PALM		26
#define GBFIR_PP_LEFT_LOWER_PALM		27
#define GBFIR_PP_LEFT_UPPER_PALM		28
#define GBFIR_PP_RIGHT_OTHER			29
#define GBFIR_PP_LEFT_OTHER				30
#define GBFIR_PP_RIGHT_INTERDIGITAL		31
#define GBFIR_PP_RIGHT_THENAR			32
#define GBFIR_PP_RIGHT_HYPOTHENAR		33
#define GBFIR_PP_LEFT_INTERDIGITAL		34
#define GBFIR_PP_LEFT_THENAR			35
#define GBFIR_PP_LEFT_HYPOTHENAR		36
//		ImpressionType:	refer to specification; the following values reported for convenience
#define GBFIR_IT_LIVE_SCAN_PLAIN          0
#define GBFIR_IT_LIVE_SCAN_ROLLED         1
#define GBFIR_IT_NONLIVE_SCAN_PLAIN       2
#define GBFIR_IT_NONLIVE_SCAN_ROLLED      3
#define GBFIR_IT_LATENT                   7
#define GBFIR_IT_SWIPE                    8
#define GBFIR_IT_LIVE_SCAN_CONTACTLESS    9
// Return values:
//      GBFIR_RET_SUCCESS - no errors 
//      or one of the return values specified above
//------------------------------------------------------------------------------
GBFIR_API int __stdcall GBFIR_AddImageToFIR(unsigned char *ImageBuffer, 
		int BufferLen, int ImageWidth, int ImageHeight, unsigned char CountView, 
		unsigned char ViewNumber, int ImageQuality, int FingerPosition, int ImpressionType);
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
// GBFIR_GetFIRLen
//      Returns the lenght of the FIR in bytes. Call this function to get
//		the size for allocating the buffer before calling GBFIR_WriteFIRToBuffer.
// Parameters:
//		None.
// Return values:
//      Lenght of the FIR in bytes
//------------------------------------------------------------------------------
GBFIR_API int __stdcall GBFIR_GetFIRLen();
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
// GBFIR_WriteFIRToBuffer
//      Writes the FIR record in the provided buffer.
// Parameters:
//		FIRBuffer: an allocated buffer of the size returned by GBFIR_GetFIRLen
// Return values:
//      GBFIR_RET_SUCCESS - no errors 
//      or one of the return values specified above
//------------------------------------------------------------------------------
GBFIR_API int __stdcall GBFIR_WriteFIRToBuffer(unsigned char *FIRBuffer);
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
// GBFIR_ReadFIRFromBuffer
//      Reads a FIR from the provoded buffer. 
//		Use GBFIR_GetFIRHeaderFields, GBFIR_GetNumberOfFingers, 
//		GBFIR_GetFingerRecordHeaderFields, GBFIR_GetFingerImageSize and 
//		GBFIR_GetFingerImage to read values from the FIR.
//		This function allocates memory internally, so call GBFIR_FreeFIR when done.
// Parameters:
//		FIRBuffer: buffer to read from
//		BufferLen: lenght of the buffer in bytes
//		Type: the type of record: GBFIR_FORMAT_ISO or GBFIR_FORMAT_ANSI 
//				Note: it is important that application is aware about used FIR format
//				as far as Finger Image Record itself does not contain this information
// Return values:
//      GBFIR_RET_SUCCESS - no errors 
//      or one of the return values specified above
//------------------------------------------------------------------------------
GBFIR_API int __stdcall GBFIR_ReadFIRFromBuffer(unsigned char *FIRBuffer, int BufferLen, int Type);
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
// GBFIR_GetFIRHeaderFields
//      Returns the fields in the FIR General Record Header.
//		Each parameter can be NULL, if the values is not desired.
// Parameters:
//		ImageAcquisitionLevel: refer to specification
//		CBEFFProductId: (only for ANSI/INCITS 381 format) refer to specification
//		CaptureDeviceID: refer to specification
//		ScaleUnits: refer to specification
//		ScanResolutionH: refer to specification
//		ScanResolutionV: refer to specification
//		ImageResolutionH: refer to specification
//		ImageResolutionV: refer to specification
//		PixelDepth: refer to specification
//		ImageCompressionAlgorithm: refer to specification
// Return values:
//      GBFIR_RET_SUCCESS - no errors 
//      or one of the return values specified above
//------------------------------------------------------------------------------
GBFIR_API int __stdcall GBFIR_GetFIRHeaderFields(unsigned short *ImageAcquisitionLevel,
                         unsigned char *CBEFFProductId, unsigned short *CaptureDeviceID,
                         unsigned char *ScaleUnits, unsigned short *ScanResolutionH,
                         unsigned short *ScanResolutionV, unsigned short *ImageResolutionH,
                         unsigned short *ImageResolutionV, unsigned char *PixelDepth,
                         unsigned char *ImageCompressionAlgorithm);
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
// GBFIR_GetNumberOfFingers
//      Returns the number of Finger Records in the FIR.
//		Use the number to iterate FIR calling GBFIR_GetFingerRecordHeaderFields,
//		GBFIR_GetFingerImageSize and GBFIR_GetFingerImage passing the index.
// Parameters:
//		None.
// Return values:
//      Number of Finger Records in the FIR
//------------------------------------------------------------------------------
GBFIR_API int __stdcall GBFIR_GetNumberOfFingers();
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
// GBFIR_GetFingerRecordHeaderFields
//      Returns the fields in the Finger Record Header at the given Index (0 based).
//		Each parameter can be NULL, if the values is not desired.
// Parameters:
//		Index:	Index of the record, 0 based
//		CountView: refer to specification
//		ViewNumber: refer to specification
//		ImageQuality: refer to specification
//		FingerPosition: refer to specification
//		ImpressionType: refer to specification
// Return values:
//      GBFIR_RET_SUCCESS - no errors 
//      or one of the return values specified above
//------------------------------------------------------------------------------
GBFIR_API int __stdcall GBFIR_GetFingerRecordHeaderFields(int Index, unsigned char *CountView, 
			unsigned char *ViewNumber, int *ImageQuality, int *FingerPosition, int *ImpressionType);
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
// GBFIR_GetFingerImageSize
//      Returns the lenght of the image contained in the Finger Record at the 
//		given Index in bytes. Call this function to get the size for allocating
//		the buffer before calling GBFIR_GetFingerImage.
// Parameters:
//		Index:	Index of the record, 0 based
//		ImageBufferSize: size needed for the buffer
// Return values:
//      GBFIR_RET_SUCCESS - no errors 
//      or one of the return values specified above
//------------------------------------------------------------------------------
GBFIR_API int __stdcall GBFIR_GetFingerImageSize(int Index, int *ImageBufferSize);
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
// GBFIR_GetFingerImage
//      Returns the image contained in the Finger Record at the given Index.
// Parameters:
//		Index:	Index of the record, 0 based
//		ImageBufferSize: an allocated buffer of the size returned by GBFIR_GetFingerImageSize
// Return values:
//      GBFIR_RET_SUCCESS - no errors 
//      or one of the return values specified above
//------------------------------------------------------------------------------
GBFIR_API int __stdcall GBFIR_GetFingerImage(int Index, unsigned char *ImageBuffer, 
											 int *ImageWidth, int *ImaheHeight);
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
// GBFIR_FreeFIR
//      Free the internally allocated memory for FIR.
//		Call this function after GBFIR_CreateFIR or GBFIR_ReadFIRFromBuffer.
// Parameters:
//		None
// Return values:
//		None
//------------------------------------------------------------------------------
GBFIR_API void __stdcall GBFIR_FreeFIR();
//------------------------------------------------------------------------------

}   // extern "C"

#endif // GBFIR_H
