#ifndef GB_AN_ENGINE_H
#define GB_AN_ENGINE_H

// ----------------------------------------------------------
/////////////////////////////////////////////////////////////
// ENGINE
//
// NOTE: For records automatically generated by GB (and not
// built by means of the build functions) morphological
// consistency is guaranteed
/////////////////////////////////////////////////////////////
// ----------------------------------------------------------
#include <an2k_dll.h>

#ifndef DllExport_an2k
#ifdef AN2K_COMPILE_FOR_DLL
#define DllExport_an2k __declspec( dllexport )
#else
#define DllExport_an2k
#endif
#endif // DllExport_an2k

// VER 2.0.0.0
//----------------------------------------------------------------------
//----------------------------------------------------------------------
// SECTION 1
// Defines and typedefs
//----------------------------------------------------------------------
//----------------------------------------------------------------------
typedef char * GB_AN_ENGINE_String;
typedef char ** GB_AN_ENGINE_StringArray;

//----------------------------------------------------------------------
//----------------------------------------------------------------------
// SECTION 2
// CREATE FUNCTIONS FOR TAGGED RECORDS
//----------------------------------------------------------------------
//----------------------------------------------------------------------
////////////////////////////////////////////////////////////////////////
// GB_AN_ENGINE_CreateSubFieldFromValuesForTaggedRecords
//
// Description:
//	Creates a subfield containing one or more than one items
//
// Parameters:
//	<outSubField>: (output) field to be created: valid only if
//		AN2K_DLL_NO_ERROR is returned. Cannot be
//		NULL. Use GB_AN_ENGINE_Free(...) to deallocate memory for this subfield.
//	<valuesToInsertArray>: (input) array of values to be inserted;
//		each value is represented by a '0' terminated string. Cannot be
//		NULL
//	<numOfValuesToInsert>: (input) number of values to be inserted. 
//		Cannot be <= 0
////////////////////////////////////////////////////////////////////////
DllExport_an2k int GB_AN_ENGINE_CreateSubFieldFromValuesForTaggedRecords(
	SUBFIELD **outSubField,
	GB_AN_ENGINE_StringArray valuesToInsertArray,
	int numOfValuesToInsert
);

////////////////////////////////////////////////////////////////////////
// GB_AN_ENGINE_CreateEmptyFieldForTaggedRecords
//
// Description:
//	Creates a field containing no items
//
// Parameters:
//	<outField>: (output) field to be created; valid only if
//		AN2K_DLL_NO_ERROR is returned. Cannot be
//		NULL. Use GB_AN_ENGINE_Free(...) to deallocate memory for this field.
//	<recordType>: (input) type of the record where
//		the newly created field should be inserted
//	<fieldId>: (input) ID of the field to be created
////////////////////////////////////////////////////////////////////////
DllExport_an2k int GB_AN_ENGINE_CreateEmptyFieldForTaggedRecords(
	FIELD **outField,
	int recordType,
	int fieldId
);

////////////////////////////////////////////////////////////////////////
// GB_AN_ENGINE_InsertSubfieldIntoFieldForTaggedRecords
//
// Description:
//	Updates a field with a new subfield, putting it at the end
//
//
// Parameters:
//	<fieldToUpdate>: (output) field to be updated.
//	<subfieldToInsert>: (input) subfield to insert into <fieldToUpdate>
////////////////////////////////////////////////////////////////////////
DllExport_an2k int GB_AN_ENGINE_InsertSubfieldIntoFieldForTaggedRecords(
	FIELD *fieldToUpdate,
	SUBFIELD *subfieldToInsert
);

////////////////////////////////////////////////////////////////////////
// GB_AN_ENGINE_CreateFieldFromValuesForTaggedRecords
//
// Description:
//	Creates a field containing one or more than one item (example:
//	"ORI" and "DOM "fields into Type-1 record)
//
// Parameters:
//	<outField>: (output) field to be created: valid only if
//		AN2K_DLL_NO_ERROR is returned. Cannot be
//		NULL. Use GB_AN_ENGINE_Free(...) to deallocate memory for this field.
//	<recordType>: (input) type of the record where
//		the newly created field should be inserted
//	<fieldId>: (input) ID of the field to be created
//		(example: 8 for "ORI" field into Type-1 record)
//	<valuesToInsertArray>: (input) array of values to be inserted;
//		each value is represented by a '0' terminated string. Cannot be
//		NULL
//	<numOfValuesToInsert>: (input) number of values to be inserted. 
//		Cannot be <= 0
////////////////////////////////////////////////////////////////////////
DllExport_an2k int GB_AN_ENGINE_CreateFieldFromValuesForTaggedRecords(
	FIELD **outField,
	int recordType,
	int fieldId,
	GB_AN_ENGINE_StringArray valuesToInsertArray,
	int numOfValuesToInsert
);

////////////////////////////////////////////////////////////////////////
// GB_AN_ENGINE_CreateImageFieldForTaggedRecord
//
// Description:
//	Creates a field containing an image. Use for NON binary records
//	(example, type-14, 15, ...)
//
// Parameters:
//	<outField>: (output) field to be created: valid only if
//		AN2K_DLL_NO_ERROR is returned. Cannot be
//		NULL. Use GB_AN_ENGINE_Free(...) to deallocate memory for this field.
//	<recordType>: (input) type of the record where
//		the newly created field should be inserted
//	<imageBuffer>: (input) Image to be inserted. Cannot be
//		NULL
//	<imageBufferLen>: (input) number of bytes of the image to be
//		inserted. Cannot be <= 0
////////////////////////////////////////////////////////////////////////
DllExport_an2k int GB_AN_ENGINE_CreateImageFieldForTaggedRecord(
	FIELD **outField,
	int recordType,
	unsigned char * imageBuffer,
	int imageBufferLen
);

////////////////////////////////////////////////////////////////////////
// GB_AN_ENGINE_CreateEmptyTaggedRecord
//
// Description:
//	Creates a tagged record containing LEN and IDC fields
//
// Parameters:
//	<outRecord>: (output) record to be created: valid only if
//		AN2K_DLL_NO_ERROR is returned. Cannot be
//		NULL. Use GB_AN_ENGINE_Free(...) to deallocate memory for this record.
//	<recordType>: (input) type of the record to be created
//	<Idc>: (input) Idc of the record to be created. If > 99 an error
//		is returned, if < 0 it's automatically set to 0
////////////////////////////////////////////////////////////////////////
DllExport_an2k int GB_AN_ENGINE_CreateEmptyTaggedRecord(
	RECORD **outRecord,
	int recordType,
	int Idc
);

////////////////////////////////////////////////////////////////////////
// GB_AN_ENGINE_InsertFieldIntoTaggedRecord
//
// Description:
//	Updates a record with a new field, putting it at the end. Use when
//	record is not still inserted into an ANSI_NIST struct. Consider
//	that, for compliance with ANSI-NIST ITL-2011 standard (see "Annex B:
//	Traditional encoding"), for all records different from Type-1:
//
//	- LEN and IDC fields MUST be the first ones into the record
//	- IMAGE field MUST be the last one
//
//	If these two rules are not fitted, AN2K_DLL_BAD_PARAMETER will be
//	returned.
//	If the record type is binary (3-8), AN2K_DLL_BAD_PARAMETER will be
//	returned.
//
// Parameters:
//	<recordToUpdate>: (output) record to be updated.
//	<fieldToInsert>: (input) field to insert into <recordToUpdate>
////////////////////////////////////////////////////////////////////////
DllExport_an2k int GB_AN_ENGINE_InsertFieldIntoTaggedRecord(
	RECORD *recordToUpdate,
	FIELD *fieldToInsert
);

//----------------------------------------------------------------------
//----------------------------------------------------------------------
// SECTION 3
// CREATE FUNCTIONS FOR BINARY RECORDS
//----------------------------------------------------------------------
//----------------------------------------------------------------------
////////////////////////////////////////////////////////////////////////
// GB_AN_ENGINE_CreateBinaryRecord
//
// Description:
//	Creates a binary record: fundamentally a type-4 record, since 3, 5
//	and 6 are deprecated by the ANSI/NIST ITL-2011 standard; type-7 is
//	user defined, but is not covered by this SDK since users are
//	encouraged to use type-13 record instead). Therefore the "strict"
//	compliance to the standard is not checked (where some restriction
//	is checked, it's clearly explained into the parameter comment)
//
// Parameters:
//	<pRecord>: (output) pointer to the record created by this function.
//		Use GB_AN_ENGINE_Free(...) to deallocate memory for this record.
//		Valid only if AN2K_DLL_NO_ERROR is returned. Cannot be NULL
//	<recordType>: (input) integer representing the record to be created
//		type. Must be a valid binary record (3,4,5,6,7) - other than 4 
//		record types are accepted, even if deprecated or not encouraged
//		by the standard
//	<idc>: (input) Idc of the record to be created. If > 99 an error
//		is returned, if < 0 it's automatically set to 0
//	<impressionType>: (input) IMP field; one byte, therefore if > 255
//		or < 0 error is returned
//	<fgp>: (input) FGP field; it's an array of bytes, and its lenght
//		cannot be < 6; cannot be NULL
//	<imageScanResolution>: (input) ISR field; one byte, therefore 
//		if > 255 or < 0 error is returned
//	<horizontalNumberOfPixels>: (input) HLL field; can assume values in
//		the [10-65535] range
//	<verticalNumberOfPixels>: (input) VLL field; can assume values in
//		the [10-65535] range
//	<compressionAlgorithm>: (input) CGA field; one byte, therefore 
//		if > 255 or < 0 error is returned
//	<imageBuffer>: (input) DATA field; it is the image to be stored
//		into the record. Cannot be NULL
//	<imageBufferLen>: (input) length in bytes of <imageBuffer> image.
//		Cannot be <= 0
////////////////////////////////////////////////////////////////////////
DllExport_an2k int GB_AN_ENGINE_CreateBinaryRecord(
	RECORD **pRecord,
	int recordType,
	int idc,
	int impressionType,
	unsigned char *fgp,
	int imageScanResolution,
	int horizontalNumberOfPixels,
	int verticalNumberOfPixels,
	int compressionAlgorithm,
	unsigned char *imageBuffer,
	int imageBufferLen
);

//----------------------------------------------------------------------
//----------------------------------------------------------------------
// SECTION 4
// CREATE FUNCTIONS FOR ALL RECORDS
//----------------------------------------------------------------------
//----------------------------------------------------------------------
// GB_AN_ENGINE_CreateEmptyAnsiNistStruct
//
// Description:
//	Creates a simple empty ANSI_NIST struct.
//
// Parameters:
//	<pAnsinistToBeCreated>: (output) ANSI_NIST struct to be created: 
//	valid only if AN2K_DLL_NO_ERROR is returned. Cannot be
//		NULL. Use GB_AN_ENGINE_Free(...) to deallocate memory for this structure.
////////////////////////////////////////////////////////////////////////
DllExport_an2k int GB_AN_ENGINE_CreateEmptyAnsiNistStruct(
	ANSI_NIST**	pAnsinistToBeCreated
);

/////////////////////////////////////////////////////////
// GB_AN_ENGINE_InsertRecordInAnsiNist
//
// Description:
//	Adds a RECORD structure to the <records> list of an
//	ANSI_NIST structure in the position specified by the
//	<RecordIndex> parameter. This routine also updates the
//	CNT field of the Type-1 record to accurately reflect
//	the insertion.
//
//	Note - If the <AutomaticIDC> is not FALSE, an IDC is
//		assigned based on the record's position in the
//		ANSI/NIST structure.
//
// Parameters:
//	<RecordToBeInserted>: (input) pointer to the RECORD
//		structure that has to be added, cannot be NULL
//		and cannot be a Type-1 record
//	<ansi_nist>: (input) pointer to the ANSI_NIST
//		structure where	<RecordToBeInserted> has to be
//		added, cannot be NULL
//	<RecordIndex>: (input) index of logical record to be
//		appended, cannot be < 0 and cannot be more
//		than "ansi_nist->num_records"
//	<AutomaticIDC>: (input) specifies if the IDC of the
//		record must be updated automatically or the
//		one in the record should be kept
//
// Return:
//	see common error definitions in "an2k_dll.h" file
/////////////////////////////////////////////////////////
DllExport_an2k int GB_AN_ENGINE_InsertRecordInAnsiNist(
	RECORD*		RecordToBeInserted,
	ANSI_NIST*	ansi_nist,
	const int	RecordIndex,
	BOOL		AutomaticIDC
);

//----------------------------------------------------------------------
//----------------------------------------------------------------------
// SECTION 5
// READ INFORMATION FUNCTIONS FOR TAGGED RECORDS
//----------------------------------------------------------------------
//----------------------------------------------------------------------
////////////////////////////////////////////////////////////////////////
// GB_AN_ENGINE_SearchFieldInTaggedRecord
//
// Description:
//	Searchs a field of the given <FieldId> into the record passed as
//	parameter.
//
// Parameters:
//	<recordToLookup>: (input) record where the field has to be searched
//		into. Cannot be NULL
//	<FieldId>: (input) id of the field to be searched. Cannot be <= 0
//	<FoundField>: (output) field that has to be found. Valid only if
//		AN2K_DLL_NO_ERROR is returned. If no fields have
//		been found, then *FoundField = NULL. Cannot be NULL
////////////////////////////////////////////////////////////////////////
DllExport_an2k int GB_AN_ENGINE_SearchFieldInTaggedRecord(
	RECORD *recordToLookup,
	int FieldId,
	FIELD ** FoundField
);


////////////////////////////////////////////////////////////////////////
// GB_AN_ENGINE_GetValuesFromFieldForTaggedRecords
//
// Description:
//	Takes all the items from a single-subfield Field.
//
// NOTE1: if the field does not contain exactly one subfield, parameter error
//	will be returned
//
// NOTE2: VERY IMPORTANT: the <pValuesToGet> has to be freed after its
//	use by using the "GB_AN_ENGINE_FreeStringArray(...)" function
//
// Parameters:
//	<inField>: (input) input field. Cannot be NULL
//	<pValuesToGet>: (output) pointer to the strings array representing
//		the items got from the input field. Cannot be NULL. Valid only if
//		AN2K_DLL_NO_ERROR is returned. Free the allocated memory by 
//		using the "GB_AN_ENGINE_FreeStringArray(...)" function
//	<pNumOfValuesToGet>: (output) number of the items got from the 
//		input field. Cannot be NULL Valid only if
//		AN2K_DLL_NO_ERROR is returned
////////////////////////////////////////////////////////////////////////
DllExport_an2k int GB_AN_ENGINE_GetValuesFromFieldForTaggedRecords(
	FIELD *inField,
	GB_AN_ENGINE_StringArray *pValuesToGet,
	int *pNumOfValuesToGet
);

////////////////////////////////////////////////////////////////////////
// GB_AN_ENGINE_GetValuesFromSubfieldForTaggedRecords
//
// Description:
//	Takes all the items from a subfield.
//
// Parameters:
//	<inSubField>: (input) input subfield. Cannot be NULL
//	<pValuesToGet>: (output) pointer to the strings array representing
//		the items got from the input field. Cannot be NULL.. Free the allocated memory by 
//		using the "GB_AN_ENGINE_FreeStringArray(...)" function
//	<pNumOfValuesToGet>: (output) number of the items got from the 
//		input field. Cannot be NULL
////////////////////////////////////////////////////////////////////////
DllExport_an2k int GB_AN_ENGINE_GetValuesFromSubfieldForTaggedRecords(
	SUBFIELD *inSubField,
	GB_AN_ENGINE_StringArray *pValuesToGet,
	int *pNumOfValuesToGet
);

////////////////////////////////////////////////////////////////////////
// GB_AN_ENGINE_GetImageFieldFromTaggedRecord
//
// Description:
//	Gets the image from a tagged record (supporting images).
//
// Parameters:
//	<recordToRead>: (input) input record. Cannot be NULL
//	<pImageBuffer>: (output) pointer to the array containing the
//		image. Cannot be NULL. Free the allocated memory by 
//		using the "GB_AN_ENGINE_FreeGenericMem(...)" function
//	<pImgLen>: (output) length in bytes of <pImageBuffer>. Cannot be NULL
////////////////////////////////////////////////////////////////////////
DllExport_an2k int GB_AN_ENGINE_GetImageFieldFromTaggedRecord(
	RECORD *recordToRead, unsigned char **pImageBuffer, int *pImgLen
);


//----------------------------------------------------------------------
//----------------------------------------------------------------------
// SECTION 6
// READ INFORMATION FUNCTIONS FOR TAGGED RECORDS
//----------------------------------------------------------------------
//----------------------------------------------------------------------
////////////////////////////////////////////////////////////////////////
// GB_AN_ENGINE_GetBinaryRecordInfo
//
// Description:
//	Reads information from a generic binary record. Validity and 
//	compliance of the retrieved data are up to the application, since some
//	values could be outside standard in order to fit application needs.
//
//	NOTE: Set to NULL the parameters whose value is not needed to be known
//
//	NOTE2: if a parameter is != NULL and corresponding field is not
//		found, AN2K_DLL_BAD_PARAMETER error is returned
//
// Parameters:
//	<recordToRead>: (input) pointer to the record to be read.
//		Cannot be NULL
//	<pRecordType>: (output) integer representing the record to be read
//		type.
//	<pIdc>: (output) Idc of the record to be read
//	<pImpressionType>: (output) IMP field
//	<fgp>: (input/output) FGP field; it's an array of bytes, and, if 
//		!= NULL, its lenght cannot be < 6
//	<pImageScanResolution>: (output) ISR field
//	<pHorizontalNumberOfPixels>: (output) HLL field
//	<pVerticalNumberOfPixels>: (output) VLL field
//	<pCompressionAlgorithm>: (output) CGA field
//	<pImageBuffer>: (output) DATA field; it's created inside the function,
//		therefore it's to be freed with GB_AN_ENGINE_FreeGenericMem(...)
//	<pImageBufferLen>: (output) length in bytes of <pImageBuffer> image.
//		It could be different from 
//		*pHorizontalNumberOfPixels x *pVerticalNumberOfPixels
////////////////////////////////////////////////////////////////////////
DllExport_an2k int GB_AN_ENGINE_GetBinaryRecordInfo(
	RECORD *recordToRead,
	int *pRecordType,
	int *pIdc,
	int *pImpressionType,
	unsigned char *fgp,
	int *pImageScanResolution,
	int *pHorizontalNumberOfPixels,
	int *pVerticalNumberOfPixels,
	int *pCompressionAlgorithm,
	unsigned char **pImageBuffer,
	int *pImageBufferLen
);

//----------------------------------------------------------------------
//----------------------------------------------------------------------
// SECTION 7
// DELETE FUNCTIONS FOR TAGGED RECORDS
//----------------------------------------------------------------------
//----------------------------------------------------------------------
////////////////////////////////////////////////////////////////////////
// GB_AN_ENGINE_DeleteFieldFromTaggedRecord
//
// Description:
//	Deletes the field whose id is <fieldId> from <recToBeUpdated>. Use
//	this function when the record is NOT still been inserted into an 
//	ANSI_NIST struct
//
// Parameters:
//	<recToBeUpdated>: (input) Record where the field should be deleted
//		from. Cannot be NULL. Cannot be a binary record (3-8). Must
//		contain more than one field (otherwise better to free the record
//		and create a new one)
//	<fieldId>: (input) id of the field to be deleted. If the field is
//		not found, function returns AN2K_DLL_NO_ERROR
////////////////////////////////////////////////////////////////////////
DllExport_an2k int GB_AN_ENGINE_DeleteFieldFromTaggedRecord(
	RECORD *recToBeUpdated,
	int fieldId
);

////////////////////////////////////////////////////////////////////////
// GB_AN_ENGINE_DeleteFieldFromTaggedRecordInAnsiNist
//
// Description:
//	Deletes the field whose id is <fieldId> from the record in 
//	<recordIndex> position inside the <anStructToBeUpdated> struct. Use
//	this function when the record is ALREADY been inserted into an 
//	ANSI_NIST struct
//
// Parameters:
//	<anStructToBeUpdated>: (input) ANSI_NIST struct containing the
//		record to be updated. Cannot be NULL.
//	<recordIndex>: (input) Index of the record to be updated. If not a
//		valid index, AN2K_DLL_BAD_PARAMETER will be returned.
//		Cannot be a binary record (3-8). Must contain more than one 
//		field (otherwise better to free the record and create a new one)
//	<fieldId>: (input) id of the field to be deleted. If the field is
//		not found, function returns AN2K_DLL_NO_ERROR
////////////////////////////////////////////////////////////////////////
DllExport_an2k int GB_AN_ENGINE_DeleteFieldFromTaggedRecordInAnsiNist(
	ANSI_NIST *anStructToBeUpdated,
	int recordIndex,
	int fieldId
);


//----------------------------------------------------------------------
//----------------------------------------------------------------------
// SECTION 8
// FREE MEMORY FUNCTIONS
//----------------------------------------------------------------------
//----------------------------------------------------------------------
////////////////////////////////////////////////////////////////////////
// GB_AN_ENGINE_Free
//
// Description:
//	Deallocates memory for the structures passed as parameters.
//
// Parameters:
//	<anStructToBeFreed>: (input) ANSI_NIST struct to be freed; set to 
//		NULL if no ANSI_NIST struct has to be freed
//	<recordToBeFreed>: (input) record to be freed; set to NULL if no
//		record has to be freed
//	<fieldToBeFreed>: (input) field to be freed; set to NULL if no
//		field has to be freed
//	<subfieldToBeFreed>: (input) subfield to be freed; set to NULL if no
//		subfield has to be freed
//	<itemToBeFreed>: (input) item to be freed; set to NULL if no
//		item has to be freed
////////////////////////////////////////////////////////////////////////
DllExport_an2k void GB_AN_ENGINE_Free(
	ANSI_NIST *anStructToBeFreed,
	RECORD *recordToBeFreed,
	FIELD *fieldToBeFreed,
	SUBFIELD *subfieldToBeFreed,
	ITEM *itemToBeFreed
);

////////////////////////////////////////////////////////////////////////
// GB_AN_ENGINE_FreeStringArray
//
// Description:
//	Frees a GB_AN_ENGINE_StringArray created by using functions like
//	"GB_AN_ENGINE_GetValuesFromFieldForTaggedRecords(...).
//
// NOTE1: do not use GB_AN_ENGINE_StringArray parameters created outside
//	this dll
//
// Parameters:
//	<ValuesToFree>: (input) GB_AN_ENGINE_StringArray to be freed.
//	<numOfValuesToFree>: (input) number of the GB_AN_ENGINE_String items
//		inside <ValuesToFree>
////////////////////////////////////////////////////////////////////////
DllExport_an2k int GB_AN_ENGINE_FreeStringArray(
	GB_AN_ENGINE_StringArray ValuesToFree,
	int numOfValuesToFree
);

////////////////////////////////////////////////////////////////////////
// GB_AN_ENGINE_FreeGenericMem
//
// Description:
//	Frees any <pointer_type> pointer allocated by this dll
//
// NOTE1: do not use GB_AN_ENGINE_FreeGenericMem for memory allocated
//	outside this dll
//
// Parameters:
//	<MemToFree>: (input) pointer to the memory to be freed.
////////////////////////////////////////////////////////////////////////
DllExport_an2k void GB_AN_ENGINE_FreeGenericMem(
	void *MemToFree
);

#endif // GB_AN_ENGINE_H
