#ifndef _AN2K_DLL_H
#define _AN2K_DLL_H

// 1.4.0.0 MODIFIED

/***********************************************************************/
/***********************************************************************/
/***********************************************************************/
/* ANSI_NIST DATA STRUCTURES */
/*
*
* The types defined below represent the data collected into an ANSI-NIST
* buffer, and reflect the "item", "subfield", "field", "record" and
* "ANSI-NIST buffer" definitions
* of the ANSI-NIST ITL 2000 (or more recent) standard.
* Remarks
* -	Remember that, fundamentally, :
*		1) An ANSI-NIST structure is a list of RECORDS
*		2) A RECORD is a list of FIELDS
*		3) A FIELD is a list of SUBFIELDS
*		4) A SUBFIELD is a list of ITEMS
*		5) A ITEM is the structure where effectively data are collected.
*
* To accurately store not only the data collected inside an ANSI-NIST
* buffer, but also the position and the way these data are collected,
* the structures defined below store not only data values, but also
* other information, whose meaning will be given more in detail.
*
* VERY IMPORTANT NOTE:
* Even if ANSI-NIST buffer structures are given here, so providing a
* user to access both in read and write records, items, fields and so on,
* an ANSI_NIST buffer should be accessed only by using the functions
* defined below in the "ROUTINES" section. At the present release they
* provide the following features:
*
* - Reading an ANSI-NIST formatted buffer into an ANSI_NIST structure
* - Writing an ANSI_NIST structure to an ANSI-NIST formatted buffer
* - Getting and setting info in the following record types:
*	type-1, type-2, type-4, type-14
* - Getting a list of records whose type is specified by the user in
*	order to get and update fields inside them.
* - Inserting a record inside an ANSI-NIST structure.
* Ver 1.2.0.0
* - Getting and setting info in the Type-15 record
* Ver 1.3.0.1
* - copy data (not pointers) from a record to another
*
*
*/
/***********************************************************************/
/***********************************************************************/
/***********************************************************************/
#pragma pack(push, 1)
//////////////////////////////////////////////////////////////////////////
////////ITEM STRUCTURE ///////////////////////////////////////////////////
// This structure represents an item as it is defined inside the standard.
// The value stored inside the item is collected inside a string
// terminated by a '\0' character. Numeric data are represented by
// a string containing the ASCII characters '0'-'9' (and '\0' to terminate
// the string).
// MEMBERS:
// -	num_bytes: the size in bytes of the item INSIDE THE	REAL BUFFER,
//		including any trailing 'US' separator.
// -	num_chars: number of characters currently used for the value 
//		inside THIS REPRESENTATION, not including the '\0' character. In
//		other terms, "num_chars" represents the strlen of "value" 
// -	alloc_chars: number of characters allocated for the value 
//		inside THIS REPRESENTATION.
// -	value: string where data are effectively stored. Must be
//		terminated by a '\0' character. The string format depends on
//		the data being stored: please refer to the standard document.
// -	us_char: can be TRUE or FALSE indicating whether the trailing
//		'US' character is set or not INSIDE THE REAL BUFFER.
//////////////////////////////////////////////////////////////////////////
typedef struct item{
   int num_bytes;
   int num_chars;
   int alloc_chars;
   unsigned char *value;
   int us_char;
} ITEM;

//////////////////////////////////////////////////////////////////////////
////////SUBFIELD STRUCTURE ///////////////////////////////////////////////
// This structure represents a subfield as it is defined inside the
// standard.
// It contains a list of ITEMS and carries information about the size of
// the subfield INSIDE THE	REAL BUFFER.
// MEMBERS:
// -	num_bytes: the size in bytes of the subfield INSIDE THE	REAL
//		BUFFER,	including any trailing 'RS' separator.
// -	num_items: number of items contained in the subfield 
// -	alloc_items: number of ITEM structures allocated.
// -	items: list of ITEM structures
// -	rs_char: can be TRUE or FALSE indicating whether the trailing
//		'RS' character is set or not INSIDE THE REAL BUFFER.
//////////////////////////////////////////////////////////////////////////
typedef struct subfield{
   int num_bytes;
   int num_items;
   int alloc_items;
   ITEM **items;
   int rs_char;
} SUBFIELD; /* items in subfields */

//////////////////////////////////////////////////////////////////////////
////////FIELD STRUCTURE //////////////////////////////////////////////////
// This structure represents a field as it is defined inside the
// standard.
// It contains a list of SUBFIELD and carries information about the size
// of the field INSIDE THE	REAL BUFFER.
// MEMBERS:
// -	id: a string representing the field ID in the format "XX.YYY:",
//		where "XX" is the record number (for example for record 1 is "1")
//		and YYY is the field Id number, as defined into standard (for
//		example for LEN field YYY is "001"). "XX" and "YYY" are replicated
//		as numbers in the following members "record_type" and "field_int"
// -	record_type: an integer representing the record type.
// -	field_int: an integer representing the field Id number.
// -	num_bytes: the size in bytes of the field INSIDE THE	REAL
//		BUFFER,	including any trailing 'GS' separator.
// -	num_subfields: number of subfields contained in the subfield list. 
// -	alloc_subfields: number of SUBFIELD structures allocated.
// -	subfields: list of SUBFIELD structures
// -	gs_char: can be TRUE or FALSE indicating whether the trailing
//		'GS' character is set or not INSIDE THE REAL BUFFER.
//////////////////////////////////////////////////////////////////////////
typedef struct field{
   char *id;
   unsigned int record_type;
   unsigned int field_int;
   int num_bytes;
   int num_subfields;
   int alloc_subfields;
   SUBFIELD **subfields;
   int gs_char;
} FIELD; /* subfields in fields */

//////////////////////////////////////////////////////////////////////////
////////RECORD STRUCTURE /////////////////////////////////////////////////
// This structure represents a record as it is defined inside the
// standard.
// It contains a list of FIELD and carries information about the size
// of the record INSIDE THE	REAL BUFFER.
// MEMBERS:
// -	type: an integer representing the record type.
// -	total_bytes: reserved for internal control. Normally should be
//		equal to "num_bytes".
// -	num_bytes: the size in bytes of the record INSIDE THE REAL
//		BUFFER,	including any trailing 'FS' separator.
// -	num_fields: number of fields contained in the field list. 
// -	alloc_fields: number of FIELD structures allocated.
// -	fields: list of FIELD structures
// -	fs_char: can be TRUE or FALSE indicating whether the trailing
//		'FS' character is set or not INSIDE THE REAL BUFFER.
//////////////////////////////////////////////////////////////////////////
typedef struct record{
   unsigned int type;
   int total_bytes;
   int num_bytes;
   int num_fields;
   int alloc_fields;
   FIELD **fields;
   int fs_char;
} RECORD; /* fields in records */

//////////////////////////////////////////////////////////////////////////
////////ANSI_NIST STRUCTURE //////////////////////////////////////////////
// This structure represents a record as it is defined inside the
// standard.
// It contains a list of RECORD and carries information about the size
// of the REAL BUFFER.
// MEMBERS:
// -	version: an integer representing the version of the standard used
//		to codify this buffer. Use the values defined below.
// -	num_bytes: the size in bytes of the REAL BUFFER.
// -	num_records: number of records contained in the records list. 
// -	alloc_records: number of RECORD structures allocated.
// -	records: list of RECORD structures.
//////////////////////////////////////////////////////////////////////////
typedef struct ansi_nist{
   unsigned int version;
   int num_bytes;
   int num_records;
   int alloc_records;
   RECORD **records;
} ANSI_NIST; /* records in ANSI_NIST file */

//////////////////////////////////////////////////////////////////////////
////////RECORD_LIST STRUCTURE ////////////////////////////////////////////
// This structure represents a simple record list.
//
// MEMBERS:
// -	num_records: number of records contained in the records list.
// -	records: list of RECORD structures.
//////////////////////////////////////////////////////////////////////////
typedef struct record_list{
	int num_records;
	RECORD **records;
} RECORD_LIST; /* simple record list */

/***********************************************************************/
/***********************************************************************/
/***********************************************************************/
/* DATA STRUCTURES FOR RECORD-14 as defined in AN2007 standard */
/*
*
* The types defined below represent new fields collecting data about:
* -	finger segmentation
* -	finger NFIQ quality
* -	missing fingers 
* -	finger quality other than NFIQ quality
*
* These data are collected into new fields added in the type-14 record
* following the AN2007 (document "ANSI/NIST-ITL 1-2007")
*
*/
/***********************************************************************/
/***********************************************************************/
/***********************************************************************/

//////////////////////////////////////////////////////////////////////////
////////FING_SEGMENT STRUCTURE ///////////////////////////////////////////
// This structure collects data about a single finger bounding boxes
// within a slap. Data inside this structure, taken together, will
// constitute a subfield into the SEG field as defined into the AN2007
// standard, while taken one by one, constitute an item into this
// subfield.
//
// MEMBERS:
// -	fing_id: represent the finger ID as choosen from Table 12
//		of the AN2007. Its value should be chosen within the finger ID
//		values defined below.
// -	left_bound: X coordinate of the left boundary.
// -	right_bound: X coordinate of the right boundary.
// -	top_bound: Y coordinate of the top boundary.
// -	left_bound: Y coordinate of the bottom boundary.
//////////////////////////////////////////////////////////////////////////
typedef struct finger_segment{
	char fing_id[3];		// one of the below specified finger IDs
	char left_bound[5];		// X coordinate of the left boundary
	char right_bound[5];	// X coordinate of the right boundary
	char top_bound[5];		// Y coordinate of the top boundary
	char bottom_bound[5];	// Y coordinate of the bottom boundary
}FING_SEGMENT; /* contains informations about boundaries for segmentation*/

//////////////////////////////////////////////////////////////////////////
////////FOUR_FING_BOUND STRUCTURE ////////////////////////////////////////
// This structure represents the SEG field as defined into the AN2007
// standard. Each item of this list, that is a FING_SEGMENT structure,
// collects bounding box info for each finger inside a slap.
//////////////////////////////////////////////////////////////////////////
typedef struct four_fing_bound{
	int num_fingers;		// number of fingers; from 1 to 4
	FING_SEGMENT **fingers;	// array of fingers
}FOUR_FING_BOUND; /* list of finger segmentation coordinates */

//////////////////////////////////////////////////////////////////////////
////////FING_QUALITY STRUCTURE ///////////////////////////////////////////
// This structure collects data about NFIQ quality for a given finger.
// Data inside this structure, taken together, will
// constitute a subfield into the NQM field as defined into the AN2007
// standard, while taken one by one, constitute an item into this
// subfield.
//
// MEMBERS:
// -	fing_id: represent the finger ID as choosen from Table 12
//		of the AN2007. Its value should be chosen within the finger ID
//		values defined below.
// -	quality: finger quality as specified by NFIQ.
//////////////////////////////////////////////////////////////////////////
typedef struct finger_quality{
	char fing_id[3];		// one of the below specified finger IDs
	char quality[2];		// finger quality as specified by NFIQ
}FING_QUALITY; /* contains informations about NFIQ Quality */

//////////////////////////////////////////////////////////////////////////
////////FOUR_FING_QUALITY STRUCTURE //////////////////////////////////////
// This structure represents the NQM field as defined into the AN2007
// standard. Each item of this list, that is a FING_QUALITY structure,
// collects NFIQ quality informations about each specified finger.
//////////////////////////////////////////////////////////////////////////
typedef struct four_fing_quality{
	int num_fingers;		// number of fingers; from 1 to 4
	FING_QUALITY **fingers;	// array of fingers
}FOUR_FING_QUALITY; /* list of finger qualities */

//////////////////////////////////////////////////////////////////////////
////////FING_AMP STRUCTURE ///////////////////////////////////////////////
// This structure collects data about missing fingers.
// Data inside this structure, taken together, will
// constitute a subfield into the AMP field (for type-14 and type-2)
// as defined into the AN2007 standard, while taken one by one, constitute
// an item into this subfield.
//
// MEMBERS:
// -	fing_id: represent the finger ID as choosen from Table 12
//		of the AN2007. Its value should be chosen within the finger ID
//		values defined below.
// -	amp_code: reason of finger unavailability as choosen from AMPCD
//		(amputation code) of the AN2007. Its value should be chosen within
//		the amputation code values defined below.
//////////////////////////////////////////////////////////////////////////
typedef struct finger_amp{
	char fing_id[3];		// one of the below specified finger IDs
	char amp_code[3];		// amputation code (see below defined codes)
}FING_AMP; /* contains informations about amputated or bandaged fingers */

//////////////////////////////////////////////////////////////////////////
////////FING_AMP_LIST STRUCTURE //////////////////////////////////////////
// This structure represents the AMP field as defined into the AN2007
// standard. Each item of this list, that is a FING_AMP structure,
// collects unavailability reason information about each specified finger.
//////////////////////////////////////////////////////////////////////////
typedef struct fing_amp_list{
	int num_fingers;		// number of fingers; from 1 to 4
	FING_AMP ** fingers;	// array of fingers
}FING_AMP_LIST; /* list of finger amputation informations */

//////////////////////////////////////////////////////////////////////////
////////FING_QUALITY STRUCTURE ///////////////////////////////////////////
// This structure collects data about the fingerprint quality metric 
// for a given finger.
// Data inside this structure, taken together, will
// constitute a subfield into the FQM or SQM fields as defined into the
// AN2007 standard, while taken one by one, constitute an item into this
// subfield.
//
// MEMBERS:
// -	fing_id: represent the finger ID as choosen from Table 12
//		of the AN2007. Its value should be chosen within the finger ID
//		values defined below.
// -	quality: finger quality ranging from 0 (worst) to 100 (best).
// -	Vendor_ID: ID of the vendor of the quality algorithm used
//		to calculate the quality score. This Id is assigned by IBIA
//		and expressed as four ASCII characters, representing the 
//		number in HEXADECIMAL notation. Specify a 4 digits number
//		(for example "0040" for 0x0040).
// -	Algorithm_ID: specify a numeric product code assigned by the
//		vendor of the quality algorithm.  This member contains the ASCII
//		representation of the integer product code and should be within
//		the range 1 to 65535.
//////////////////////////////////////////////////////////////////////////
typedef struct alternate_quality{
	char fing_id[3];		// one of the below specified finger IDs
	char quality[4];		// 0...100;255 = failure;254 = no calculation
	char Vendor_ID[6];		// IBIA vendor_ID code (www.ibia.org)
	char Algorithm_ID[6];	// IBIA Algorithm_ID code (www.ibia.org):0...65535
}ALT_QUAL; /* contains informations about alternate and segmentation quality*/

//////////////////////////////////////////////////////////////////////////
////////ALT_QUAL_LIST STRUCTURE //////////////////////////////////////////
// This structure can represent both the FQM and SQM fields as defined 
// into the AN2007 standard. Each item of this list, that is an ALT_QUAL
// structure, collects quality information about each specified finger.
//////////////////////////////////////////////////////////////////////////
typedef struct alt_qual_list{
	int num_fingers;		// number of fingers; from 1 to 4
	ALT_QUAL ** fingers;	// array of fingers
}ALT_QUAL_LIST; /* list of finger alternate quality values */

// Ver 1.2.0.0: added quality structure for palmprint
//////////////////////////////////////////////////////////////////////////
//////// PALM_QUALITY STRUCTURE //////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
// This structure collects data about the palmprint quality metric 
// for a given palm.
// Data inside this structure, taken together, will
// constitute a subfield into the PQM field as defined into the
// AN2007 standard, while taken one by one, constitute an item into this
// subfield.
//
// MEMBERS:
// -	PalmID: represent the finger ID as choosen from Table 12
//		of the AN2007. Its value should be chosen within the finger ID
//		values defined below.
// -	quality: finger quality ranging from 0 (worst) to 100 (best).
// -	Vendor_ID: ID of the vendor of the quality algorithm used
//		to calculate the quality score. This Id is assigned by IBIA
//		and expressed as four ASCII characters, representing the 
//		number in HEXADECIMAL notation. Specify a 4 digits number
//		(for example "0040" for 0x0040).
// -	Algorithm_ID: specify a numeric product code assigned by the
//		vendor of the quality algorithm.  This member contains the ASCII
//		representation of the integer product code and should be within
//		the range 1 to 65535.
//////////////////////////////////////////////////////////////////////////
typedef struct palm_quality{
	char PalmID[3];			// one of the below specified finger PalmIDs
	char Quality[4];		// 0...100;255 = failure;254 = no calculation
	char Vendor_ID[6];		// IBIA vendor_ID code (www.ibia.org)
	char Algorithm_ID[6];	// IBIA Algorithm_ID code (www.ibia.org):0...65535
}PALM_QUAL; /* contains informations about alternate and segmentation quality*/

//////////////////////////////////////////////////////////////////////////
//////// PALM_QUAL_LIST STRUCTURE ////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
// This structure represents the PQM field as defined 
// into the AN2007 standard. Each item of this list, that is a PALM_QUAL
// structure, collects quality information for the given palm.
//////////////////////////////////////////////////////////////////////////
typedef struct palm_qual_list{
	int AlgorhythmsNumber;		// number of algorhythms; can be any
	PALM_QUAL ** Algorhythms;	// array of PALM_QUAL
}PALM_QUAL_LIST; /* list of palm quality structures */
// end Ver 1.2.0.0: added quality structure for palmprint
#pragma pack(pop)
/***********************************************************************/
/* COMMON ERROR DEFINITIONS */
/***********************************************************************/
// ERRORS
#define AN2K_DLL_NO_ERROR				0
#define AN2K_DLL_BAD_PARAMETER			-1
#define AN2K_DLL_MEMORY					-2
#define AN2K_DLL_FIELD_NOT_FOUND		-3
#define AN2K_DLL_GENERIC				-4
#define AN2K_DLL_RECORD_NOT_FOUND		-5
// 1.4.0.0 Added subfield and item not found for GB_AN_Engine
#define AN2K_DLL_SUBFIELD_NOT_FOUND		-6
// 1.4.0.0 Added subfield and item not found for GB_AN_Engine
#define AN2K_DLL_ITEM_NOT_FOUND			-7

/***********************************************************************/
/* VERSION DEFINITIONS */
/***********************************************************************/
// before AN2K
#define VERSION_0200      200
// AN2K version
#define VERSION_0300      300
// AN2007 version
#define VERSION_0400      400
// 1.4.0.0 Added version 500 for AN2011
#define VERSION_0500      500

/***********************************************************************/
/* PIXEL DEPTH DEFINITIONS */
/***********************************************************************/
// 8 bpp grayscale
#define PIXEL_DEPTH_8BPP				8

/***********************************************************************/
/* COMPRESSION ALGORITHMS DEFINITIONS */
// Values preceded by a "BIN_" are to be used with type-4 records
/***********************************************************************/
// no compression
#define COMP_NONE						"NONE"
#define BIN_COMP_NONE					"0"
// wsq 2.0
#define COMP_WSQ						"WSQ20"
#define BIN_COMP_WSQ					"1"
// jpegb (lossy)
#define COMP_JPEGB						"JPEGB"
#define BIN_COMP_JPEGB					"2"
// jpegl (lossless)
#define COMP_JPEGL						"JPEGL"
#define BIN_COMP_JPEGL					"3"
// jpeg2000 (lossy)
#define JPG2K_COMP						"JP2"
#define BIN_COMP_JPEG2K					"4"
// jpeg2000 (lossless)
#define JPG2KL_COMP						"JP2L"
#define BIN_COMP_JPEG2KL				"5"
// png
#define PNG_COMP						"PNG"
#define BIN_COMP_PNG					"6"

/***********************************************************************/
/* IMPRESSION CODE DEFINITIONS */
/***********************************************************************/
#define ETFS_14_LIVE_SCAN_PLAN			0
#define ETFS_14_LIVE_SCAN_ROLLED		1
#define ETFS_14_NON_LIVE_SCAN_PLAN		2
#define ETFS_14_NON_LIVE_SCAN_ROLLED	3
#define ETFS_14_LATENT_IMPRESSION		4
#define ETFS_14_LATENT_TRACING			5
#define ETFS_14_LATENT_PHOTO			6
#define ETFS_14_LATENT_LIFT				7

/***********************************************************************/
/* FINGER POSITION DEFINITIONS */
/***********************************************************************/
#define ETFS_14_FGP_UNKNOWN				"0"
#define ETFS_14_FGP_RIGHT_THUMB			"1"
#define ETFS_14_FGP_RIGHT_INDEX			"2"
#define ETFS_14_FGP_RIGHT_MIDDLE		"3"
#define ETFS_14_FGP_RIGHT_RING			"4"
#define ETFS_14_FGP_RIGHT_LITTLE		"5"
#define ETFS_14_FGP_LEFT_THUMB			"6"
#define ETFS_14_FGP_LEFT_INDEX			"7"
#define ETFS_14_FGP_LEFT_MIDDLE			"8"
#define ETFS_14_FGP_LEFT_RING			"9"
#define ETFS_14_FGP_LEFT_LITTLE			"10"
#define ETFS_14_FGP_PLAIN_RIGHT_THUMB	"11"
#define ETFS_14_FGP_PLAIN_LEFT_THUMB	"12"
#define ETFS_14_FGP_RIGHT_4_FING		"13"
#define ETFS_14_FGP_LEFT_4_FING			"14"
#define ETFS_14_FGP_2_THUMBS			"15"

// Ver 1.2.0.0
/***********************************************************************/
/* PALM POSITION DEFINITIONS */
/***********************************************************************/
#define ETFS_15_PLP_UNKNOWN				"20"
#define ETFS_15_PLP_RIGTH_FULL_PALM		"21"
#define ETFS_15_PLP_RIGHT_WRITER_PALM	"22"
#define ETFS_15_PLP_LEFT_FULL_PALM		"23"
#define ETFS_15_PLP_LEFT_WRITER_PALM	"24"
#define ETFS_15_PLP_RIGTH_LOWER_PALM	"25"
#define ETFS_15_PLP_RIGHT_UPPER_PALM	"26"
#define ETFS_15_PLP_LEFT_LOWER_PALM		"27"
#define ETFS_15_PLP_LEFT_UPPER_PALM		"28"
#define ETFS_15_PLP_RIGHT_OTHER			"29"
#define ETFS_15_PLP_LEFT_OTHER			"30"
#define ETFS_15_PLP_RIGHT_INTERDIGITAL	"31"
#define ETFS_15_PLP_RIGHT_THENAR		"32"
#define ETFS_15_PLP_RIGHT_HYPOTHENAR	"33"
#define ETFS_15_PLP_LEFT_INTERDIGITAL	"34"
#define ETFS_15_PLP_LEFT_THENAR			"35"
#define ETFS_15_PLP_LEFT_HYPOTHENAR		"36"
// end Ver 1.2.0.0

/***********************************************************************/
/* AMPUTATION CODE DEFINITIONS */
/***********************************************************************/
#define AMPUTATION						"XX"
#define UNABLE_TO_PRINT					"UP"

/***********************************************************************/
/* ETFS type-4 parameters */
/***********************************************************************/
#define ETFS_TYPE4_MAX_RES_VAL			1 /*max resolution used for current fingerprint*/
#define ETFS_TYPE4_MIN_RES_VAL			0 /*min resolution used for current fingerprint*/

/***********************************************************************/
/* ANSI-NIST 2007 records */
/***********************************************************************/
#define TYPE_1_ID						1
#define TYPE_2_ID						2
#define TYPE_3_ID						3
#define TYPE_4_ID						4
#define TYPE_5_ID						5
#define TYPE_6_ID						6
#define TYPE_7_ID						7
#define TYPE_8_ID						8
#define TYPE_9_ID						9
#define TYPE_10_ID						10
#define TYPE_11_ID						11
#define TYPE_12_ID						12
#define TYPE_13_ID						13
#define TYPE_14_ID						14
#define TYPE_15_ID						15
#define TYPE_16_ID						16
#define TYPE_17_ID						17
#define TYPE_18_ID						18 // 1.4.0.0: AN2011
#define TYPE_19_ID						19 // 1.4.0.0: AN2011
#define TYPE_20_ID						20 // 1.4.0.0: AN2011
#define TYPE_21_ID						21 // 1.4.0.0: AN2011
#define TYPE_98_ID						98 // 1.4.0.0: AN2011
#define TYPE_99_ID						99

/***********************************************************************/
/* ROUTINES */
/***********************************************************************/
#ifndef DllExport_an2k
#ifdef AN2K_COMPILE_FOR_DLL
#define DllExport_an2k __declspec( dllexport )
#else
#define DllExport_an2k
#endif
#endif // DllExport_an2k

#ifdef __cplusplus
extern "C"
{
#endif



/***********************************************************************/
/* TYPE1_ETFS.C : ETFS Type-1 ROUTINES */
/***********************************************************************
************************************************************************
#cat: ANDREA
#cat: Create_ANSI_NIST - Creates an ANSI NIST structure adding a type-1
#cat: record from the input data.
#cat: Parameters indicated as mandatory must be entered.
#cat: If an invalid value is entered for the "Ver" parameter, it will be
#cat: set to VERSION_0400 (ANSI-NIST 2007 version).

	Input:
		Version		- ANSI/NIST file version. Choose between values defined above
		TypeOfTransaction		- Type of transaction string. MANDATORY
		Priority		- Priority value. If entered, must be between 0 and 9. Enter
				any value < 0 if not required
		DestinationAgency		- Destination Agency ID string. MANDATORY
		OriginatingAgency		- Originating Agency ID string. MANDATORY
		TransactionControlID		- Transaction control ID string. MANDATORY
		TransactionControlReference		- Transaction control reference ID string. OPTIONAL. Enter NULL
				if not required
		NominalScanningResolution		- Nominal Scanning resolution. Its format must be XX.XX. MANDATORY
		NominalTransmitResolution		- Nominal Transmitting resolution. Its format must be XX.XX.
				MANDATORY
		DomainNameID		- Domain name ID string. OPTIONAL. Enter NULL if not required
   Output:
		oansi_nist	- points to resulting structure
   Return Code:
		AN2K_DLL_NO_ERROR	- successful completion
		Negative			- system error
************************************************************************/
DllExport_an2k int Create_ANSI_NIST(
	ANSI_NIST**	oansi_nist,
	int			Version,
	char*		TypeOfTransaction,
	int			Priority, 
	char*		DestinationAgency, 
	char*		OriginatingAgency, 
	char*		TransactionControlID,
	char*		TransactionControlReference,
	float		NominalScanningResolution,
	float		NominalTransmitResolution,
	char*		DomainNameID
);

// Ver 1.1.0.0: added next function
DllExport_an2k int GetEFTSType1_Info(
	RECORD*		record,
	int*		Version,				// version
	char*		TransactionType,		// pass a char[12]
	int*		Priority,				// priority
	char**		pDestinationAgency,		// destination agency: no limits on length of this field, so
									// a developer needs next field:
	int*		DestAgencyLen,			// DestinationAgency string length
	char**		pOriginatingAgency,		// originating agency: no limits on length of this field, so
									// a developer needs next field:
	int*		OriginatingAgencyLen,	// OriginatingAgency string length
	char**		pTransContrNum,			// transaction control number: no limits on length of this field, so
									// a developer needs next field:
	int*		TransContrNumLen,		// TransContrNum string length 
	char**		pTransContrRef,			// transaction control reference: no limits on length of this field, so
									// a developer needs next field:
	int*		TransContrRefLen,		// TransContrRef string length
	float*		NominalScanRes,			// Nominal Scanning Resolution
	float*		NominalTXRes,			// Nominal Transmitting Resolution
	char**		pDomain,				// Domain: no limits on length of this field, so
									// a developer needs next field:
	int*		DomainLen				// Domain string length
);
// end Ver 1.1.0.0

// Ver 1.3.0.0
/*************************************************************************
**************************************************************************
#cat: ANDREA
#cat: UpdateETFStype_1 - Updates an ETFS Type-1 tagged record
#cat: with new values passed as parameters.

   Input:
	NominalScanningResolution - Nominal Scanning resolution. Its format 
			must be XX.XX. MANDATORY
	NominalTransmitResolution - Nominal Transmitting resolution. Its format
			must be XX.XX. MANDATORY

   Output:
	orecord		- points to resulting Type-1 record
   Return Code:
	Zero		- successful completion
	Negative	- system error
**************************************************************************/
DllExport_an2k int UpdateETFStype_1(RECORD *orecord,
	float		NominalScanningResolution,
	float		NominalTransmitResolution);
// end Ver 1.3.0.0

/***********************************************************************/
/***********************************************************************/
/* TYPE14_ETFS.C : ETFS Type-14 ROUTINES */
/*************************************************************************
**************************************************************************
#cat: ANDREA
#cat: image2ETFStype_14 - Takes a fingerprint image pixmap (possibly
#cat: compressed) and converts it into an ETFS Type-14 tagged
#cat: field image record.

	Input:
		FingerprintImage			- fingerprint image data
		FPImageSize			- length in bytes of the image data buffer
		FPImageWidth				- width (in pixels) of the fingerprint image
		FPImageHeight				- height (in pixels) of the fingerprint image
		FPImagePixelDepth				- pixel depth (in bits) of the fingerprint image: enter 8 always
		ScanResolutionPPMM			- the scan resolution (in pixels/mm) of the fingerprint image
		CompressionType		- compression type string. Look at values defined above
		ImpressionType			- the fingerprint image's impression type (IMP)
		ImageSource		- string identifying the originating source of the image its max len can be MAX_SOURCE_STR_LEN						
		Comment			- optional string ('\0' terminated) to enter a comment.
						its max len can be MAX_COMMENT_LEN
						Enter NULL if not required
		FingerPosition	- string indicating the finger position as defined in
						appendix N of ETFS 7.1. Look at values defined above
		boundaries		- boundaries of each of four (or two thumbs) fingers
						in a slap. Enter NULL if not required
		qualities		- qualities of each of four or two thumbs or 
						single fingers in a slap. Enter NULL if not required
		AmputationList		- list of missing fingers and amputation codes. Enter NULL
						if not required
		segqual			- Segmentation quality for fingers. Enter NULL
						if not required
		altqual			- Alternate quality for fingers. Enter NULL
						if not required
   Output:
		orecord			- points to resulting Type-14 tagged field image record
   Return Code:
		AN2K_DLL_NO_ERROR			- successful completion
		Negative		- system error
**************************************************************************/
// Ver 1.1.0.0 : added check for source_str and comment fields max length
#define MAX_SOURCE_STR_LEN 43
#define MAX_COMMENT_LEN 135
// end Ver 1.1.0.0
DllExport_an2k int image2ETFStype_14(
	RECORD**			orecord,
	unsigned char*		FingerprintImage,
	const int			FPImageSize,
	const int			FPImageWidth,
	const int			FPImageHeight,
	const int			FPImagePixelDepth,
	const double		ScanResolutionPPMM,
	char*				CompressionType,
	const int			ImpressionType,
	char*				ImageSource,
	char*				Comment,
	char*				FingerPosition,
	FING_AMP_LIST*		AmputationList,
	FOUR_FING_BOUND*	boundaries,
	FOUR_FING_QUALITY*	qualities,
	ALT_QUAL_LIST*		segqual,
	ALT_QUAL_LIST*		altqual
);

/***********************************************************************/
/*************************************************************************
**************************************************************************
#cat: ANDREA
#cat: UpdateETFStype_14 - Updates an ETFS Type-14 tagged record
#cat: with new values passed as parameters.

Input:
	FingerprintImage			- fingerprint image data. To avoid image
					loading, set this parameter to NULL
	FPImageSize			- length in bytes of the image data buffer. To avoid image
					loading, set this parameter to 0
	FPImageWidth				- width (in pixels) of the fingerprint image: if no update
					is needed for this field, set it to 0
	FPImageHeight				- height (in pixels) of the fingerprint image: if no update
					is needed for this field, set it to 0
	ScanResolutionPPMM			- the scan resolution (in pixels/mm) of the fingerprint
					image: if no update	is needed for this field, set it to 0
	boundaries		- boundaries of each of four (or two thumbs) fingers
					in a slap. Enter NULL if update is not required
	qualities		- qualities of each of four or two thumbs or 
					single fingers in a slap. Enter NULL if update is not required
	amp_list		- list of missing fingers and amputation codes. Enter NULL
					if update is not required
	segqual			- Segmentation quality for fingers. Enter NULL
					if update is not required
	altqual			- Alternate quality for fingers. Enter NULL
					if update is not required
Output:
	orecord			- points to resulting Type-14 tagged field image record
Return Code:
	AN2K_DLL_NO_ERROR			- successful completion
	Negative		- system error
**************************************************************************/
DllExport_an2k int UpdateETFStype_14(
	RECORD*			orecord,
	unsigned char*		FingerprintImage,
	const int			FPImageSize,
	const int			FPImageWidth,
	const int			FPImageHeight,
	const double		ScanResolutionPPMM,
	FING_AMP_LIST*		amp_list,
	FOUR_FING_BOUND*	boundaries,
	FOUR_FING_QUALITY*	qualities,
	ALT_QUAL_LIST*		segqual,
	ALT_QUAL_LIST*		altqual
);

// ver 2.0.0.2
/***********************************************************************/
/*************************************************************************
**************************************************************************
UpdateType1415ScaleUnits
Description:
	Updates the SLC field into a Type-14 or Type-15 tagged record with 
	the new value passed as parameter.

Parameters:
	<recordToUpdate>:	record to be updated
	<newScale>:	new scale unit to be set. Can be in the range [0,2]

Return Code:
	see return codes
**************************************************************************/
DllExport_an2k int UpdateType1415ScaleUnits(RECORD *recordToUpdate, int newScale);
// end ver 2.0.0.2

// Ver 1.1.0.0: added next function
DllExport_an2k int Get_EFTStype14_Info(
	RECORD*					record,				// input record
	unsigned char**			ImagePtr,			// image frame: allocated by the library
	int*					ImageSize,			// allocated memory for image frame (can be different from w x h)
	int*					w,
	int*					h,					// image dimensions
	int*					PixelDepth,			// bits per pixel
	double*					Resolution_ppmm,	// resolution in pixel per mm
	char*					ImageCompression,	// image compression: see definitions: pass a char[10]
	int*					ImageImpression,	// image impression type (live-scan, roll, etc, see definitions)
	char*					source_str,			// string identifying the originating source of the image,
												// pass a char[44]
	char*					comment,			// comment, pass a char[136]
	char					finger_position_list[6][3],	// list of strings indicating the finger position.
														// See definitions, pass a list of char[3]
														// composed of 6 items
	FING_AMP_LIST**			pamp_list,			// list of missing fingers and amputation codes: allocated 
												// by the library
	FOUR_FING_BOUND**		pboundaries,		// boundaries of each of four (or two thumbs) fingers in a slap
	FOUR_FING_QUALITY**		pqualities,			// list of NFIQ quality of each of four or two thumbs or single 
												// fingers in a slap
	ALT_QUAL_LIST**			psegqual,			// list of segmentation quality for fingers
	ALT_QUAL_LIST**			paltqual			// list of quality of each of four or two thumbs or single fingers in 
													// a slap calculated with an algorithm different from NFIQ
);
// end Ver 1.1.0.0
/***********************************************************************/
/* TYPE4_ETFS.C : ETFS Type-4 ROUTINES */
/*************************************************************************
**************************************************************************
#cat: ANDREA
#cat: image2ETFStype_4 - Takes a fingerprint image pixmap (possibly
#cat: compressed) and converts it into an ETFS Type-14 tagged
#cat: field image record.

	Input:
		FingerprintImage		- fingerprint image data
		FPImageSize				- length in bytes of the image data buffer
		FPImageWidth			- width (in pixels) of the fingerprint image
		FPImageHeight			- height (in pixels) of the fingerprint image
		ScanResolutionID		- the scan resolution of the fingerprint image.
								Can be 0 (min res) or 1 (max res)
		CompressionType			- binary compression type (look at values in an2k_dll.h)
		ImpressionType			- the fingerprint image's impression type (look at values
								defined above)
		FingerPosition			- finger position (look at values defined above). Even if
								up to 6 positions could be entered for this field, for
								our purpose (tenprint capture for BioAPI 2.0), finger
								position must be well specified, so just one value can be
								entered
	Output:
		orecord					- points to resulting Type-14 tagged field image record
	Return Code:
		Zero					- successful completion
		Negative				- system error
**************************************************************************/
DllExport_an2k int image2ETFStype_4(
	RECORD**			orecord,
	unsigned char*		FingerprintImage,
	const int			FPImageSize,
	const int			ImpressionType,
	const int			FingerPosition,
	const int			ScanResolutionID,
	const int			FPImageWidth,
	const int			FPImageHeight,
	const char			*CompressionType
);
/***********************************************************************/
/*************************************************************************
**************************************************************************
#cat: ANDREA
#cat: UpdateETFStype_4 - Updates an ETFS Type-4 binary record
#cat: with new values passed as parameters.

Input:
	FingerprintImage		- fingerprint image data.  To avoid image
				loading, set this parameter to NULL
	FPImageSize		- length in bytes of the image data buffer. To avoid image
				loading, set this parameter to 0
	FPImageWidth		- width (in pixels) of the fingerprint image: if no update
				is needed for this field, set it to 0
	FPImageHeight		- height (in pixels) of the fingerprint image: if no update
				is needed for this field, set it to 0
	iisr		- the scan resolution of the fingerprint image: if no update
				is needed for this field, set it to a value < 0
Output:
	orecord		- points to resulting Type-4 tagged field image record
Return Code:
	Zero		- successful completion
	Negative	- system error
**************************************************************************/
DllExport_an2k int UpdateETFStype_4(
	RECORD*			orecord, 
	unsigned char*	FingerprintImage,
	const int		FPImageSize,
	const int		ScanResolutionID,
	const int		FPImageWidth,
	const int		FPImageHeight
);

// Ver 1.1.0.0: added next function
DllExport_an2k int Get_EFTStype4_Info(
	RECORD*			record,					// input record
	unsigned char**	ImagePtr,				// image frame: allocated by the library
	int*			ImageSize,				// allocated memory for image frame (can be different from w x h)
	int*			ImageImpression,		// image impression type (live-scan, roll, etc, see definitions)
	int				finger_position[6],		// int indicating the finger position. See definitions
	int*			ImageResolution,		// the scan resolution of the fingerprint image. See definitions
	int*			w,
	int*			h,						// image dimensions
	char*			ImageCompression		// image compression: see definitions: pass a char[10]
);
// end Ver 1.1.0.0
/***********************************************************************/
/* TYPE2_ETFS.C : ETFS Type-2 ROUTINES */
/*************************************************************************
**************************************************************************
#cat: ANDREA
#cat: CreateETFStype_2 - Creates a type-2 ETFS record from the input data.

	Input:
		Mak			- String describing equipment vendor. Enter NULL
					if not required
		Modl		- String describing equipment model. Enter NULL
					if not required
		SerNo		- String describing equipment serial number. Enter NULL
					if not required
		amp_list	- list of missing fingers and amputation codes. Enter NULL
					if not required
	Output:
		orecord		- points to resulting Type-14 tagged field image record
	Return Code:
		Zero		- successful completion
		Negative	- system error
**************************************************************************/
// Ver 1.1.0.0: added length check
#define MAX_MAK_LEN 25
#define MAX_MODL_LEN 25
#define MAX_SERNO_LEN 50
// end Ver 1.1.0.0
DllExport_an2k int CreateETFStype_2(
	RECORD**		orecord,
	char*			Mak,
	char*			Modl,
	char*			SerNo,
	FING_AMP_LIST*	amp_list
);
/***********************************************************************/
/*************************************************************************
**************************************************************************
#cat: ANDREA
#cat: UpdateETFStype_2 - Updates an ETFS Type-2 tagged record
#cat: with new values passed as parameters.

   Input:
	Mak			- String describing equipment vendor. Enter NULL
				if not required
	Modl		- String describing equipment model. Enter NULL
				if update is not required
	SerNo		- String describing equipment serial number. Enter NULL
				if update is not required
	amp_list	- list of missing fingers and amputation codes. Enter NULL
				if update is not required
   Output:
	orecord		- points to resulting Type-2 record
   Return Code:
	Zero		- successful completion
	Negative	- system error
**************************************************************************/
DllExport_an2k int UpdateETFStype_2(
	RECORD*			orecord,
	char*			Mak,
	char*			Modl,
	char*			SerNo,
	FING_AMP_LIST*	amp_list
);

// Ver 1.1.0.0: added next function
DllExport_an2k int Get_EFTStype2_Info(
	RECORD*			record,
	char*			Mak, // MAX 25
	char*			Modl, // MAX 25
	char*			SerNo, // MAX 50
	FING_AMP_LIST**	pamp_list
);
// end Ver 1.1.0.0

// Ver 1.2.0.0: added Type-15 Record management
/*************************************************************************
**************************************************************************
#cat: ANDREA
#cat: image2ETFStype_15 - Takes a palmprint image pixmap (possibly
#cat: compressed) and converts it into an ETFS Type-15 tagged
#cat: field image record.

Input:
	PPImage			- palm image data. To avoid image
					loading, set this parameter to NULL
	PPImageSize		- length in bytes of the image data buffer. To avoid image
					loading, set this parameter to 0
	PPImageWidth	- width (in pixels) of the palm image
	PPImageHeight	- height (in pixels) of the palm image
	BitsPerPixel	- pixel depth (in bits) of the palm image (8)
	PPImageResolutionPPMM	- the scan resolution (in pixels/mm) of the palm
							image
	CompressionType	- compression type string
	ImpressionType	- the fingerprin image's impression type (IMP)
	ImageSource		- string identifying the originating source of the image
	Comment			- optional string ('\0' terminated) to enter a comment.
					Enter NULL if not required
	PalmPosition	- string indicating the palm position as defined in
					Table 35 of ETFS 7.1. Look at values in an2k_dll.h
	PalmQualitiesList	- list of qualities attributed to the palm. Add a
						PALM_QUAL item inside this list for eache algorhythm
						used. Enter NULL if not required
Output:
	orecord			- points to resulting Type-15 tagged field image record
Return Code:
	AN2K_DLL_NO_ERROR	- successful completion
	Negative		- system error
**************************************************************************/
DllExport_an2k int image2ETFStype_15(
	RECORD**			orecord,
	unsigned char*		PPImage,
	const int			PPImageSize,
	const int			PPImageWidth, 
	const int			PPImageHeight, 
	const int			BitsPerPixel,
	const double		PPImageResolutionPPMM,
	char*				CompressionType,
	const int			ImpressionType,
	char*				ImageSource,
	char*				Comment,
	char*				PalmPosition,
	PALM_QUAL_LIST*		PalmQualitiesList
);

/*************************************************************************
**************************************************************************
#cat: ANDREA
#cat: UpdateETFStype_15 - Updates an ETFS Type-15 tagged record
#cat: with new values passed as parameters.

Input:
	PPImage			- palmprint image data. To avoid image
					loading, set this parameter to NULL
	PPImageSize		- length in bytes of the image data buffer. To avoid image
					loading, set this parameter to 0
	PPImageWidth	- width (in pixels) of the palmprint image: if no update
					is needed for this field, set it to 0
	PPImageHeight	- height (in pixels) of the palmprint image: if no update
					is needed for this field, set it to 0
	PPImageResolutionPPMM	- the scan resolution (in pixels/mm) of the palmprint
							image: if no update	is needed for this field,
							set it to 0
	PalmQualitiesList	- list of qualities attributed to the palm. Add a
						PALM_QUAL item inside this list for eache algorhythm
						used. Enter NULL if not required
Output:
	orecord			- points to resulting Type-15 tagged field image record
Return Code:
	AN2K_DLL_NO_ERROR	- successful completion
	Negative		- system error
**************************************************************************/
DllExport_an2k int UpdateETFStype_15(
	RECORD*			orecord,
	unsigned char*	PPImage,
	const int		PPImageSize,
	const int		PPImageWidth, 
	const int		PPImageHeight,
	const double	PPImageResolutionPPMM,
	PALM_QUAL_LIST*	PalmQualitiesList
);

/*************************************************************************
**************************************************************************
#cat: ANDREA
#cat: Get_EFTStype15_Info - Takes info from a record 15. For each info,
	pass NULL if info is not needed

Input:
	
Output:
	
Return Code:
	AN2K_DLL_NO_ERROR	- successful completion
	Negative			- system error
**************************************************************************/
DllExport_an2k int Get_EFTStype15_Info(
	RECORD*				record,				// input record
	unsigned char**		ImagePtr,			// image frame: allocated by the library
	int*				ImageSize,			// allocated memory for image frame (can be different from w x h)
	int*				w,
	int*				h,					// image dimensions
	int*				BitsPerPixel,		// bits per pixel
	double*				Resolution_ppmm,	// resolution in pixel per mm
	char*				ImageCompression,	// image compression: see definitions: pass a char[10]
	int*				ImageImpression,	// image impression type (live-scan, roll, etc, see definitions)
	char*				source_str,			// string identifying the originating source of the image, pass a char[44]
	char*				comment,			// comment, pass a char[136]
	char*				PalmPosition,		// string indicating the palm position. See definitions
	PALM_QUAL_LIST**	PalmQualitiesList	// list of quality values calculated by using several algorhythms
);
// end Ver 1.2.0.0

/***********************************************************************/
/* insert_ETFS.C : ETFS insert ROUTINES */
/***********************************************************************
************************************************************************
#cat: ANDREA
#cat: insert_ANSI_NIST_record_ETFS - Routine used to insert the logical
#cat:              record passed as parameter into an ANSI/NIST file
#cat:              structure.  This routine also updates the CNT field of
#cat:              the Type-1 record to accurately reflect the insertion.
#cat:              Note - In this routine, an IDC is assigned based on the 
#cat:              record's position in the ANSI/NIST structure.

	Input:
		RecordIndex	- index of logical record to be inserted
		RecordToBeInserted	- record to be inserted
		ansi_nist	- ANSI/NIST file structure to be modified
	Output:
		ansi_nist	- the modified ANSI/NIST file structure
	Return Code:
		Zero		- successful completion
		Negative	- system error
************************************************************************/
DllExport_an2k int insert_ANSI_NIST_record_ETFS(
	const int	RecordIndex,
	RECORD*		RecordToBeInserted,
	ANSI_NIST*	ansi_nist
);
/***********************************************************************/

// Ver 1.3.0.0: adding possibility to delete a record from an AN2K file
/***********************************************************************
************************************************************************
#cat: ANDREA
#cat: delete_ANSI_NIST_record - Routine used to delete a logical record
#cat:			   from	an ANSI/NIST file structure.  This routine also
#cat:			   updates the CNT field of the Type-1 record to 
#cat:			   accurately reflect the deletion.
#cat:              Note - In this routine, an IDC is assigned based on the 
#cat:              record's position in the ANSI/NIST structure.

	Input:
		RecordIndex	- index of logical record to be deleted
		ansi_nist	- ANSI/NIST file structure to be modified
	Output:
		ansi_nist	- the modified ANSI/NIST file structure
	Return Code:
		Zero		- successful completion
		Negative	- system error
************************************************************************/
DllExport_an2k int delete_ANSI_NIST_record(
	const int	RecordIndex,
	ANSI_NIST*	ansi_nist
);
// end Ver 1.3.0.0

/***********************************************************************/
/* ParseBuffer.C : ROUTINES for parsing a buffer into an ANSI/NIST structure*/
/***********************************************************************
************************************************************************
#cat: ANDREA
#cat: read_ANSI_NIST_buffer - Routine reads the contents of the specified
#cat: ANSI/NIST buffer into memory.

	Input:
		buffer		- input buffer
	Output:
		oansi_nist	- points to resulting structure
	Return Code:
		Zero		- successful completion
		Negative	- system error
************************************************************************/
DllExport_an2k int read_ANSI_NIST_buffer(
	unsigned char*	buffer,
	int				bufLen,
	ANSI_NIST**		oansi_nist
);
/***********************************************************************/
/***********************************************************************
************************************************************************
#cat: ANDREA
#cat: search_ANSI_NIST_record - Routine searchs for all the records whose
#cat: type is specified as parameter inside an ANSI_NIST structure.

	Input:
		ansi_nist	- ANSI_NIST structure
		RecordType	- type of the wanted record
	Output:
		oRecList	- points to resulting structure
	Return Code:
		Zero		- successful completion
		Negative	- system error or record not found
************************************************************************/
DllExport_an2k int search_ANSI_NIST_record(
	RECORD_LIST**	oRecList,
	unsigned int	RecordType,
	ANSI_NIST*		ansi_nist
);
/***********************************************************************/

/***********************************************************************/
/* WriteBuffer.C : ROUTINES for writing an ANSI/NIST structure into
a buffer*/
/***********************************************************************
************************************************************************
#cat: ANDREA
#cat: write_ANSI_NIST_2_buffer - Routine writes the contents of an ANSI/NIST
#cat: structure to a buffer.

	Input:
		ansi_nist	- allocated and filled ANSI/NIST structure
	Output:
		pOutBuffer	- buffer pointer to be written to
		bufLen		- number of characters written to buffer
	Return Code:
		Zero		- successful completion
		Negative	- system error
************************************************************************/
DllExport_an2k int write_ANSI_NIST_2_buffer(
	unsigned char **pOutBuffer,
	int *bufLen,
	const ANSI_NIST *ansi_nist
);
/***********************************************************************/

/***********************************************************************/
/* alloc.C : ROUTINES for freeing several ANSI/NIST structures*/
/***********************************************************************
************************************************************************
#cat: ANDREA
#cat: free_ANSI_NIST - Routine for freeing an ANSI/NIST structure.

	Input:
		ansi_nist	- allocated filled ANSI/NIST structure to be freed
************************************************************************/
DllExport_an2k void free_ANSI_NIST(ANSI_NIST *ansi_nist);
/***********************************************************************/
/***********************************************************************
************************************************************************
#cat: ANDREA
#cat: free_ANSI_NIST_record - Routine for freeing a RECORD structure.

	Input:
		record	- allocated RECORD structure to be freed
************************************************************************/
DllExport_an2k void free_ANSI_NIST_record(RECORD *record);
/***********************************************************************
************************************************************************
#cat: ANDREA
#cat: free_ANSI_NIST_record_list - Routine for freeing a RECORD_LIST
#cat: structure.

	Input:
		record_list	- allocated RECORD_LIST structure to be freed
************************************************************************/
DllExport_an2k void free_ANSI_NIST_record_list(RECORD_LIST *record_list);
/***********************************************************************
************************************************************************
#cat ANDREA
#cat: free_FOUR_FING_BOUND - Deallocates a FOUR_FING_BOUND structure 
#cat                   and all of its allocated substructures.

   Input:
      boundaries  - points to the structure to be deallocated
************************************************************************/
DllExport_an2k void free_FOUR_FING_BOUND(FOUR_FING_BOUND *boundaries);
/***********************************************************************
************************************************************************
#cat ANDREA
#cat: free_FOUR_FING_QUALITY - Deallocates a FOUR_FING_QUALITY structure 
#cat                   and all of its allocated substructures.

   Input:
      qualities  - points to the structure to be deallocated
************************************************************************/
DllExport_an2k void free_FOUR_FING_QUALITY(FOUR_FING_QUALITY *qualities);
/***********************************************************************
************************************************************************
#cat: ANDREA
#cat: free_ANSI_NIST_amp_list - Routine for freeing a FING_AMP_LIST
#cat: structure.

	Input:
		amp_list	- allocated FING_AMP_LIST structure to be freed
************************************************************************/
DllExport_an2k void free_ANSI_NIST_amp_list(FING_AMP_LIST *amp_list);
/***********************************************************************
************************************************************************
#cat: ANDREA
#cat: free_ANSI_NIST_alt_qual_list - Routine for freeing a ALT_QUAL_LIST
#cat: structure.

	Input:
		alt_qual_list	- allocated ALT_QUAL_LIST structure to be freed
************************************************************************/
DllExport_an2k void free_ANSI_NIST_alt_qual_list(ALT_QUAL_LIST *alt_qual_list);

// Ver 1.2.0.0
/***********************************************************************
************************************************************************
#cat: ANDREA
#cat: free_ANSI_NIST_palm_qual_list - Routine for freeing a PALM_QUAL_LIST
#cat: structure.

	Input:
		palm_qual_list	- allocated PALM_QUAL_LIST structure to be freed
************************************************************************/
DllExport_an2k void free_ANSI_NIST_palm_qual_list(PALM_QUAL_LIST *palm_qual_list);

/***********************************************************************
************************************************************************
#cat: ANDREA
#cat: free_ANSI_NIST_buffer - Routine for freeing the buffer allocated
#cat: into write_ANSI_NIST_buffer.

	Input:
		buffer	- allocated buffer to be freed
************************************************************************/
// Ver 1.1.0.0: prototype changed
DllExport_an2k void free_ANSI_NIST_buffer(void *buffer);
// end Ver 1.1.0.0

/***********************************************************************
************************************************************************
#cat: ANDREA
#cat: rec_copy - Routine for copying data (not pointers) from a record
#cat: to another.

	Input:
		src			- record to copy data from
	Output:
		dst			- record to copy data to
	Return Code:
		Zero		- successful completion
		Negative	- system error
************************************************************************/
// Ver 1.3.0.1: added rec_copy prototype
DllExport_an2k int rec_copy(RECORD *dst,RECORD *src);
// End Ver 1.3.0.1
#ifdef __cplusplus
}
#endif

#endif /* !_AN2k_H */