#ifndef  GBFRSW_H
#define  GBFRSW_H

//**********************************************************************************************************
//**********************************************************************************************************
//								GBFRSW ARCHITECTURE, FINGERPRINT CODE TYPES 
//**********************************************************************************************************
//**********************************************************************************************************
// GBFRSW.DLL contains all necessary functions to perform fingerprint image recognition in both verification 
//   and identification scenarios. The input parameters of functions allow tuning the operational performance 
//   of GBFRSW in accordance with specific application requirements and limits. 
// An essential feature of GBFRSW - it is an asymmetric system from point of view of fingerprint code size. 
//
// SAMPLE CODE
// In particular, the fingerprint code initially generated from fingeprint image (by GBFRSW_Coding() function)
//   has a structure optimized for faster matching operation and contains complete detailed fingerprint data 
//   that leads to bigger code size. However, as far as SAMPLE code is not intended for storage and is located 
//   exclusively in RAM memory, bigger size does not produce any inconvinience, but instead provides 
//   better structure and complete detailed data for faster and more reliable matching.
//
// TEMPLATE CODE
// Oppositely, a TEMPLATE code that is intended for storage in databases as a fingerprint reference, has a
//   structure optimized to occupy less space due to the fact that it contains less detailed fingerprint 
//   information. This information is also structured for faster matching operation. The fact that TEMPLATE
//   code contains less detailed information, does not degrade overall recognition algorithm performance, as
//   far as TEMPLATE code is always compared against SAMPLE code containing complete detailed fingerprint 
//   data. TEMPLATE code is generated from SAMPLE code by GBFRSW_Enroll() function. 
//
// PACKED SAMPLE CODE
// For applications where matching operation is executed on remote server, a PACKED SAMPLE code will
//   be usefull. PACKED SAMPLE code is produced by GBFRSW_Convert() from SAMPLE code and contains all the 
//   data of SAMPLE code in compressed form. Its structure is optimized to occupy less space. Therefore, 
//   before transferring SAMPLE code by some communication channel to remote server (especially, if the 
//   communication channel is of low bandwidth), it is recommended to convert it to PACKED SAMPLE code. 
//   It is necessary to note, that PACKED SAMPLE can not be directly used by GBFRSW_Match(), so, use again 
//   GBFRSW_Convert() to reconvert it back to SAMPLE code before matching.
//
// COMPACT TEMPLATE CODE
// For applications where fingerprint storage is particularly limited (smart card memory, for example), 
//   COMPACT TEMPLATE code can be usefull. COMPACT TEMPLATE code is produced by GBFRSW_Convert() from TEMPLATE 
//   code and contains all the data of TEMPLATE code in compressed form. Its structure is optimized to occupy 
//   as little space as possible. Therefore, before storing TEMPLATE code into a space-limited storage, it is 
//   recommended to convert it to COMPACT TEMPLATE code. It is necessary to note, that COMPACT TEMPLATE can not 
//   be directly used by GBFRSW_Match(), so, use again GBFRSW_Convert() to reconvert it back to TEMPLATE code 
//   before matching.
//
// MEMORY CONFIGURATIONS:
// There are two pre-defined memory configurations of GBFRSW.dll module:
//   1. Light (recommended for applications with limited memory resources such as embedded systems):  
//		max supported image: 504 x 504 pixels; max MemoryBufferSize is about 1.5 MBytes
//   2. Normal (recommended for normal applications where there are no particular memory limitations): 
//		max supported image: 1008 x 1008 pixels, max MemoryBufferSize is about 16 MBytes
// To check memory configuration of the given GBFRSW.DLL module use GBFRSW_GetVersionInfo() function from 
//   programming environment, or look to DLL module properties (tab "Version",field "Description").
// It is important to note that Light memory configuration is a little faster then Full configuration,
//   the difference is especially notable on embedded CPUs with small cash size.
//
// SUPPORTED PLATFORMS:
// Currently, GBFRSW.DLL is available for Win32/64 platforms. Linux/Windows Mobile platforms support is 
//   planned for future versions.
//
// TESTING RESULTS:
// See GBFRSW_TEST_RESULTS.XLS for details.


//**********************************************************************************************************
//**********************************************************************************************************
#ifdef _WIN64
#define GBFRSW_API //VER1341
#else
#define GBFRSW_API __declspec(dllexport)
#endif

//**********************************************************************************************************
//**********************************************************************************************************
//												 RETURN CODE DEFINITION
//**********************************************************************************************************
//**********************************************************************************************************
#define GBFRSW_ERROR	 0 // an error occured, call GBFRSW_GetLastError() to obtain specific details
#define GBFRSW_SUCCESS	 1 // an operation completed successfully


//**********************************************************************************************************
//**********************************************************************************************************
//												ERROR CODE DEFINITION
//**********************************************************************************************************
//**********************************************************************************************************
// returned by GBFRSW_GetLastError()
#define GBFRSW_ERROR_UNKNOWN						0
#define GBFRSW_ERROR_NOT_ENOUGH_MEMORY				1
#define GBFRSW_ERROR_IMAGE_ABSENT					2
#define GBFRSW_ERROR_SIZE_NOT_SUPPORTED				3
#define GBFRSW_ERROR_GBFRSW_CODE_TYPE_NOT_SUPPORTED	4
#define GBFRSW_ERROR_CODE_VERSION_NOT_SUPPORTED		5
#define GBFRSW_ERROR_POINTER_NOT_ALIGNED			6
#define GBFRSW_ERROR_CODE_CORRUPTED					7 // wrong CRC checksum 
#define GBFRSW_ERROR_LOW_QUALITY_FINGERPRINT		8 // fail due to extremely low-quality fingerprint data
#define GBFRSW_ERROR_LICENSE_NOT_GRANTED			9 // call GBFRSW_GetLicenseInfo() for more information
#define GBFRSW_ERROR_INVALID_PARAMETER              10 // VER1310
#define GBFRSW_ERROR_UNKNOWN_FORMAT                 11 // VER1310
#define GBFRSW_ERROR_COORD_COMPACT_FORMAT           12 // VER1310 - for  <ISO/IEC 19794-2> card compact format, 
													   // if minutiae coordinate value is higher than 502 (in pixel) 


//**********************************************************************************************************
//**********************************************************************************************************
//											ERROR DETAIL CODE DEFINITION
//**********************************************************************************************************
//**********************************************************************************************************
// returned by GBFRSW_GetLastError()
#define GBFRSW_DETAIL_UNKNOWN						0
#define GBFRSW_DETAIL_IMAGE							1
#define GBFRSW_DETAIL_SAMPLE_CODE					2
#define GBFRSW_DETAIL_SAMPLE_CODE_2					3
#define GBFRSW_DETAIL_SAMPLE_CODE_3					4
#define GBFRSW_DETAIL_TEMPLATE_CODE					5
#define GBFRSW_DETAIL_SAMPLE_CODE_PACKED			6
#define GBFRSW_DETAIL_TEMPLATE_CODE_COMPACT			7



//**********************************************************************************************************
//**********************************************************************************************************
//										FRSW SERVICE FUNCTION DEFINITION
//**********************************************************************************************************
//**********************************************************************************************************

//----------------------------------------------------------------------------------------------------------
extern "C" GBFRSW_API int	__stdcall 
GBFRSW_GetVersionInfo(unsigned char  *VersionField1,
					  unsigned char  *VersionField2,
					  unsigned char  *VersionField3,
					  unsigned char  *VersionField4,
					  unsigned int   *MaxImageSizeX,
					  unsigned int   *MaxImageSizeY,
					  unsigned int   *MinImageSizeX,
					  unsigned int   *MinImageSizeY,
					  unsigned int   RequestedOperations,
					  unsigned int   *MemoryBufferSize);
//----------------------------------------------------------------------------------------------------------
// The function performs the following actions:
//	- obtains GBFRSW.dll module information such as version, the maximum supported fingerprint image size, 
//	    an amount of memory required for proper functioning for different operations.
//  - checks a license. The license check is performed only if GBFRSW_MEMORY_REQUEST_CODING bit is set,
//      that is, just GBFRSW_Coding() operation is licensed. All other operations are of free use.
//      If GBFRSW_Coding() operation is requested and license is not granted, the function returns 
//	    GBFRSW_ERROR_LICENSE_NOT_GRANTED and successive calls of GBFRSW_Coding() will fail. 
//		For more information see section GBFRSW LICENSE CHECK below.
// Enough to call GBFRSW_GetVersionInfo() once after loading DLL, and if license is successfully granted, 
//	 it remains valid all the time GBFRSW.DLL remains loaded. 
// It is strongly recommended for application developer to call this function before any Green Bit USB 
//   fingerprint scanner function calls in cases when an application uses both 


//----------------------------------------------------------------------------------------------------------
// Return value: 
//   See section RETURN CODE DEFINITION
//----------------------------------------------------------------------------------------------------------
// Parameters:
//
//	<VersionField1>		- returns the first field of module version
//	<VersionField2>		- returns the second field of module version
//	<VersionField3>		- returns the third field of module version
//	<VersionField4>		- returns the fourth field of module version
//
//	<*MaxImageSizeX>	- returns maximal supported fingerprint image width
//	<*MaxImageSizeY>	- returns maximal supported fingerprint image height
//
//	<*MinImageSizeX>	- returns minimal supported fingerprint image width
//	<*MinImageSizeY>	- returns minimal supported fingerprint image height
//
//	<RequestedOperations> - set the specific bits corresponding to the operations need to be executed 
//							(or GBFRSW_MEMORY_REQUEST_ALL for all operations) in order
//							to be informed about amount of memory required for proper functioning
//							(returned by <*MemoryBufferSize> parameter).
#define GBFRSW_MEMORY_REQUEST_CODING   0x00000001 // GBFRSW_Coding()
#define GBFRSW_MEMORY_REQUEST_ENROLL   0x00000002 // GBFRSW_Enroll()
#define GBFRSW_MEMORY_REQUEST_MATCH    0x00000004 // GBFRSW_Match()
#define GBFRSW_MEMORY_REQUEST_CONVERT  0x00000008 // GBFRSW_Convert()
#define GBFRSW_MEMORY_REQUEST_CORRECT  0x00000010 // GBFRSW_Correct()
#define GBFRSW_MEMORY_REQUEST_ALL      0x80000000 // all operations
//
//	<*MemoryBufferSize> - returns amount of memory required for proper functioning depending on 
//						  <RequestedOperations> setting
//----------------------------------------------------------------------------------------------------------



//----------------------------------------------------------------------------------------------------------
extern "C" GBFRSW_API int	__stdcall 
GBFRSW_CodeMaxSize(unsigned int   ImageSizeX,
				   unsigned int   ImageSizeY,
				   unsigned int   *SampleCodeSize,
				   unsigned int   *PackedSampleCodeSize,
				   unsigned int   *TemplateCodeSize,
				   unsigned int   *CompactTemplateCodeSize);
//----------------------------------------------------------------------------------------------------------
// The function returns the biggest possible fingerprint code size of the specific type depending on fingerprint 
// image dimensions. Normally, the bigger fingerprint image, the bigger code size.
// Use this function for fingerprint code memory allocation specifying the maximal fingerprint image dimensions
// expected in application. 
//----------------------------------------------------------------------------------------------------------
// Return value: 
//   See section RETURN CODE DEFINITION
//----------------------------------------------------------------------------------------------------------
// Parameters:
//
//	<ImageSizeX>	- input image width, in pixels; must be not higher than GBFRSW_GetVersionInfo.MaxImageSizeX
//
//	<ImageSizeY>	- input image height, in pixels; must be not higher than GBFRSW_GetVersionInfo.MaxImageSizeY
//
//  <*SampleCodeSize> - if it is not NULL, returns the biggest possible size of SAMPLE code
//
//  <*PackedSampleCodeSize> - if it is not NULL, returns the biggest possible size of PACKED SAMPLE code
//
//  <*TemplateCodeSize> - if it is not NULL, returns the biggest possible size of TEMPLATE code
//
//  <*CompactTemplateCodeSize> - if it is not NULL, returns the biggest possible size of COMPACT TEMPLATE code
//----------------------------------------------------------------------------------------------------------



//----------------------------------------------------------------------------------------------------------
extern "C" GBFRSW_API void  __stdcall 
GBFRSW_GetLastError(int *ErrorCode,
					int *Detail);
//----------------------------------------------------------------------------------------------------------
// The function returns the error code and the error details of the last error happened. 
// Call this function to get more information after one of GBFRSW functions returns GBFRSW_ERROR.
//----------------------------------------------------------------------------------------------------------
// Return value: 
//   See section RETURN CODE DEFINITION
//----------------------------------------------------------------------------------------------------------
// Parameters:
//
//	<*ErrorCode>	- see section ERROR CODE DEFINITION 
//
//	<*Detail>		- see section ERROR DETAIL CODE DEFINITION
//----------------------------------------------------------------------------------------------------------



//----------------------------------------------------------------------------------------------------------
extern "C" GBFRSW_API int	__stdcall 
GBFRSW_Init(unsigned char *MemoryBuffer,
		    unsigned int  MemoryBufferSize);
//----------------------------------------------------------------------------------------------------------
// The function initializes GBFRSW system. 
// Call this function before using any of GBFRSW CORE FUNCTION. An application is responsible 
// to allocate memory buffer of the required size obtained previously by GBFRSW_GetVersionInfo() function.
//----------------------------------------------------------------------------------------------------------
// Return value: 
//   See section RETURN CODE DEFINITION
//----------------------------------------------------------------------------------------------------------
// Parameters:
//
//	<*MemoryBuffer>			- pointer to memory buffer of <MemoryBufferSize> size 
//
//	<MemoryBufferSize>		- specifies memory buffer size
//----------------------------------------------------------------------------------------------------------



//**********************************************************************************************************
//**********************************************************************************************************
//									GBFRSW CORE FUNCTION DEFINITION
//**********************************************************************************************************
//**********************************************************************************************************

//----------------------------------------------------------------------------------------------------------
extern "C" GBFRSW_API int	__stdcall 
GBFRSW_Coding(unsigned int    ImageSizeX,
		      unsigned int    ImageSizeY,
			  unsigned char   *ImageBuffer, 
			  unsigned char   ImageFlags,
			  unsigned char   Options,
			  unsigned char   *SampleCode); 
//----------------------------------------------------------------------------------------------------------
// The function extracts identification information from the provided fingerprint image and converts this 
// information into SAMPLE code to be used further in matching operation. 
//----------------------------------------------------------------------------------------------------------
// Return value: 
//   See section RETURN CODE DEFINITION
//----------------------------------------------------------------------------------------------------------
// Parameters:
//
//	<ImageSizeX>  - input image width, in pixels; must be not higher than GBFRSW_GetVersionInfo.MaxImageSizeX
//
//	<ImageSizeY>  - input image height, in pixels; must be not higher than GBFRSW_GetVersionInfo.MaxImageSizeY
//
//  <*InputImage> - pointer to buffer of <ImageSizeX>*<ImageSizeY> size allocated by application containing  
//					input fingerprint image; expected fingerprint image characteristics are:				
//						- 8-bit grayscale, 
//						- ridges are dark (lower grayscales) and valleys are light (higher grayscale values)
//						- 500 dpi resolution or near
//						- no limitations for fingerprint orientation or displacement
//
//  <ImageFlags>  - this information helps GBFRSW system to tune different parameters of fingerprint 
//					encoding/matching depending on how fingerprint image was produced. It is recommended 
//					to set this flags when image origin is known. Take care to don't set mutually exclusive 
//					flags (like GBFRSW_FLAT_IMAGE and GBFRSW_ROLLED_IMAGE), otherwise, both will be ignored.
//					<ImageFlags> content will be stored inside fingerprint code header - see GBFRSW_CODE_HEADER
//					structure description.
#define GBFRSW_LIVESCAN_IMAGE   0x01 
#define GBFRSW_CARDSCAN_IMAGE   0x02
#define GBFRSW_FLAT_IMAGE		0x04   
#define GBFRSW_ROLLED_IMAGE	    0x08 
#define GBFRSW_LATENT_IMAGE	    0x10 
#define GBFRSW_FRAGMENT_IMAGE	0x20 // used in cases when fingerprint imaging technology limitations can lead to 
									 // potentially high image fragmentation: ratio of common overlaped region to 
									 // total summary area of template and sample fingerprints is low (< 50%) 
									 // example: small-area sensors like AUTHENTEC, etc.
//
// <Options>      - specify additional optional operations that can be usefull in different applications:
#define GBFRSW_OPTION_FINGERPRINT_PATTERN_CHECK	 0x01 // used to activate fingerprint pattern check capability 
												      // (helps to reject images containing patterns of 
												      // non-fingerprint origin: gloves, tissues, etc.). If option 
												      // is activated (bit set), GBFRSW_Coding() returns GBFRSW_ERROR
													  // (GBFRSW_ERROR_LOW_QUALITY_FINGERPRINT error code)
													  // in cases when non-fingerprint pattern is detected.
//
//  <*SampleCode> - pointer to buffer of <SampleCodeSize> size (obtained by GBFRSW_GetVersionInfo()) allocated 
//				    by application. If the function returns GBFRSW_SUCCESS, this buffer contains generated SAMPLE code.
//----------------------------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------------------------
extern "C" GBFRSW_API int	__stdcall 
GBFRSW_Enroll(unsigned char   *SampleCode,
			  unsigned char   *TemplateCode);
//----------------------------------------------------------------------------------------------------------
// The function transforms SAMPLE code to TEMPLATE code. This operation is not reversable.
//----------------------------------------------------------------------------------------------------------
// Return value: 
//   See section RETURN CODE DEFINITION
//----------------------------------------------------------------------------------------------------------
// Parameters:
//
//  <*SampleCode>	- pointer to buffer of <SampleCodeSize> size (obtained by GBFRSW_GetVersionInfo()) 
//					  allocated by application containing SAMPLE code.
//
//  <*TemplateCode>	- pointer to buffer of <TemplateCodeSize> size (obtained by GBFRSW_GetVersionInfo()) 
//					  allocated by application.  If the function returns GBFRSW_SUCCESS, this buffer contains 
//					  generated TEMPLATE code.
//----------------------------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------------------------
extern "C" GBFRSW_API int	__stdcall 
GBFRSW_Match(  unsigned char		*SampleCode,
			   unsigned char		*TemplateCode,
			   unsigned char		SpeedVsPrecisionTradeoff,
			   unsigned char		UnmatchedDataFactor,
			   unsigned char		Options,
			   int					MaxRotationAngleDegree,       
			   float				*MatchingScore);
//----------------------------------------------------------------------------------------------------------
// The function matches SAMPLE code against TEMPLATE code. 
//----------------------------------------------------------------------------------------------------------
// Return value: 
//   See section RETURN CODE DEFINITION
//----------------------------------------------------------------------------------------------------------
// Parameters:
//
//  <*SampleCode>	- pointer to buffer of <SampleCodeSize> size (obtained by GBFRSW_GetVersionInfo()) 
//					  allocated by application containing SAMPLE code.
//
//  <*TemplateCode>	- pointer to buffer of <TemplateCodeSize> size (obtained by GBFRSW_GetVersionInfo()) 
//					  allocated by application.  If the function returns GBFRSW_SUCCESS, this buffer contains 
//					  generated TEMPLATE code.
//
//  <SpeedVsPrecisionTradeoff> - this parameter gives possibility for application to tune so-called 
//								 "Speed Versus Precision" tradeoff. Normally, the longer time is allowed for 
//								 match operation, the more precise matching results in terms EER/FRR/FAR can be 
//								 achieved. Also, this parameter can be particularly usefull for applications
//								 with low CPU resource (like embedded platforms) with critical response time 
//								 limitations. The functioning of this tradeoff can be compared with gear-box
//								 (transmission) of a car. The lower gear, the higher power (precision) and the 
//								 smaller speed; and vice versa. It is important to note, that every next step
//								 of tradeoff value just insignificantly deteriorates matching precision and 
//								 much more stronger improves matching speed.
//								 For PC-based verification applications, GBFRSW_TRADEOFF_ROBUST is recommended.
//								 If GBFRSW_TRADEOFF_ROBUST can not be used in a certain application due to 
//								 CPU resource limitations or unacceptable response time, higher tradeoff value 
//								 can be selected to satisfy specific timing requirements.
#define GBFRSW_TRADEOFF_ROBUST				2 // Robust Verification
#define GBFRSW_TRADEOFF_NORMAL				3 // Normal Verification
#define GBFRSW_TRADEOFF_FAST				4 // Fast Verification/Identification
//
//  <Options>       - specify additional options for matching:
#define GBFRSW_MATCH_OPTION_FRAGMENT_IMAGE  0x01 // set this mask to inform matching algorithm about highly-possible 
												 // fingerprint fragmentation if by some reason it was not marked
												 // in SAMPLE/TEMPLATE fingerprint code at coding phase by using 
												 // GBFRSW_FRAGMENT_IMAGE image flag.
// <UnmatchedDataFactor> - specifies influence of unmatched data to matching score -  0(no influence), 255 (max influence);
//						   Recommended value: 
//									- 90 for verification matching scenario (1 to few), 
//									- 128 for identification matching scenario (1 to many)
//
// <MaxRotationAngle> -	specifies maximum allowable rotation angle (in degrees) between compared fingerprints:
//						   - 0 means that the rotation angle is not limited - no orientation limitations
//						   - minimal acceptible angle is 20 degress (-20;+20);
//						   - maximal acceptible angle is 180 degress (-180;+180) - no orientation limitations
//						   - the less the angle, the faster matching operation;
//						   - by practice, rotation angle of 40-60 degrees is recommended for most applications
//
// <*MatchingScore> -	if the function returns GBFRSW_SUCCESS, this parameter contains the measure of 
//						coincidence(matching) between SAMPLE and TEMPLATE codes. GBFRSW matching algorithm calculates 
//						the matching score in the way that allows to express decision threshold required by application 
//						in usual terms of FAR (False Acceptance Rate): 10^-N . 
//						The following formula can be used to verify that fingerprints are matched successfully:  
//											<MatchingScore>  >=  1 / <decision threshold (FAR)>;
//						Example1: if <MatchingScore> = 1000000 (10^6) and <decision threshold (FAR)> = 0.00001 (10^-5), 
//								  the fingeprints are matched:  10^6 >= 1/(10^-5), that is, 10^6 >= 10^5.
//						Example2: if <MatchingScore> = -10 (-10^1) and <decision threshold (FAR)> = 0.00001 (10^-5), 
//								  the fingeprints are NOT matched:  -10^1 < 1/(10^-5), that is, -10^1 < 10^5.
#define GBFRSW_MIN_MATCHING_SCORE  (-10000) 
#define GBFRSW_MAX_MATCHING_SCORE  (3.4E+36F)
//----------------------------------------------------------------------------------------------------------



//----------------------------------------------------------------------------------------------------------
extern "C" GBFRSW_API int	__stdcall 
GBFRSW_Convert(unsigned char *OriginalCodePtr,
			   unsigned char *ConvertedCodePtr);
//----------------------------------------------------------------------------------------------------------
// The function converts:
//		- SAMPLE code to PACKED SAMPLE code and vice versa. 
//		- TEMPLATE code to TEMPLATE COMPACT code and vice versa. 
// Conversion operations are reversable.
//----------------------------------------------------------------------------------------------------------
// Return value: 
//   See section RETURN CODE DEFINITION
//----------------------------------------------------------------------------------------------------------
// Parameters:
// <*OriginalCodePtr> - pointer to buffer allocated by application and containing one of the following code types:
//								- SAMPLE code
//								- PACKED SAMPLE code
//								- TEMPLATE code
//								- COMPACT TEMPLATE code
//						Depending on original code type, the function produces its corresponding alternative code:
//						SAMPLE <-> PACKED SAMPLE, TEMPLATE <-> COMPACT TEMPLATE.
// <*ConvertedCodePtr>- pointer to buffer of the corresponding size allocated by application. An application is 
//						responsible to supply correct buffer with the correct size depending on original code type
//						to be converted. If the function returns GBFRSW_SUCCESS, the buffer contains converted code.
//----------------------------------------------------------------------------------------------------------



//----------------------------------------------------------------------------------------------------------
// NOTE: GBFRSW_Correct() will be available in the next versions of GBFRSW.
//----------------------------------------------------------------------------------------------------------
extern "C" GBFRSW_API int	__stdcall 
GBFRSW_Correct(unsigned char  *SampleCode1,
			   unsigned char  *SampleCode2,
			   unsigned char  *SampleCode3,
			   unsigned char  *CorrectedSampleCode);
//----------------------------------------------------------------------------------------------------------
// The function attempts to generate higher-quality corrected SAMPLE code originating from
// initial SAMPLE codes. It can be used in applications where acqusition of the same fingerprint can 
// be repeated several times during as enrollment as verification/identification operations.
// The function is particularly usefull for low-quality fingerprints that in itself does not contain
// enough information to exceed application decision threshold for successfull match. 
//----------------------------------------------------------------------------------------------------------
// Return value: 
//   See section RETURN CODE DEFINITION
//----------------------------------------------------------------------------------------------------------
// Parameters:
//
//  <*SampleCode1>	- NULL or pointer to buffer of <SampleCodeSize> size (obtained by GBFRSW_GetVersionInfo()) 
//					  allocated by application containing SAMPLE code 1.
//
//  <*SampleCode2>	- NULL or pointer to buffer of <SampleCodeSize> size (obtained by GBFRSW_GetVersionInfo()) 
//					  allocated by application containing SAMPLE code 2.
//
//  <*SampleCode3>	- NULL or pointer to buffer of <SampleCodeSize> size (obtained by GBFRSW_GetVersionInfo()) 
//					  allocated by application containing SAMPLE code 3.
//
//  <*CorrectedSampleCode>	- pointer to buffer of <SampleCodeSize> size (obtained by GBFRSW_GetVersionInfo()) 
//							  allocated by application. If the function returns GBFRSW_SUCCESS, 
//							  the buffer contains corrected SAMPLE code.
//----------------------------------------------------------------------------------------------------------



//**********************************************************************************************************
//**********************************************************************************************************
//									CODE HEADER DEFINITION
//**********************************************************************************************************
//**********************************************************************************************************
// Every fingerprint code (SAMPLE/TEMPLATE/etc...) has the same header structure located at the beginning 
// of the code. Application can easily having the access to this header simply converting code buffer 
// pointer by the following cast (GBFRSW_CODE_HEADER *) to header pointer. 
typedef struct
{
	unsigned long             CRC;					// checksum, internally used to check the code integrity
	unsigned long             Size;					// size of fingerrpint code, in bytes, 
													//   including GBFRSW_CODE_HEADER size
	unsigned long             SizeAfterConversion;  // size of this fingerrpint code after Convert() 
													//    operation, in bytes, informative scope
	unsigned char             Type;					// fingerrpint code type, see definitions below
	unsigned char             Version;				// fingerrpint code version, numerical (1,2,3,..)
	unsigned char             ImageFlags;			// reports the value of <ImageFlags> parameter of 
													//   GBFRSW_Coding() function used for code generation
	unsigned char             Quality;				// fingerprint pattern quality, in range [0-100]: 
													//	 0 - lowest quality, 100-highest quality
	float		              InformationCapacity;	// reflectf informational capacity of fingerprint code;
													//   normally, in practice, this value denotes the highest 
													//   achievable matching score value for the given  
													//   fingerprint code; so, it could be used as a certain 
													//   "predictor" for further matching success,
													//   (see recommendations for use below). 
	
	unsigned short			  ImageSizeX;			// VER1310, width of original fingerprint image 
	unsigned short			  ImageSizeY;			// VER1310, height of original fingerprint image 

	unsigned char			  Reserve[4];			// reserved for future use, initialized by 0

	unsigned long             SizeConfirm;			// must contain the same value as <Size> field
}
GBFRSW_CODE_HEADER;  // 32 bytes 

// GBFRSW_CODE_HEADER.Type
#define   GBFRSW_CODE_TYPE_SAMPLE			1
#define   GBFRSW_CODE_TYPE_SAMPLE_PACKED	2
#define   GBFRSW_CODE_TYPE_TEMPLATE			3
#define   GBFRSW_CODE_TYPE_TEMPLATE_COMPACT	4

// GBFRSW_CODE_HEADER.ImageFlags  - see GBFRSW_Coding(ImageFlags)

// GBFRSW_CODE_HEADER.Quality:
#define	GBFRSW_FINGER_CODE_MAX_QUALITY		100
#define	GBFRSW_FINGER_CODE_MIN_QUALITY		0

// Code header - recommendations for use:
// Code header contains usefull information that can be checked by application to take decisions about the 
//  usability of the given fingerprint code under specific application conditions. 
//  In more details:
// 1. Compare value of <InformationCapacity> field to the verification/identification threshold used
//     in application for matching decision. 
//    For reliable enrollment it is strongly recommended to check that <InformationCapacity> value of TEMPLATE 
//     code is at least 100 times bigger than verification/identification threshold value used by application. 
//     It will generally ensure that the TEMPLATE code contains a certain reserve of information available 
//     for further reliable matching even in conditions when fingerprint data of low quality will be captured or 
//	   significant fragmentation (partial overlapping) between TEMPLATE and SAMPLE fingerprint will take place. 
//    By the same reason, in applications with remote matching operation, before sending PACKED SAMPLE code to 
//     the remote matching station, it is strongly recommended to check at local capturing station that 
//	   <InformationCapacity> value of SAMPLE code is at least 50-100 times bigger than verification/identification 
//	   threshold value used by application. 
//	  Also, for identification applications with big enough databases that require significant period of time to 
//	   complete idenfification process, SAMPLE code <InformationCapacity> value check is recommended before 
//     starting idenfification process to achieve higher rate of "first-pass" succeessful identifications.
//   In case when <InformationCapacity> value is not big enough, it is recommended to repeat fingerprint
//    capturing with the scope to increase the availble fingerprint information (and/or use GBFRSW_Correct() 
//	  function when it will be available in future GBFRSW versions).
// 2. Use <Quality> field value check to inform a user in case of low-quality fingerprint pattern (normally due 
//		to bad skin conditions or other reasons depending on fingerprint scanner principle). This allows to 
//      applications (where possible) to repeat fingerprint capturing giving informational feedback to a user 
//		with a scope to improve fingerprint data quality thus increasing the successful matching rate. 
//      Low-quality threshold is about 70: if <Quality> field value is lower than the threshold, capturing 
//		repeat is recommended.


//----------------------------------------------------------------------------------------------------------
// VER1310 
extern "C" GBFRSW_API int	__stdcall 
GBFRSW_GetCodeHeader(GBFRSW_CODE_HEADER *OutputHeader,
					  unsigned char  *InputCode);
//----------------------------------------------------------------------------------------------------------
// The function returns header information structure from a given fingerprint code 
//
//----------------------------------------------------------------------------------------------------------
// Return value: 
//   GBFRSW_ERROR:	one or both parameters are NULL pointers
//----------------------------------------------------------------------------------------------------------
// Parameters:
//
//	<Header>		- returns header information structure: must be allocated by developer
//	<SourceCode>	- Code where header information must be extracted from
//----------------------------------------------------------------------------------------------------------



//**********************************************************************************************************
//**********************************************************************************************************
//										GBFRSW LICENSE CHECK 
//**********************************************************************************************************
//**********************************************************************************************************
// A permission for GBFRSW.DLL legitimate usage can be obtained by two independent ways:
//
//---------------------------------------------------------------------------------------------------------
// 1. GBFRSW license based on Green Bit USB fingerprint scanner
//---------------------------------------------------------------------------------------------------------
//
//   GBFRSW license is supported by the following Green Bit fingerprint scanner models:
//		- DactyScan26(i)
//		- DactyScan40i
//		- DactyScan84(n)
//		- MultiScan500/MC500/MSC500
//		- MultiScan1000
//		- DactyScan84C - starting from GBFRSW VER1340
//		- MC517/MSC517 - starting from GBFRSW VER1340
//   GBFRSW license will be granted if at least one Green Bit scanner of supported model with activated 
//		GBFRSW license will be found connected on USB bus of PC. 
//	 GBFRSW license is controlled by the fourth position of scanner serial number string:
//		- if an alphabetical character <A,B,C,...,J> is present in the fourth position, GBFRSW license 
//		  will be granted
//		- if an alphabetical character <U,V,W,X,Y,Z,*,#,!,? > is present in the fourth position, GBFRSW license 
//		  will be granted - starting from GBFRSW VER1340 
//	    - VER1340 - otherwise, GBFRSW license will NOT be granted - for example, if an numerical character    
//		  <0,1,2,...,9> is present in the fourth position
//   Serial number examples: <D8RARP00035> - GBFRSW license will be granted
//						     <D8R1RP00035> - GBFRSW license will NOT be granted
//	 Practically, in this case no additional licensing procedures (like PC's HostID generation, 
//     license file transmission and so on...) are required from user part. 
//
//---------------------------------------------------------------------------------------------------------
// 2. GBFRSW license based on license file (PC-locked)
//---------------------------------------------------------------------------------------------------------
//
//   GBFRSW.DLL uses specially designed third-party software tools to protect company's copy-right and prevent 
//      illegitimate usage of GBFRSW.DLL copies. LM-X tool by X-FORMATION have been choosen for these purposes. 
//   In order to obtain a license file, the following three-steps procedure should be performed by a user:
//		1. A user run the special utility on its PC to obtain a PC's specific HostID value using  
//	       LM-X End-user Configuration Tool (lmxconfigtool.exe) or LM-X End-user utility (lmxendutil.exe).
//         For more information, please, refer to LM-X document <enduser.pdf>, sections:
//				- 3.3 LM-X End-user Configuration Tool 
//				- 3.2 LM-X End-user utility 
//		2. Send to Green Bit support service (support@greenbit.com):
//				- obtained HostID value 
//				- OrderID provided by Green Bit upon GBFRSW licenses purchase
//		3. Receive from Green Bit support service a license file and allocate it into the directory where 
//		   GBFRSW.DLL resided or in predefined license directories (see LM-X document <enduser.pdf>, 
//		   section <2.2 Search paths>).
//
// It is important to note, that trial version of GBFRSW.DLL is available and allows to estimate the performance 
//  and make different trials during 60 days from the first launch on a concrete PC. Trial GBFRSW.DLL version 
//  displays a dialog box where trial expiration date is shown. It is necessary to press OK button to continue.

//----------------------------------------------------------------------------------------------------------
typedef struct
{
	int			  Result;
	unsigned int  UsbDriverErrorCodeDetail; // to be interpreted if Result == GBSCAN_ERROR_USB_DRIVER
}
GBFRSW_LICENSE_USB_SCANNER_CHECK;  
//GBFRSW_LICENSE_USB_SCANNER.Result:
#define	GBSCAN_NO_ERROR						0  // GBFRSW license is granted
#define	GBSCAN_ERROR_USB_DRIVER				1  // An USB driver error happend during license check, 
											   //    see UsbDriverErrorCodeDetail for more detail
#define	GBSCAN_ERROR_USB_DEVICE_NOT_FOUND	2  // GBFRSW license is NOT granted
//----------------------------------------------------------------------------------------------------------
#define GBFRSW_LICENSE_ERROR_DETAIL_STRING_LENGTH 1000 
typedef struct
{
	int  ErrorCode; // see LM-X document <enduser.pdf>, section <9. Error messages> 
	int  Phase;     // internal phase number
	char ErrorDetailString[GBFRSW_LICENSE_ERROR_DETAIL_STRING_LENGTH]; // error string generated by LM-X 
}
GBFRSW_LICENSE_FILE_CHECK; 
//----------------------------------------------------------------------------------------------------------
extern "C" GBFRSW_API void	__stdcall 
GBFRSW_GetLicenseInfo(GBFRSW_LICENSE_USB_SCANNER_CHECK *ScannerBasedLicense,
					  GBFRSW_LICENSE_FILE_CHECK *FileBasedLicense);
//----------------------------------------------------------------------------------------------------------
// The function returns more detailed information describing the reasons of license check failure. 
//----------------------------------------------------------------------------------------------------------
// Return value: 
//   See section RETURN CODE DEFINITION
//----------------------------------------------------------------------------------------------------------
// Parameters:
// <*ScannerBasedLicense>	- if not NULL, returns more detailed scanner-based license check failure reason.
// <*FileBasedLicense>	- if not NULL, returns more detailed file-based license check failure reason.
//----------------------------------------------------------------------------------------------------------





//**********************************************************************************************************
//**********************************************************************************************************
//					Template code convertion to/from <ISO/IEC 19794-2> and <INCITS 378> 
//								Finger Minutiae Record (FMR) formats
//**********************************************************************************************************
//**********************************************************************************************************

// VER1310 
//------------------------------------------------------------------------------
extern "C" GBFRSW_API int __stdcall 
GBFRSW_ISO_GetMaxFMRLength(unsigned int   ImageSizeX,
						   unsigned int   ImageSizeY,
                           unsigned char  FormatType,
						   unsigned char  GBProprietaryData,
						   unsigned short *MaxFRMLength);
//------------------------------------------------------------------------------
// This function returns the maximum length of the Finger Minutiae Record for both 
//  the standards <ISO/IEC 19794-2> and <INCITS 378> depending on fingerprint 
//  image dimensions and proprietary data section format. 
// Normally, the bigger fingerprint image, the bigger FMR length.
// Use this function for memory allocation specifying the maximal fingerprint image dimensions
//  expected in application. 
//------------------------------------------------------------------------------
// Return value: 
//   See section RETURN CODE DEFINITION
//------------------------------------------------------------------------------
// Parameters:
//	<ImageSizeX>	- input image width, in pixels; must be not higher than GBFRSW_GetVersionInfo.MaxImageSizeX
//	<ImageSizeY>	- input image height, in pixels; must be not higher than GBFRSW_GetVersionInfo.MaxImageSizeY
//  <FormatType>    - specifies the desired format:
#define GBFRSW_ISO_FMR_FORMAT_ISO               1
//                      - FMR in <ISO/IEC 19794-2> format will be created;
#define GBFRSW_ISO_FMR_FORMAT_INCITS            2
//					    - FMR in <INCITS 378> format will be created;
#define GBFRSW_ISO_FMR_FORMAT_MINEX             3
//					    - FMR in <INCITS 378> format will be created, with special
//                        requirements for MINEX test; note that GBProprietaryData
//                        must be set to GBFRSW_ISO_FMR_GB_PROPR_DATA_NONE for this format type.
//	<GBProprietaryData>  - specifies if the generated FMR record should contain
//						   Green Bit proprietary data section and its format:		
#define GBFRSW_ISO_FMR_GB_PROPR_DATA_NONE    0   // no proprietary data section will be stored
											     // into FMR record; in this case the FMR will be the smallest
											     // but reverse conversion back to Green Bit Template code
											     // is not more possible; 
#define GBFRSW_ISO_FMR_GB_PROPR_DATA_FAST    1   // proprietary data section will be stored
											     // into FMR record in uncompressed format; in this case the FMR will
											     // be bigger and reverse conversion back to Green Bit Template code
											     // will be performed in a faster way;
#define GBFRSW_ISO_FMR_GB_PROPR_DATA_COMPRESSED 2// proprietary data section will be stored
											     // into FMR record in compresseed format; in this case the FMR will
											     // be smaller respect to uncompressed form and reverse conversion back
											     // to Green Bit Template code will require more time;
//  <*MaxFRMLength> - returns the biggest possible FMR length
//------------------------------------------------------------------------------


// VER1310
//------------------------------------------------------------------------------
extern "C" GBFRSW_API int __stdcall
GBFRSW_ISO_GBTemplateToFMR(unsigned char  *GBTemplate,
						   unsigned char  *FMR, 
						   unsigned short *FMRLen, 
						   unsigned char  FingerPos, 
						   unsigned char  FormatType,
						   unsigned char  GBProprietaryData);
//------------------------------------------------------------------------------
// The function converts Green Bit's proprietary Template Code 
//  to Finger Minutiae Record format defined by <ISO/IEC 19794-2> or <INCITS 378>. 
// Generated Finger Minutiae Record contains standard information section (defined 
//  by ISO/IEC or INCITS) and, optionally, proprietary data section containing Green Bit's 
//  proprietary template code information. For FMR recodrs with 
//  proprietary data section it is possible to perform reverse (ISO/IEC or INCITS to Green Bit)
//  code format conversion without loosing essential biometric information. Such a 
//  reversibility gives to an application a certain freedom to select a format (standard 
//  or proprietary) in which template codes can be stored in application database.
//------------------------------------------------------------------------------
// Return value: 
//   See section RETURN CODE DEFINITION
//------------------------------------------------------------------------------
// Parameters:
//    <*GBTemplate> - pointer to Green Bit proprietary Template Code 
//    <*FMR>		   - pointer to a buffer allocated by application for 
//					 Finger Minutiae Record storage (call GBFRSW_ISO_GetMaxFMRLength
//					 to obtain the required size)
//    <*FMRLen>	   - returns the effective length of FMR, in bytes (that is normally 
//					 less than max size)
//    <FingerPos>  - specifies a <finger position/name> in accordance with <ISO/IEC 19794-2>:
#define GBFRSW_ISO_FMR_FINGER_POS_UNKNOWN      0
#define GBFRSW_ISO_FMR_FINGER_POS_RIGHT_THUMB  1
#define GBFRSW_ISO_FMR_FINGER_POS_RIGHT_INDEX  2
#define GBFRSW_ISO_FMR_FINGER_POS_RIGHT_MIDDLE 3
#define GBFRSW_ISO_FMR_FINGER_POS_RIGHT_RING   4
#define GBFRSW_ISO_FMR_FINGER_POS_RIGHT_LITTLE 5
#define GBFRSW_ISO_FMR_FINGER_POS_LEFT_THUMB   6
#define GBFRSW_ISO_FMR_FINGER_POS_LEFT_INDEX   7
#define GBFRSW_ISO_FMR_FINGER_POS_LEFT_MIDDLE  8
#define GBFRSW_ISO_FMR_FINGER_POS_LEFT_RING    9
#define GBFRSW_ISO_FMR_FINGER_POS_LEFT_LITTLE  10
//
//     <FormatType> - specifies the desired format; see GBFRSW_ISO_GetMaxFMRLength()
//                    description, parameter <FormatType>
//	   <GBProprietaryData>  - specifies if the generated FMR record should contain
//							  Green Bit proprietary data section and its format:
//							  see GBFRSW_ISO_GetMaxFMRLength() description, parameter 
//							  <GBProprietaryData>.
//------------------------------------------------------------------------------



// VER1310 
//------------------------------------------------------------------------------
extern "C" GBFRSW_API int __stdcall 
GBFRSW_ISO_FMRToGBTemplate(unsigned char *FMR,
						   unsigned char *GBTemplate, 
						   unsigned char *FingerPos, 
						   unsigned char  FormatType);
//------------------------------------------------------------------------------
// Converts Finger Minutiae Record to Green Bit proprietary Template Code.
// If Finger Minutiae Record previously generated by GBFRSW_ISO_GBTemplateToFMR function 
//  contains Green Bit's proprietary data section it is possible 
//  to perform reverse (ISO/IEC or INCITS to Green Bit) code format conversion without 
//  loosing essential biometric information. Such a reversibility gives to an application 
//  a certain freedom to select a format (standard or proprietary) in which template codes 
//  can be stored in application database.
//------------------------------------------------------------------------------
// Return value: 
//   See section RETURN CODE DEFINITION
//------------------------------------------------------------------------------
// Parameters:
//    <*FMR>	    - pointer to buffer containing Finger Minutiae Record
//    <*GBTemplate> - pointer to a buffer allocated by application for
//					  Green Bit proprietary Template Code storage
//					  (call GBFRSW_CodeMaxSize to obtain the required size)
//    <*FingerPos>  - returns <finger position/name> read from the FMR in accordance 
//					  with <ISO/IEC 19794-2> (make reference to GBFRSW_ISO_GBTemplateToFMR
//					  function description, parameter <FingerPos> )
//     <FormatType> -  specifies the format of Finger Minutiae Record pointed by <*FMR>:
//                    see <FormatType> parameter description in GBFRSW_ISO_GetMaxFMRLength
//                      - if GBFRSW_ISO_FMR_FORMAT_ISO, parses FMR according <ISO/IEC 19794-2> format;
//					    - if GBFRSW_ISO_FMR_FORMAT_INCITS, parses FMR according to <INCITS 378> format;
//                    GBFRSW_ISO_FMR_FORMAT_MINEX is NOT convertible to Green Bit proprietary Template Code.
//					  Note: it is important that application is aware about used FMR format
//						    as far as Finger Minutiae Record itself does not contain this 
//						    information
//------------------------------------------------------------------------------




//**********************************************************************************************************
//**********************************************************************************************************
//					Template code convertion to Finger Minutia Card format <ISO/IEC 19794-2>
//                                          (not reversible)
//**********************************************************************************************************
//**********************************************************************************************************


//------------------------------------------------------------------------------
extern "C" GBFRSW_API int __stdcall
GBFRSW_ISO_GetMaxFMCLength(unsigned short FormatType,
						   unsigned int   ImageSizeX,
						   unsigned int   ImageSizeY,
                           unsigned int   MaxMinutiaeNum,
						   unsigned short *MaxFMCLength);
//------------------------------------------------------------------------------
// This function returns the maximum length of the Finger Minutie Card (FMC)
// format (<ISO/IEC 19794-2>) depending on fingerprint image dimensions and
// maximum number of minutiae.
// Normally, the bigger fingerprint image, the bigger FMC length.
// Use this function for memory allocation specifying the maximal fingerprint
// image dimensions expected in application.
//------------------------------------------------------------------------------
// Return value:
//   See section RETURN CODE DEFINITION
//------------------------------------------------------------------------------
// Parameters:
//	<FormatType>	- specified Format type for minutiae:
#define GBFRSW_ISO_MINUTIAE_NORMAL_FORMAT  0x0004  // normal card format, defined by <ISO/IEC 19794-2>
#define GBFRSW_ISO_MINUTIAE_COMPACT_FORMAT 0x0006  // compact card format, defined by <ISO/IEC 19794-2>
//	<ImageSizeX>	- input image width, in pixels; must be not higher than GBFRSW_GetVersionInfo.MaxImageSizeX
//	<ImageSizeY>	- input image height, in pixels; must be not higher than GBFRSW_GetVersionInfo.MaxImageSizeY
//  <MaxMinutiaeNum> - maximum number of minutiae to be stored inside the FMC;
//                     application can use this parameter to limit the size of the produced FMC;
//                     if set to 0, all minutiae will be stored
//  <*MaxFMCLength> - returns the biggest possible FMC length
//------------------------------------------------------------------------------



//------------------------------------------------------------------------------
extern "C" GBFRSW_API int __stdcall
GBFRSW_ISO_GBTemplateToFMC(unsigned char  *GBTemplate,
						   unsigned char  *FMC,
						   unsigned short *FMCLen,
						   unsigned short FormatType,
                           unsigned int   MaxMinutiaeNum);
//------------------------------------------------------------------------------
// The function converts Green Bit's proprietary Template Code
//  to Finger Minutie Card (FMC) format defined by <ISO/IEC 19794-2> for card usage.
// Because this format contains only minutiae, is not possible to perform reverse
// (ISO/IEC to Green Bit) code format conversion.
//------------------------------------------------------------------------------
// Return value:
//   See section RETURN CODE DEFINITION
//------------------------------------------------------------------------------
// Parameters:
//    <*GBTemplate> - pointer to Green Bit proprietary Template Code
//    <*FMC>   	   - pointer to a buffer allocated by application for
//					 FMC storage (call GBFRSW_ISO_GetMaxFMCLength to obtain the required size)
//    <*FMCLen>	   - returns the effective length of FMC, in bytes (that is normally
//					 less than max size)
//    <FormatType> - specified Format type for minutiae:
//					 see GBFRSW_ISO_GetMaxBITLength() description, parameter <FormatType>.
//    <MaxMinutiaeNum> - maximum number of minutiae to be stored inside the FMC;
//                       see <MaxMinutiaeNum> description in GBFRSW_ISO_GetMaxFMCLength()
//------------------------------------------------------------------------------


#endif

