﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

using BOZORTH_NET_WRAPPER;
using Bozorth_Net_Wrapper_Utilities;
using LFS_NET_WRAPPER;
using Lfs_Net_Wrapper_Utilities;

namespace Bozorth_Net_Wrapper_Utilities
{
    /************************************************************
     ************************************************************
     *
     * Direct dll wrapper
     * 
     ************************************************************
    ************************************************************/
    public class Bozorthdll_NetWrapper
    {
        [DllImport("Bozorth.dll", EntryPoint = "boz_direct_call")]
        public static extern int boz_direct_call(
            int maxminutiae,
            ref MINUTIAE probe, // MINUTIAE *
            ref MINUTIAE gallery // MINUTIAE *
        );
    }
}

namespace BOZORTH_NET_WRAPPER
{
	/// <summary>
	/// <para>In this class the matching score calculation is purchased.</para>
	/// </summary>
	/// <remarks>
	/// This class has no equivalents in the C version of the library, but here methods for matching score calculation are collected.
	/// </remarks>
	public class NW_BOZORTH_MatchScoreCalculator
	{
        private List<NW_LFS_MINUTIA > PrMin;
		private List<NW_LFS_MINUTIA > GallMin;

		/// <summary>
		/// <para>Constructor Overload</para>
		/// </summary>
		public NW_BOZORTH_MatchScoreCalculator(List<NW_LFS_MINUTIA > PrIn,
			List<NW_LFS_MINUTIA > GallIn)
        {
            this.GallMin = GallIn;
            this.PrMin = PrIn;
        }

		/// <summary>
		/// Minutiae list of the first fingerprint
		/// </summary>
		/// <remarks>
		/// This property represents the "probe" parameter of the "boz_direct_call" function of the C version of the library.
		/// </remarks>
		public List<NW_LFS_MINUTIA > Probe {
            set
            {
                this.PrMin = value;
            }
		}

		/// <summary>
		/// Minutiae list of the second fingerprint
		/// </summary>
		/// <remarks>
		/// This property represents the "gallery" parameter of the "boz_direct_call" function of the C version of the library.
		/// </remarks>
        public List<NW_LFS_MINUTIA> Gallery
        {
            set
            {
                this.GallMin = value;
            }
		}

		/// <summary>
		/// <para>Matching Score calculation</para>
		/// </summary>
		/// <exception cref="System.Exception">Thrown when an error occurs.
		/// <para>Read the <see cref ="System.Exception.Message"/>  </para>
		/// </exception>
		/// <returns>
		/// The calculated matching score.
		/// </returns>
		/// <paramref name = "MaxMinutiaeNumber"/>: 
		/// <para>Max number of minutiae from each fingerprint that must be used to calculate matching score.
		/// It represents the "maxminutiae" parameter of the "boz_direct_call" function of the C version of the library.</para>
        public Int32 NW_boz_direct_call(Int32 MaxMinutiaeNumber)
        {
            if (this.PrMin == null || this.GallMin == null)
		    {
			    Exception ex = new Exception(
                    "NW_BOZORTH_MatchScoreCalculator NW_boz_direct_call: probe and gallery cannot be null");
			    throw (ex);
		    }
            
            /////////////////////////////
            // build probe minutiae list
            /////////////////////////////
            MINUTIAE probeList = new MINUTIAE();
            probeList.list = Marshal.AllocHGlobal(this.PrMin.Count * IntPtr.Size);            
            if (probeList.list == IntPtr.Zero)
		    {
			    // free memory
			    // throw exception
			    Exception ex = new Exception("NW_BOZORTH_MatchScoreCalculator NW_boz_direct_call: not enough memory");
			    throw (ex);
		    }
            // build list
            for (int i=0;i<this.PrMin.Count;i++)
            {
                // create list item
                MINUTIA Item = new MINUTIA();
                Item.direction = (int)(this.PrMin[i].Direction_Degrees / 11.25);
			    Item.reliability = this.PrMin[i].reliability;
			    Item.type = this.PrMin[i].type;
			    Item.x = this.PrMin[i].x;
			    Item.y = this.PrMin[i].y;
                // get pointer to item
                IntPtr pItem = Marshal.AllocHGlobal(Marshal.SizeOf(Item));
                Marshal.StructureToPtr(Item, pItem, false);
                if (pItem == IntPtr.Zero)
			    {
				    // free memory
				    for (int j=0;j<i;j++)
				    {
					    IntPtr Dummy = Marshal.ReadIntPtr(probeList.list,j * IntPtr.Size);
                        Marshal.FreeHGlobal(Dummy);
				    }
				    Marshal.FreeHGlobal(probeList.list);
				    // throw exception
				    Exception ex = new Exception("NW_BOZORTH_MatchScoreCalculator NW_boz_direct_call: not enough memory");
				    throw (ex);
			    }
                // write ptr in list
                Marshal.WriteIntPtr(probeList.list,i * IntPtr.Size,pItem);
		    }
		    probeList.num = this.PrMin.Count;

		    /////////////////////////////
            // build gallery minutiae list
		    /////////////////////////////
            MINUTIAE galleryList = new MINUTIAE();
            galleryList.list = Marshal.AllocHGlobal(this.GallMin.Count * IntPtr.Size);            
            if (galleryList.list == IntPtr.Zero)
		    {
			    // free memory
			    // throw exception
			    Exception ex = new Exception("NW_BOZORTH_MatchScoreCalculator NW_boz_direct_call: not enough memory");
			    throw (ex);
		    }
            // build list
            for (int i=0;i<this.GallMin.Count;i++)
            {
                // create list item
                MINUTIA Item = new MINUTIA();
                Item.direction = (int)(this.GallMin[i].Direction_Degrees / 11.25);
			    Item.reliability = this.GallMin[i].reliability;
			    Item.type = this.GallMin[i].type;
			    Item.x = this.GallMin[i].x;
			    Item.y = this.GallMin[i].y;
                // get pointer to item
                IntPtr pItem = Marshal.AllocHGlobal(Marshal.SizeOf(Item));
                if (pItem == IntPtr.Zero)
			    {
				    // free memory
				    for (int j=0;j<i;j++)
				    {
					    IntPtr Dummy = Marshal.ReadIntPtr(galleryList.list,j * IntPtr.Size);
                        Marshal.FreeHGlobal(Dummy);
				    }
				    Marshal.FreeHGlobal(galleryList.list);
				    // throw exception
				    Exception ex = new Exception("NW_BOZORTH_MatchScoreCalculator NW_boz_direct_call: not enough memory");
				    throw (ex);
                }
                Marshal.StructureToPtr(Item, pItem, false);
                // write ptr in list
                Marshal.WriteIntPtr(galleryList.list,i * IntPtr.Size,pItem);
		    }
		    galleryList.num = this.GallMin.Count;

            //////////////////////////////////////
            // call low level function
            //////////////////////////////////////
            int RetVal = Bozorthdll_NetWrapper.boz_direct_call(MaxMinutiaeNumber, ref probeList, ref galleryList);

            //////////////////////
            // Free memory
            //////////////////////
            for (int i = 0; i < galleryList.num; i++)
            {
                IntPtr Dummy = Marshal.ReadIntPtr(galleryList.list, i * IntPtr.Size);
                Marshal.FreeHGlobal(Dummy);
            }
            Marshal.FreeHGlobal(galleryList.list);
            for (int i = 0; i < probeList.num; i++)
            {
                IntPtr Dummy = Marshal.ReadIntPtr(probeList.list, i * IntPtr.Size);
                Marshal.FreeHGlobal(Dummy);
            }
            Marshal.FreeHGlobal(probeList.list);

		    return RetVal;
        }
	};
}
