﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

using GBNFIQ_NET_WRAPPER;
using Gbnfiq_Net_Wrapper_Utilities;
using LFS_NET_WRAPPER;
using Lfs_Net_Wrapper_Utilities;

namespace Gbnfiq_Net_Wrapper_Utilities
{
    /************************************************************
     ************************************************************
     *
     * Constants definition
     * 
     ************************************************************
    ************************************************************/
    public class GBNFIQ_Constants
    {
        public const int GBNFIQ_MAX_MINUTIAE = 1000;
        public const int GBNFIQ_NO_ERROR = 0;
        public const int GBNFIQ_EMPTY_IMG = 1;
        public const int GBNFIQ_TOO_FEW_MINUTIAE = 2;
    }
    /************************************************************
     ************************************************************
     *
     * Structures definition: PACK 1 for compatibility with
     * C structures
     * 
     ************************************************************
    ************************************************************/
    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
    public struct GBNFIQ_MINUTIA_DESCRIPTOR
    {
        public int x;
        public int y;
        public int direction;
        public int type;
        public double reliability;
    }

    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
    public struct GBNFIQ_MINUTIAE_INFO
    {
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = GBNFIQ_Constants.GBNFIQ_MAX_MINUTIAE)]
        public GBNFIQ_MINUTIA_DESCRIPTOR[] MinutiaeList;
        public int MinutiaeNumber;
    }
    /************************************************************
     ************************************************************
     *
     * Direct dll wrapper
     * 
     ************************************************************
    ************************************************************/
    public class Gbnfiqdll_NetWrapper
    {
        [DllImport("GBNFIQ.dll", EntryPoint = "GBNFIQ_GetImageQuality")]
        public static extern int GBNFIQ_GetImageQuality(
            IntPtr  ImagePtr, // BYTE *
            int 	ImageWidth,
            int		ImageHeight,
            out int ImageQuality,
            ref GBNFIQ_MINUTIAE_INFO MinutiaeInfo
        );

        [DllImport("GBNFIQ.dll", EntryPoint = "GBNFIQ_GetVersion")]
        public static extern void GBNFIQ_GetVersion(
            out Byte VersionField1,
            out Byte VersionField2,
            out Byte VersionField3,
            out Byte VersionField4);
    }
}

namespace GBNFIQ_NET_WRAPPER
{
	/// <summary>
	/// <para>Input data for the "NW_GBNFIQ_GetImageQuality" in the GBNFIQ_NET_WRAPPER.NW_GBNFIQ_ImageQuality_Info" class</para>
	/// </summary>
	public class NW_GBNFIQ_ImageQuality_Info_InputData
	{
		/// <summary>
		/// <para>Input image frame</para>
		/// </summary>
		/// <remarks>
		/// Represents the "ImagePtr" parameter of the "GBNFIQ_GetImageQuality" function of the C version of the library.
		/// </remarks>
		public Byte [] Frame;
		/// <summary>
		/// <para>Input image width</para>
		/// </summary>
		/// <remarks>
		/// Represents the "ImageWidth" parameter of the "GBNFIQ_GetImageQuality" function of the C version of the library.
		/// </remarks>
		public Int32		Width;
		/// <summary>
		/// <para>Input image height</para>
		/// </summary>
		/// <remarks>
		/// Represents the "ImageHeight" parameter of the "GBNFIQ_GetImageQuality" function of the C version of the library.
		/// </remarks>
		public Int32		Height;
	};
											   
	/// <summary>
	/// <para>Output data for the "NW_GBNFIQ_GetImageQuality" in the GBNFIQ_NET_WRAPPER.NW_GBNFIQ_ImageQuality_Info" class</para>
	/// </summary>
	public class NW_GBNFIQ_ImageQuality_Info_OutputData
	{
		/// <summary>
		/// <para>Output image quality in range [1;5], where 1 is the best</para>
		/// </summary>
		/// <remarks>
		/// Represents the "ImageQuality" parameter of the "GBNFIQ_GetImageQuality" function of the C version of the library.
		/// </remarks>
		public Int32 NFIQ_Quality;
		/// <summary>
		/// <para>Output image minutiae</para>
		/// </summary>
		/// <remarks>
		/// Represents the "MinutiaeInfo" parameter of the "GBNFIQ_GetImageQuality" function of the C version of the library.
		/// </remarks>
		public List<NW_LFS_MINUTIA > MinInfo;
	};

	/// <summary>
	/// <para>In this class methods for quality calculation and getting low level version info are collected.</para>
	/// </summary>
	public class NW_GBNFIQ_ImageQuality_Info
	{
        private NW_GBNFIQ_ImageQuality_Info_InputData Inp;

		/// <summary>
		/// <para>Constructot Overload</para>
		/// </summary>
		/// <exception cref="System.Exception">Thrown when <paramref name="InputData"/> is null.</exception>
		/// <param name = "InputData">
		/// Must be not null
		/// </param>
        public NW_GBNFIQ_ImageQuality_Info(NW_GBNFIQ_ImageQuality_Info_InputData InputData)
        {
		    if (InputData == null)
		    {
			    Exception ex = new Exception("NW_GBNFIQ_ImageQuality_Info constructor: parameter InputData cannot be null");
			    throw (ex);
		    }
		    this.Inp = InputData;
        }
		/// <summary>
		/// <para>Set Input data</para>
		/// </summary>
		/// <exception cref="System.Exception">Thrown when <paramref name="InputData"/> is null.</exception>
		/// <param name = "InputData">
		/// Must be not null
		/// </param>
        public void SetInput(NW_GBNFIQ_ImageQuality_Info_InputData InputData)
        {
            if (InputData == null)
            {
                Exception ex = new Exception("NW_GBNFIQ_ImageQuality_Info constructor: parameter InputData cannot be null");
                throw (ex);
            }
            this.Inp = InputData;
        }
		/// <summary>
		/// <para>Gets low level dll version in format x.x.x.x</para>
		/// </summary>
		/// <remarks>
		/// This method represents the "GBNFIQ_GetVersion" function of the C version of the library.
		/// </remarks>
		/// A <see cref = "String"/> containing the low level dll version in format x.x.x.x.
		/// </returns>
        public String NW_GBNFIQ_GetVersion()
        {
            Byte VersionField1, VersionField2, VersionField3, VersionField4;
            Gbnfiqdll_NetWrapper.GBNFIQ_GetVersion(
                out VersionField1,
                out VersionField2,
                out VersionField3,
                out VersionField4);

            String ValToRet = "" + VersionField1 + "." + VersionField2 + "." + VersionField3 + "." + VersionField4;
            return ValToRet;
        }

		/// <summary>
		/// <para>Quality Calculation</para>
		/// </summary>
		/// <exception cref="System.Exception">Thrown when an error occurs.
		/// <para>Read the <see cref ="System.Exception.Message"/>  </para>
		/// </exception>
		/// <returns>
		/// A <see cref = "NW_GBNFIQ_ImageQuality_Info_OutputData"/> structure.
		/// </returns>
		/// <remarks>
		/// This method represents the "GBNFIQ_GetImageQuality" function of the C version of the library.
		/// </remarks>
        public NW_GBNFIQ_ImageQuality_Info_OutputData NW_GBNFIQ_GetImageQuality()
        {
            if (this.Inp == null || this.Inp.Frame == null)
		    {
			    Exception ex = new Exception(
                    "NW_GBNFIQ_ImageQuality_Info NW_GBNFIQ_GetImageQuality(): input data is null, try setting it again");
			    throw (ex);
		    }

		    IntPtr ImgPtr = Marshal.AllocHGlobal(this.Inp.Frame.Length);
		    if (ImgPtr == IntPtr.Zero)
		    {
			    Exception ex = new Exception("NW_GBNFIQ_ImageQuality_Info NW_GBNFIQ_GetImageQuality(): not enough memory");
			    throw (ex);
		    }
		    Marshal.Copy(this.Inp.Frame,0,ImgPtr,this.Inp.Frame.Length);

		    int ImageQuality;
		    GBNFIQ_MINUTIAE_INFO	MinutiaeInfo = new GBNFIQ_MINUTIAE_INFO();
            MinutiaeInfo.MinutiaeList = new GBNFIQ_MINUTIA_DESCRIPTOR[GBNFIQ_Constants.GBNFIQ_MAX_MINUTIAE];
            for (int i = 0; i < GBNFIQ_Constants.GBNFIQ_MAX_MINUTIAE; i++)
            {
                MinutiaeInfo.MinutiaeList[i] = new GBNFIQ_MINUTIA_DESCRIPTOR();
            }

		    int RetVal = Gbnfiqdll_NetWrapper.GBNFIQ_GetImageQuality(
                ImgPtr,
			    this.Inp.Width, this.Inp.Height,
			    out ImageQuality,
                ref MinutiaeInfo);

            if (RetVal != GBNFIQ_Constants.GBNFIQ_NO_ERROR &&
                // ver 3.2.0.0
                RetVal != GBNFIQ_Constants.GBNFIQ_TOO_FEW_MINUTIAE)
		    {
			    Exception ex = new Exception(
                    "NW_GBNFIQ_ImageQuality_Info NW_GBNFIQ_GetImageQuality(): low level function error " + RetVal);
			    throw (ex);
		    }

		    // copy info
		    NW_GBNFIQ_ImageQuality_Info_OutputData ObjToReturn = new NW_GBNFIQ_ImageQuality_Info_OutputData();
		    ObjToReturn.NFIQ_Quality = ImageQuality;
		    ObjToReturn.MinInfo = new List<LFS_NET_WRAPPER.NW_LFS_MINUTIA>();
		    for (int i=0;i<MinutiaeInfo.MinutiaeNumber;i++)
		    {
			    LFS_NET_WRAPPER.NW_LFS_MINUTIA Item = new LFS_NET_WRAPPER.NW_LFS_MINUTIA(
				    MinutiaeInfo.MinutiaeList[i].x,
				    MinutiaeInfo.MinutiaeList[i].y,
				    MinutiaeInfo.MinutiaeList[i].direction,
				    MinutiaeInfo.MinutiaeList[i].reliability,
				    MinutiaeInfo.MinutiaeList[i].type);
			    if (Item == null)
			    {
				    Exception ex = new Exception("NW_GBNFIQ_ImageQuality_Info NW_GBNFIQ_GetImageQuality(): not enough memory");
				    throw (ex);
			    }
			    ObjToReturn.MinInfo.Add(Item);
		    }

		    return ObjToReturn;
        }
	};
}
