﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

using WSQPACK_NET_WRAPPER;
using Wsq_Net_Wrapper_Utilities;

namespace Wsq_Net_Wrapper_Utilities
{
    /************************************************************
     ************************************************************
     *
     * Constants definition
     * 
     ************************************************************
    ************************************************************/
    public class Wsq_Constants
    {
        public const int WSQPACK_OK = 0;
    }
    /************************************************************
     ************************************************************
     *
     * Direct dll wrapper
     * 
     ************************************************************
    ************************************************************/
    public class Wsqdll_NetWrapper
    {
        [DllImport("WSQPack.dll", EntryPoint = "WSQPACK_Compress")]
        public static extern int WSQPACK_Compress(
			IntPtr dest, // unsigned char **
			out int destLen, 
			float r_bitrate,
            IntPtr source, // unsigned char * 
			int width, 
			int height, 
			int depth, 
			int ppi, 
			String comment_text);

        [DllImport("WSQPack.dll", EntryPoint = "WSQPACK_Uncompress")]
        public static extern int WSQPACK_Uncompress(
			IntPtr dest, // unsigned char **
			out int width,
			out int height,
			out int depth,
			out int ppi,
			out int lossyflag,
            IntPtr source, // unsigned char *
			int sourceLen);

        [DllImport("WSQPack.dll", EntryPoint = "WSQPACK_Free")]
        public static extern void WSQPACK_Free(IntPtr PtrToFree);

    }
}

namespace WSQPACK_NET_WRAPPER
{

    /**********************************************************************/
    /**********************************************************************/
    //**********			SECTION 1: CONSTANTS		******************//
    /**********************************************************************/
    /**********************************************************************/

    /// <summary> Error codes </summary>
    public class NW_WSQPACK_ERRORS
    {
        public static Int32 NW_WSQPACK_OK = Wsq_Constants.WSQPACK_OK;
        /// <summary> This field has no equivalents in the underlying C version of the library.
        /// This error is returned when an exception is encountered while processing. 
        /// The exception message is stored into the 
        /// <see cref = "NW_WSQPACK_ERROR_EXCEPTION_STRING"/> string.</summary>
        public static Int32 NW_WSQPACK_ERROR_EXCEPTION = 255;
        /// <summary> When an exception is encountered while processing, its message is stored here.</summary>
        public static String NW_WSQPACK_ERROR_EXCEPTION_STRING;
    };

    /// <summary> Bit per pixel constants </summary>
    public class NW_WSQPACK_PIXEL_DEPTH_DEFINITIONS
    {
        public static Int32 NW_WSQPACK_8_BITS_PER_PIXEL = 8;
        public static Int32 NW_WSQPACK_24_BITS_PER_PIXEL = 24;
    };

    /// <summary> Compression rate constants </summary>
    public class NW_WSQPACK_RATE_DEFINITIONS
    {
        /// <summary> Compression rate = 1 </summary>
        public static Single NW_WSQPACK_RATE_LOSSLESS = 1; // compression rate = 1
        /// <summary> Compression rate = infinite </summary>
        public static Single NW_WSQPACK_ZERO_RATE = 0; // compression rate = infinite
        /// <summary> Compression rate = 15:1 </summary>
		public static Single NW_WSQPACK_RECOMMENDED = (Single)0.75; // compression rate 15:1
		// VER 1.1.0.1
		public static Single NW_WSQPACK_ALTERNATIVE = (Single)2.25; // compression rate 5:1
    };

    /**********************************************************************/
    /**********************************************************************/
    //**********			SECTION 2: METHODS		**********************//
    /**********************************************************************/
    /**********************************************************************/

    /// <summary>
    /// This class allows user to get images encoded by using the WSQ algorithm
    /// with the requested compression rate.
    /// </summary>
    /// <remarks>
    /// <para>the "Encoded" property returns the image encoded with the currently set properties.
    /// Therefore it's recommended to set them just before encoding.</para>
    /// <para>This class is the equivalent of the
    /// "WSQPACK_Compress" function  in the C version of the library. Input parameters for encoding are passed
    /// as <see cref = "NW_WSQPACK_Compress"/> properties, while the output frame is given by getting the
    /// <see cref = "Encoded"/> property.</para>
    /// </remarks>
    public class NW_WSQPACK_Compress
    {
        /// <summary>
        /// constructor overload
        /// </summary>
        /// <param name = "RawImage">
        /// Image to be encoded
        /// </param>
        /// <param name = "w">
        /// Image width in pixel
        /// </param>
        /// <param name = "h">
        /// Image height in pixel
        /// </param>
        /// <remarks>
        /// the encoding settings specified by the other properties in this class
        /// are set to their default values
        /// </remarks>
        public NW_WSQPACK_Compress(Byte[] RawImage, Int32 w, Int32 h)
        {
            this.Original = RawImage;
            this.ImageW = w;
            this.ImageH = h;
            this.Rate = NW_WSQPACK_RATE_DEFINITIONS.NW_WSQPACK_RECOMMENDED;
            this.BPP = NW_WSQPACK_PIXEL_DEPTH_DEFINITIONS.NW_WSQPACK_8_BITS_PER_PIXEL;
            this.DPI = 500;
            this.WSQComment = null;
        }

        /// <summary>
        /// This property returns the encoded array.
        /// </summary>
        /// <remarks>
        /// If a null value is returned see the <see cref = "NW_WSQPACK_ERRORS.NW_WSQPACK_ERROR_EXCEPTION_STRING"/> error message.
        /// <para>This property is the equivalent of the "dest" output parameter of the "WSQPACK_Compress" in the C version of
        /// the library.</para>
        /// </remarks>
        public Byte[] Encoded
        {
            get
            {
                if (this.Original == null) 
                {
                    throw new Exception ("Cannot Encode: Class not initialized");
                }
                
                // encode
                IntPtr RawImage = Marshal.AllocHGlobal(this.Original.Length);
                if (RawImage == IntPtr.Zero)
                {
                    throw new Exception ("Cannot Encode: memory error for RawImage");
                }
                
                Marshal.Copy(this.Original,0,RawImage,this.Original.Length);
                IntPtr pImgBuffer = Marshal.AllocHGlobal(IntPtr.Size);
                if (RawImage == IntPtr.Zero)
                {
                    Marshal.FreeHGlobal(RawImage);
                    throw new Exception ("Cannot Encode: memory error for pImgBuffer");
                }
                int BufLen;
                int RetVal;
                
                RetVal =  Wsqdll_NetWrapper.WSQPACK_Compress(
                    pImgBuffer, 
                    out BufLen, 
                    this.CompressionRate,
                    RawImage,
                    this.ImageW,this.ImageH, 
                    this.BitPerPixel, 
                    this.ImageResolution, 
                    this.WSQComment
                );
                
                // free memory for RawImage
                Marshal.FreeHGlobal(RawImage);
                
                if (RetVal != Wsq_Constants.WSQPACK_OK)
                {
                    Marshal.FreeHGlobal(pImgBuffer);
                    throw new Exception ("Cannot Encode: low level WSQPACK_Compress returned: " + RetVal);
                }

                IntPtr ImgBuffer = Marshal.ReadIntPtr(pImgBuffer);
                
                // copy image to output
                Byte []  Output = new Byte [BufLen];
                Marshal.Copy((IntPtr)ImgBuffer,Output,0,BufLen);
                
                // free memory
                Wsqdll_NetWrapper.WSQPACK_Free(ImgBuffer);
                
                // return normally
                return Output;
            }
        }

        /// <summary>
        /// This property gets or sets the compression rate for the WSQ algorithm.
        /// Default value is
        /// <see cref = "NW_WSQPACK_RATE_DEFINITIONS.NW_WSQPACK_RECOMMENDED"/>
        /// </summary>
        /// <remarks>
        /// <para> when setting, value must be 
        /// bigger or equal than <see cref = "NW_WSQPACK_RATE_DEFINITIONS.NW_WSQPACK_ZERO_RATE"/>
        /// and smaller or equal than <see cref = "NW_WSQPACK_RATE_DEFINITIONS.NW_WSQPACK_RATE_LOSSLESS"/>
        /// otherwise operation will take no effect
        /// </para>
        /// <para>This property is the equivalent of the "r_bitrate" parameter of the "WSQPACK_Compress" in the C version of
        /// the library.</para>
        /// </remarks>
        public Single CompressionRate
        {
            get
            {
                return this.Rate;
            }
            set
            {
                if (value <= WSQPACK_NET_WRAPPER.NW_WSQPACK_RATE_DEFINITIONS.NW_WSQPACK_RATE_LOSSLESS &&
                    value >= WSQPACK_NET_WRAPPER.NW_WSQPACK_RATE_DEFINITIONS.NW_WSQPACK_ZERO_RATE)
                {
                    this.Rate = value;
                }
            }
        }

        /// <summary>
        /// <para> 
        /// This property gets or sets the number of bits per pixel used in the raw image.
        /// Can be one of the values defined into the
        /// <see cref = "NW_WSQPACK_PIXEL_DEPTH_DEFINITIONS"/> class.
        /// </para>
        /// <para> 
        /// Default value is
        /// <see cref = "NW_WSQPACK_PIXEL_DEPTH_DEFINITIONS.NW_WSQPACK_8_BITS_PER_PIXEL"/>
        /// </para>
        /// </summary>
        /// <remarks>
        /// <para>When setting, value must be within the values defined into the
        /// <see cref = "NW_WSQPACK_PIXEL_DEPTH_DEFINITIONS"/> class,
        /// otherwise operation will take no effect.</para>
        /// <para>This property is the equivalent of the "depth" parameter of the "WSQPACK_Compress" in the C version of
        /// the library.</para>
        /// </remarks>
        public Int32 BitPerPixel
        {
            get
            {
                return this.BPP;
            }
            set
            {
                if (value == WSQPACK_NET_WRAPPER.NW_WSQPACK_PIXEL_DEPTH_DEFINITIONS.NW_WSQPACK_8_BITS_PER_PIXEL ||
                    value == WSQPACK_NET_WRAPPER.NW_WSQPACK_PIXEL_DEPTH_DEFINITIONS.NW_WSQPACK_24_BITS_PER_PIXEL)
                {
                    this.BPP = value;
                }
            }
        }

        /// <summary>
        /// <para> 
        /// This property gets or sets resolution of the raw image in Dots per Inch units.
        /// </para>
        /// <para> 
        /// Default value is 500 dpi.
        /// </para>
        /// <para>This property is the equivalent of the "ppi" parameter of the "WSQPACK_Compress" in the C version of
        /// the library.</para>
        /// </summary>
        public Int32 ImageResolution
        {
            get
            {
                return this.DPI;
            }
            set
            {
                this.DPI = value;
            }
        }

        /// <summary>
        /// <para> 
        /// This property gets or sets an optional comment.
        /// </para>
        /// <para> 
        /// Default value is null.
        /// </para>
        /// <para>This property is the equivalent of the "comment_text" parameter of the "WSQPACK_Compress" in the C version of
        /// the library.</para>
        /// </summary>
        public String WSQComment
        {
            get
            {
                return this.Comm;
            }
            set
            {
                this.Comm = value;
            }
        }

        protected Byte[] Original;
        protected Int32 ImageW, ImageH;
        protected Single Rate;
        protected Int32 DPI;
        protected Int32 BPP;
        protected String Comm;
    };

	/// <summary>
	/// This structure is used to collect information generated by the NW_WSQPACK_Uncompress class
	/// </summary>
	/// <remarks>
	/// This class represents the output of the "WSQPACK_Uncompress" function of the C version of the library
	/// </remarks>
	public class NW_WSQPACK_DecoderInfo
	{
		/// <summary>
		/// Represents the "dest" output parameter of the "WSQPACK_Uncompress" function.
		/// </summary>
		public Byte [] RawImage;
		/// <summary>
		/// Represents the "width" output parameter of the "WSQPACK_Uncompress" function.
		/// </summary>
		public Int32		ImageWidth;
		/// <summary>
		/// Represents the "height" output parameter of the "WSQPACK_Uncompress" function.
		/// </summary>
		public Int32		ImageHeight;
		/// <summary>
		/// Represents the "depth" output parameter of the "WSQPACK_Uncompress" function.
		/// </summary>
		public Int32		BitsPerPixel;
		/// <summary>
		/// Represents the "ppi" output parameter of the "WSQPACK_Uncompress" function.
		/// </summary>
		public Int32		ImageResolution;
	};

	/// <summary>
	/// This class allows user to get the original images previously encoded by using the WSQ algorithm.
	/// </summary>
	/// <remarks>
	/// <para>This class is the equivalent of the
	/// "WSQPACK_Uncompress" function  in the C version of the library. The output frame is given by getting the
	/// <see cref = "ImageDecoded"/> property.</para>
	/// </remarks>
	public class NW_WSQPACK_Uncompress
	{	
		/// <summary>
		/// constructor overload
		/// </summary>
		/// <param name = "EncodedImage">
		/// Image to be decoded
		/// </param>
		public NW_WSQPACK_Uncompress(Byte []  EncodedImage)
        {
            this.Original = EncodedImage;
        }

		/// <summary>
		/// This property returns the decoded array
		/// </summary>
		/// <remarks>
		/// <para> If a null value is returned see the
		/// <see cref = "NW_WSQPACK_ERRORS.NW_WSQPACK_ERROR_EXCEPTION_STRING"/>
		/// error message.</para>
		/// <para> This property represents the output of the "WSQPACK_Uncompress" function of the C version of the library.</para>
		/// </remarks>
		public NW_WSQPACK_DecoderInfo  ImageDecoded {
            get
            {
                if (this.Original == null) 
                {
                    throw new Exception ("Cannot Decode: Class not initialized");
                }
                
                NW_WSQPACK_DecoderInfo  Info = new NW_WSQPACK_DecoderInfo();
                
                // decode
                IntPtr ImgBuffer = Marshal.AllocHGlobal(this.Original.Length);
                if (ImgBuffer == IntPtr.Zero)
                {
                    throw new Exception ("Cannot Decode: memory error for ImgBuffer");
                }
                
                Marshal.Copy(this.Original,0,ImgBuffer,this.Original.Length);
                IntPtr pRawImage = Marshal.AllocHGlobal(IntPtr.Size);
                if (pRawImage == IntPtr.Zero)
                {
                    Marshal.FreeHGlobal(ImgBuffer);
                    throw new Exception ("Cannot Decode: memory error for pRawImage");
                }
                Int32 ImageWidth,ImageHeight;
                Int32 BitsPerPixel;
                Int32 ImageResolution;
                Int32 lossyflag;
                int RetVal;
                
                RetVal = Wsqdll_NetWrapper.WSQPACK_Uncompress(
                    pRawImage,
                    out ImageWidth, out ImageHeight,
                    out BitsPerPixel,
                    out ImageResolution,
                    out lossyflag,
                    ImgBuffer,
                    this.Original.Length
                );
                
                // free memory for ImgBuffer
                Marshal.FreeHGlobal(ImgBuffer);
                
                if (RetVal != Wsq_Constants.WSQPACK_OK)
                {
                    Marshal.FreeHGlobal(pRawImage);
                    throw new Exception ("Cannot Decode: low level WSQPACK_Uncompress returned: " + RetVal);
                }

                IntPtr RawImage = Marshal.ReadIntPtr(pRawImage);
                
                // copy image to output
                Info.RawImage = new Byte[ImageWidth * ImageHeight];
                Marshal.Copy((IntPtr)RawImage, Info.RawImage, 0, ImageWidth * ImageHeight);

                Info.ImageWidth = ImageWidth;
                Info.ImageHeight = ImageHeight;
                Info.BitsPerPixel = BitsPerPixel;
                Info.ImageResolution = ImageResolution;
                
                // free memory
                Wsqdll_NetWrapper.WSQPACK_Free(RawImage);
                
                // return normally
                return Info;
            }
		}

		protected Byte []	Original;
	};
}
